<?php
if ( ! class_exists( 'SLPEXP_Admin_GeneralSettings' ) ) {

	/**
	 * Class SLPEXP_Admin_GeneralSettings
	 *
	 * The things that modify the Admin / General tab.
	 *
	 * Text Domain: slp-experience
	 *
	 * @property 		SLP_Experience 	$addon
	 * @property-read   array        $group_params
	 * @property-read   SLP_Settings $Settings
	 */
	class SLPEXP_Admin_GeneralSettings extends SLPlus_BaseClass_Object {
		public  $addon;
		private $group_params;
		private $Settings;

		/**
		 * Init.
		 */
		public function initialize() {
			$this->group_params = array( 'plugin' => $this->addon, 'section_slug' => null, 'group_slug' => null );
			add_action( 'slp_build_general_settings_panels'  , array( $this , 'modify_tab' ), 90 );
		}

		/**
		 * Modify entries on the tab.
		 *
		 * @param SLPlus_Settings $Settings
		 */
		public function modify_tab( $Settings ) {
			$this->Settings = $Settings;
			$this->modify_user_interface();
		}

		/**
		 * User Interface
		 */
		private function modify_user_interface( ) {
			$this->group_params['section_slug'] = 'user_interface';
			$this->modify_user_interface_settings();
		}

		/**
		 * User Interface / Settings
		 */
		private function modify_user_interface_settings() {
			$this->group_params['group_slug'] = 'settings';
			$this->Settings->add_ItemToGroup( array(
				'group_params' => $this->group_params,
				'option'       => 'url_allow_address',
				'type'         => 'checkbox',
				'label'        => __('Allow Address In URL','slp-experience'),
				'description'  => __('If checked an address can be pre-loaded via a URL string ?address=my+town. ', 'slp-experience') .
				                  __('This will disable the location sensor whenever the address is used in the URL.', 'slp-experience'),
			) );
		}
	}
}