<?php
if (! class_exists('SLPExperience_Admin')) {

    require_once( SLPLUS_PLUGINDIR . 'include/base_class.admin.php');

    /**
     * Holds the admin-only code.
     *
     * This allows the main plugin to only include this file in admin mode
     * via the admin_menu call.   Reduces the front-end footprint.
     *
     * @package StoreLocatorPlus\Experience\Admin
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2015-2016 Charleston Software Associates, LLC
     *
     * Text Domain: slp-experience
     *
     * @property        SLP_Experience                  $addon
     * @proeprty        string[]                        $admin_checkboxes   List of option keys that are checkboxes.
     * @property     	string			                $current_section;
     * @property	    string			                $current_group;
     * @property-read   SLPEXP_Admin_Locations          $locations
     * @property-read	SLP_Settings	                $settings
     * @property-read   SLPER_Admin_UXSettings          $experience
     * @property-read   SLPER_Admin_GeneralSettings     $general
     */
    class SLPExperience_Admin extends SLP_BaseClass_Admin {
        public $addon;
        private $experience;
        private $general;
        private $locations;
        private $settings;
        public  $settings_interface;
        public  $settings_pages = array(
            'slp_experience' => array(
                    'add_tel_to_phone'                  ,       // serialized
                    'disable_initial_directory'         ,
                    'hide_address_entry'                ,
                    'hide_bubble'                       ,       // serialized
                    'hide_distance'                     ,       // serialized
                    'hide_map'                          ,       // serialized
                    'hide_radius_selector'              ,       // serialized
                    'hide_search_form'                  ,
                    'map_options_scaleControl'          ,       // serialized
                    'map_options_scrollwheel'           ,       // serialized
                    'map_options_mapTypeControl'        ,       // serialized
                    'no_autozoom'                       ,       // serialized
                    'no_homeicon_at_start'              ,       // serialized
                    'search_by_name'                    ,       // serialized
                    'show_country'                      ,       // serialized
                    'show_hours'                        ,       // serialized
                    'show_maptoggle'                    ,       // serialized
            ),

            'slp_general'   => array(
                    'url_allow_address'                 ,       // serialized
            )

        ) ;

        /**
         * Add our SLP hooks and Filters for Admin Mode
         */
        public function add_hooks_and_filters() {
	        parent::add_hooks_and_filters();

	        // Load objects based on which admin page we are on.
	        //
	        if ( isset( $_REQUEST['page'] ) ) {
		        switch ( $_REQUEST['page'] ) {
			        case 'slp_experience':
				        $this->create_object_experience();
				        break;
			        case 'slp_general':
				        $this->create_object_general();
				        break;
		        }
	        }

            // Admin : Locations
            //
            add_action('slp_manage_locations_action'        , array( $this , 'feature_locations_action'             )           );
            add_filter('slp_locations_manage_bulkactions'   ,array($this,'filter_LocationsBulkAction'                   )           );
            add_filter('slp_locations_manage_cssclass'      ,array($this,'filter_HighlightFeatured'                     )           );

            // Pro Pack Imports
            //
            add_filter('slp_csv_locationdata_added'         ,array($this,'filter_CSVImportLocationFeatures'             ), 90 , 2   );


            // Data Saving
            //
            add_filter( 'slp_save_map_settings_inputs'          , array( $this , 'save_inputs'                  )       );
        }

        /**
         * Create and attach the user experience object.
         */
        private function create_object_experience() {
            if ( ! isset( $this->experience ) ) {
                require_once('class.admin.experience.php');
                $this->experience = new SLPEXP_Admin_ExperienceSettings(array( 'addon' => $this->addon ) );
            }
        }

        /**
         * Create and attach the general object.
         */
        private function create_object_general() {
            if ( ! isset( $this->general ) ) {
                require_once('class.admin.general.php');
                $this->general = new SLPEXP_Admin_GeneralSettings(array( 'addon' => $this->addon ) );
            }
        }

        /**
         * Create and attach the locations object.
         */
        private function create_object_locations() {
            if ( ! isset( $this->locations ) ) {
                require_once('class.admin.locations.php');
                $this->locations = new SLPEXP_Admin_Locations(array( 'addon' => $this->addon ) );
            }
        }

        /**
         * Deactivate any plugins that this add-on replaces.
         */
        private function deactivate_replaced_addons() {
            include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
            $this->slplus->createobject_AddOnManager();
            $replaced_addons = array( 'slp-enhanced-map' , 'slp-enhanced-results' , 'slp-enhanced-search' , 'slp-widgets' );
            foreach ( $replaced_addons as $addon_slug ) {
                if ( $this->slplus->is_AddonActive( $addon_slug ) ) {
                    deactivate_plugins( $this->slplus->add_ons->instances[$addon_slug]->file );
                    $this->slplus->helper->add_wp_admin_notification(
                        sprintf(
                            __( 'The %s add-on deactivated the conflicting %s add-on. '  , 'slp-experience' ) ,
                            $this->addon->name ,
                            $this->slplus->add_ons->instances[$addon_slug]->name
                        )
                    );
                }
            }
        }

        /**
         * Additional location processing on manage locations admin page.
         *                                            
         * @param   SLPlus_Admin_Locations_Actions  $processor
         */
        function feature_locations_action( $processor ) {
            if ( ! $processor->set_locations() ) { return; }

            if ( $_REQUEST['act'] === 'feature_location' ) {
                $this->create_object_locations();
                $this->locations->feature_locations( 'add' , $processor );
            } elseif ( $_REQUEST['act'] === 'remove_feature_location' ) {
                $this->create_object_locations();
                $this->locations->feature_locations( 'remove' , $processor );
            }
        }

        /**
         * Process incoming CSV import data and add our extended field attributes.
         *
         * note: CSV import field names always get the sl_ prefixed.
         *
         * @param mixed[] $locationData
         * @param string $result
         * @return the original data, unchanged
         */
        function filter_CSVImportLocationFeatures($locationData, $result) {
            $newData = array();
            $extended_data_fields = array('featured', 'rank');
            foreach ($extended_data_fields as $field) {
                if ( isset($locationData['sl_'.$field]) ) { $newData[$field] = $locationData['sl_'.$field]; }
            }
            if ( count( $newData ) > 0 ) {
                $this->slplus->database->extension->update_data(
                    $this->slplus->currentLocation->id,
                    $newData
                );
            }
            return array($locationData,$result);
        }

        /**
         * Highlight the featured elements on the manage locations panel.
         *
         * @param string $extraCSSClasses
         * @return string
         */
        function filter_HighlightFeatured($extraCSSClasses) {
            return $extraCSSClasses . (($this->slplus->currentLocation->featured)?' featured ':'');
        }

        /**
         * Add more actions to the Bulk Action drop down on the admin Locations/Manage Locations interface.
         *
         * @param mixed[] $items
         *
         * @return mixed[]
         */
        function filter_LocationsBulkAction($items) {
            return
                array_merge(
                    $items,
                    array(
                        array(
                            'label'     =>  __('Feature Location','slp-experience')      ,
                            'value'     => 'feature_location'                        ,
                        ),
                        array(
                            'label'     =>  __('Stop Featuring Location','slp-experience') ,
                            'value'     => 'remove_feature_location'                 ,
                        ),
                    )
                );
        }

        /**
         * Save input boxes.  Hooked into old-school system for non-serialized inputs.
         * TODO: Get rid of legacy settings.
         */
        function save_inputs( $input_names ) {

            // The Checkboxes
            //
            $BoxesToHit = array(
                SLPLUS_PREFIX.'-'.'no_autozoom',
                SLPLUS_PREFIX.'-'.'no_homeicon_at_start',
            );
            foreach ($BoxesToHit as $BoxName) {
                if (!isset($_REQUEST[$BoxName])) {
                    $_REQUEST[$BoxName] = '';
                    $_REQUEST[$BoxName] = '';
                }
            }

            // Legacy non-serialized to save.
            //
            return array_merge(
                $input_names,
                array(
                    'sl_starting_image'                 ,
                    SLPLUS_PREFIX.'-enhanced_results_string',
                )
            );
        }

        /**
         * Deactivate the competing add-on packs.
         *
         * Does not deal with Widget pack since that is called in WP init, way before this sucker.
         */
        function update_install_info() {
            parent::update_install_info();
            $this->deactivate_replaced_addons();
        }
   }
}
