<?php

if ( ! class_exists('SLP_Experience') ) {
    require_once( SLPLUS_PLUGINDIR . 'include/base_class.addon.php');

    /**
     * Class SLP_Experience
     *
     * @package StoreLocatorPlus\Experience
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2015 - 2016 Charleston Software Associates, LLC
     *
     * @property    array                   $options        Settable options for this plugin.
     * @property    SLPExperience_Admin     $admin          The admin object.
     *
     * @property    SLPExperience_Widget    $widget
     *
     */
    class SLP_Experience extends SLP_BaseClass_Addon {
        public $options = array(
            'address_autocomplete'              => 'none'               ,
            'address_autocomplete_min'          => '3'                  ,
            'address_placeholder'               => ''                   ,
            'add_tel_to_phone'                  => '0'                  ,
            'allow_addy_in_url'                 => '0'                  ,
            'append_to_search'                  => ''                   ,
            'bubblelayout'                      => ''                   ,
            'city'                              => ''                   ,
            'city_selector'                     => 'hidden'             ,
            'country'                           => ''                   ,
            'country_selector'                  => 'hidden'             ,
            'custom_css'                        => ''                   ,
            'disable_initial_directory'         => '0'                  ,
            'email_link_format'                 => 'label_link'         ,
            'featured_location_display_type'    => 'show_within_radius' ,
            'first_entry_for_city_selector'     => ''                   , // default set in init_options
            'first_entry_for_country_selector'  => ''                   , // default set in init_options
            'first_entry_for_radius_selector'   => ''                   , // default set in init_options
            'first_entry_for_state_selector'    => ''                   , // default set in init_options
            'google_map_style'                  => ''                   ,
            'hide_address_entry'                => '0'                  ,
            'hide_bubble'                       => '0'                  ,
            'hide_distance'                     => '0'                  ,
            'hide_map'                          => '0'                  ,
            'hide_radius_selector'              => '0'                  ,
            'hide_search_form'                  => '0'                  ,
            'ignore_radius'                     => '0'                  , // used only to read in old value from versions < 4.1.05
            'installed_version'                 => ''                   ,
            'label_for_city_selector'           => ''                   , // default set in init_options
            'label_for_country_selector'        => ''                   , // default set in init_options
            'label_for_find_button'             => ''                   , // default set in init_options
            'label_for_map_toggle'              => ''                   , // default set in init_options
            'label_for_name'                    => ''                   , // default set in init_options
            'label_for_state_selector'          => ''                   , // default set in init_options
            'layout'                            => ''                   ,
            'maplayout'                         => ''                   ,
            'map_initial_display'               => 'map'                ,
            'map_options_scaleControl'          => '0'                  ,
            'map_options_scrollwheel'           => '0'                  ,
            'map_options_mapTypeControl'        => '0'                  ,
            'map_region'                        => ''                   ,
            'name_placeholder'                  => ''                   ,
            'no_autozoom'                       => '0'                  ,
            'no_homeicon_at_start'              => '1'                  ,
            'orderby'                           => 'sl_distance ASC'    ,
            'popup_email_title'                 => ''                   ,
            'popup_email_from_placeholder'      => ''                   ,
            'popup_email_subject_placeholder'   => ''                   ,
            'popup_email_message_placeholder'   => ''                   ,
            'resultslayout'                     => ''                   ,
            'search_box_title'                  => ''                   , // default set in init_options
            'search_by_name'                    => '0'                  ,
            'searchlayout'                      => ''                   ,
            'searchnear'                        => 'world'              ,
            'selector_behavior'                 => 'use_both'           ,
            'show_country'                      => '1'                  ,
            'show_hours'                        => '1'                  ,
            'show_maptoggle'                    => '0'                  ,
            'starting_image'                    => ''                   ,
            'state'                             => ''                   ,
            'state_selector'                    => 'hidden'             ,
            'url_allow_address'                 => '0'                  ,
        );
        public $admin;

        /**
         * Initialize a singleton of this object.
         *
         * @return SLP_Experience
         */
        public static function init() {
            static  $instance = false;
            if ( ! $instance ) {
                load_plugin_textdomain( 'slp-experience' , false, SLPEXP_REL_DIR . '/languages/');
                $instance = new SLP_Experience(
                        array(
                            'version'           => '4.7.1',
                            'min_slp_version'   => '4.7',

                            'name'              => __( 'Experience' , 'slp-experience' ),
                            'option_name'       => 'slp-experience'                     ,
                            'file'              => SLPEXP_FILE                          ,

                            'activation_class_name'     => 'SLPExperience_Activation',
                            'admin_class_name'          => 'SLPExperience_Admin',
                            'ajax_class_name'           => 'SLPExperience_AJAX',
                            'userinterface_class_name'  => 'SLPExperience_UI',
                            'widget_class_name'         => 'SLPExperience_Widget'
                        )
                    );
            }
            return $instance;
        }

        /**
         * Add cross-element hooks & filters.
         *
         * Haven't yet moved all items to the AJAX and UI classes.
         */
        function add_hooks_and_filters() {

            // UI + AJAX
            add_filter( 'slp_js_options'                        , array( $this , 'filter_AddOptionsToJS'            )           );

            // ADMIN UI Locations
            add_filter('slp_column_data'                        , array( $this , 'filter_FieldDataToManageLocations'), 90 , 3   );

            // Pro Pack Export
            //
            add_filter('slp-pro-dbfields'                       , array( $this , 'filter_Locations_Export_Field'    ), 90       );
            add_filter('slp-pro-csvexport'                      , array( $this , 'filter_Locations_Export_Data'     ), 90       );
        }

        /**
         * Initialize the options properties from the WordPress database.
         */
        function init_options() {

            // Set the defaults for first-run
            // Especially useful for gettext stuff you cannot put in the property definitions.
            //
            $this->option_defaults = $this->options;
            $this->option_defaults['first_entry_for_city_selector'      ] = __( 'All Cities...'             , 'slp-experience' );
            $this->option_defaults['first_entry_for_country_selector'   ] = __( 'All Countries...'          , 'slp-experience' );
            $this->option_defaults['first_entry_for_state_selector'     ] = __( 'All States...'             , 'slp-experience' );
            $this->option_defaults['label_for_city_selector'            ] = __( 'City'                      , 'slp-experience' );
            $this->option_defaults['label_for_country_selector'         ] = __( 'Country'                   , 'slp-experience' );
            $this->option_defaults['label_for_find_button'              ] = __( 'Find Locations'            , 'slp-experience' );
            $this->option_defaults['label_for_name'                     ] = __( 'Name'                      , 'slp-experience' );
            $this->option_defaults['label_for_map_toggle'               ] = __( 'Map'                       , 'slp-experience' );
            $this->option_defaults['label_for_state_selector'           ] = __( 'State'                     , 'slp-experience' );
            $this->option_defaults['popup_email_title'                  ] = __( 'Send An Email'             , 'slp-experience' );
            $this->option_defaults['popup_email_from_placeholder'       ] = __( 'Your email address.' 		, 'slp-experience' );
            $this->option_defaults['popup_email_subject_placeholder'    ] = __( 'Email subject line.' 		, 'slp-experience' );
            $this->option_defaults['popup_email_message_placeholder'    ] = __( 'What do you want to say?' 	, 'slp-experience' );
            $this->option_defaults['search_box_title'                   ] = __( 'Find Our Locations'        , 'slp-experience' );

            parent::init_options();

            // Fix some stuff if empty.
            //
            if (empty($this->options['resultslayout'])) {
                $this->options['resultslayout'] = $this->slplus->defaults['resultslayout'];
            }
            if (empty($this->options['bubblelayout'])) {
                $this->options['bubblelayout'] = $this->slplus->defaults['bubblelayout'];
            }
            if ( empty( $this->options['searchlayout'] ) ) {
                $this->options['searchlayout'] =  $this->slplus->defaults['searchlayout'];
            }
        }

        /**
         * Add the options from this plugin to the list of options going to JavaScript on the UI.
         *
         * @param mixed[] $options the current options array going into the JS localize call
         * @return mixed[] the options array with this plugin's options added to it
         */
        public function filter_AddOptionsToJS( $options ) {
            return
                array_merge(
                    $options,
                    $this->options
                );
        }

        /**
         * Change the export location row data.
         *
         * @param mixed[] $location A location data ih associated array
         * @return mixed[] Location data need to export
         */
        function filter_Locations_Export_Data($location) {
            $this->slplus->database->createobject_DatabaseExtension();
            $exData = $this->slplus->database->extension->get_data($location['sl_id']);
            $location['featured'] = isset($exData['featured']) && $this->slplus->is_CheckTrue($exData['featured']) ? '1' : '0';
            $location['rank'    ] = isset($exData['rank']) ? $exData['rank'] : '';

            return $location;
        }

        /**
         * Change the export location field.
         *
         * @param mixed[] $location Field array
         * @return mixed[] Fields need to export
         */
        function filter_Locations_Export_Field($dbFields) {
            array_push($dbFields, 'featured');
            array_push($dbFields, 'rank'    );

            return $dbFields;
        }

        /**
         * Render the extra fields on the manage location table.
         *
         * SLP Filter: slp_column_data
         *
         * @param string $theData  - the option_value field data from the database
         * @param string $theField - the name of the field from the database (should be sl_option_value)
         * @param string $theLabel - the column label for this column (should be 'Categories')
         * @return type
         */
        function filter_FieldDataToManageLocations($theData,$theField,$theLabel) {
            if (
                ($theField === 'featured') &&
                ($theData  === '0')
            ) {
                $theData = '';
            }
            return $theData;
        }

        /**
         * SQL to select all cities within the given state.
         *
         * TODO: This should go in the SLPExperience_Data class and the calling method should use add_filter( 'slp_extend_get_SQL' , array( $this , 'add_data_queries' ) ); See class.userinterface.php
         *
         * @param   string  $command     The name of the command
         *
         * @return  string               The SQL
         */
        public function select_cities_in_state( $command ) {
            if ( $command !== 'select_cities_in_state'  ) { return $command; }
            return
                'SELECT sl_city' . $this->slplus->database->from_slp_table .
                "WHERE sl_state = '%s' " .
                'GROUP BY sl_city ' .
                'ORDER BY sl_city ASC '
                ;
        }

    }
}