<?php
/*
Plugin Name: Store Locator Plus - Experience
Plugin URI: https://www.storelocatorplus.com/product/experience/
Description: Extends the Store Locator Plus plugin with features that allow site authors and developers to create a customized robust user experience.
Author: Store Locator Plus
Author URI: https://www.storelocatorplus.com
License: GPL3
Tested up to: 4.7
Version: 4.6.5

Text Domain: slp-experience
Domain Path: /languages/

Copyright 2015 - 2016  Charleston Software Associates (support@storelocatorplus.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


// We do not do any heartbeat processing
if ( defined( 'DOING_AJAX' ) && DOING_AJAX && ! empty( $_POST[ 'action' ] ) && ( $_POST['action'] === 'heartbeat' ) ) {
	return;
}

// No direct access allowed outside WordPress
//
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}


// Since this is an add-on for Store Locator Plus
// we only want to "get started" after all plugins are loaded.
//
// This allows us to check our dependencies knowing SLP
// should be loaded into the PHP stack by now.
//
// Concept Credits: @pippinsplugins, @ipstenu
// @see https://pippinsplugins.com/checking-dependent-plugin-active/
// @see https://make.wordpress.org/plugins/2015/06/05/policy-on-php-versions/
//
function SLPExperience_loader() {
    $this_plugin_name = __( 'Store Locator Plus - Experience' , 'slp-experience' );
    $min_wp_version   = '4.4';

    // Requires SLP
    //
    if ( ! defined( 'SLPLUS_PLUGINDIR' ) ) {
        add_action(
            'admin_notices',
            create_function(
                '',
                "echo '<div class=\"error\"><p>".
                sprintf(
                    __( '%s requires Store Locator Plus to function properly. ' , 'slp-experience' ) ,
                    $this_plugin_name
                ).'<br/>'.
                __( 'This plugin has been deactivated.'                         , 'slp-experience' ) .
                __( 'Please install Store Locator Plus.'                        , 'slp-experience' ) .
                "</p></div>';"
            )
        );
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        deactivate_plugins( plugin_basename( __FILE__ ) );
        return;
    }

    // Requires WP Version 4.4+
    //
    global $wp_version;
    if ( version_compare( $wp_version , $min_wp_version , '<' ) ) {
        add_action(
            'admin_notices',
            create_function(
                '',
                "echo '<div class=\"error\"><p>".
                    sprintf(
                        __( '%s requires WordPress %s to function properly. ' , 'slp-experience' ) ,
                        $this_plugin_name,
                        $min_wp_version
                    ).
                __( 'This plugin has been deactivated.'                                           , 'slp-experience' ) .
                __( 'Please upgrade WordPress.'                                                   , 'slp-experience' ) .
                "</p></div>';"
            )
        );
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        deactivate_plugins( plugin_basename( __FILE__ ) );
        return;
    }

    /**
     * Deactivate the widget pack.
     */
    if ( current_user_can( 'activate_plugins' ) ) {
        $slp_widget_slug = 'slp-widgets/slp-widgets.php';
        include_once(ABSPATH . 'wp-admin/includes/plugin.php');
        if (is_plugin_active($slp_widget_slug)) {
            include_once(ABSPATH . 'wp-admin/includes/plugin.php');
            deactivate_plugins($slp_widget_slug);

            add_action(
                'admin_notices',
                create_function(
                    '',
                    "echo '<div class=\"error\"><p>" .
                    sprintf(
                        __('%s deactivated the conflicting SLP Widget Pack add-on. ', 'slp-experience'),
                        $this_plugin_name
                    ) .
                    "</p></div>';"
                )
            );
            return;
        }
    }

    // Define some path constants in an attempt to bypass depth-driven confusion.
    //
    if ( ! defined( 'SLPEXP_REL_DIR'  ) ) { define( 'SLPEXP_REL_DIR'    , plugin_basename( dirname( __FILE__ ) ) ); }  // Relative directory for this plugin in relation to wp-content/plugins
    if ( ! defined( 'SLPEXP_FILE'     ) ) { define( 'SLPEXP_FILE'       ,  __FILE__                              ); }  // FQ File name for this file.

    // Go forth and sprout your tentacles...
    // Get some Store Locator Plus sauce.
    //
    require_once( 'include/class.slp-experience.php' );
    SLP_Experience::init();
}

add_action('plugins_loaded', 'SLPExperience_loader');
