<?php
if (! class_exists('SLPPower_Activation')) {
    require_once(SLPLUS_PLUGINDIR.'/include/base_class.activation.php');

    /**
     * Manage plugin activation.
     *
     * @package StoreLocatorPlus\Power\Activation
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     */
    class SLPPower_Activation  extends SLP_BaseClass_Activation {

        /**
         * Change these single get_option() settings into an slp-power serialized option.
         * key = old setting key , value = new options array key
         * @var array
         */
        public $legacy_options = array(
            'csl-slplus-enhanced_search_auto_submit'    => array( 'key' => 'tag_autosubmit'           , 'since' => '4.4.01'  ),
            'csl-slplus-enhanced_search_show_tag_radio' => array( 'key' => 'tag_selector'             , 'since' => '4.0.019'  , 'callback' =>  'SLPPower_Activation::set_tag_selector'   ) ,
            'csl-slplus-custom_css'                     => array( 'key' => 'custom_css '              , 'since' => '4.2.00'  ),
            'csl-slplus-reporting_enabled'              => array( 'key' => 'reporting_enabled '       , 'since' => '4.1.00'  ),
            'csl-slplus_search_tag_label'               => array( 'key' => 'tag_label '               , 'since' => '4.4.01'  ),
            'csl-slplus_show_tag_any'                   => array( 'key' => 'tag_show_any'             , 'since' => '4.4.01'  ),
            'csl-slplus_show_tag_search'                => array( 'key' => 'tag_selector'             , 'since' => '4.1.02'   , 'callback' => 'SLPPower_Activation::set_tag_selector_search' ),
            'csl-slplus_show_tags'                      => array( 'key' => 'tag_output_processing'    , 'since' => '4.0.014' ),
            'csl-slplus_tag_pulldown_first'             => array( 'key' => 'tag_dropdown_first_entry' , 'since' => '4.4.01'  ),
            'csl-slplus_tag_search_selections'          => array( 'key' => 'tag_selections'           , 'since' => '4.0.014' ),
            'csl-slplus_use_location_sensor'            => array( 'key' => 'use_sensor'               , 'since' => '4.4.01'  ),
        );

        /**
         * Add the contact fields.
         */
        public function add_data_extensions() {
            $this->slplus->database->extension->add_field(
                __( 'Identifier'       ,'slp-power' ),
                'varchar',
                array(
                    'addon'         => $this->addon->short_slug ,
                    'slug'          => 'identifier'             ,
                    'display_type'  => 'text'                   ,
                    'help_text'     => __( 'The identifier field is meant to store a unique location record ID from an external data source. ' , 'slp-power' ) .
                                       __( 'During a CSV import, this field is used to match up incoming data with existing locations. '       , 'slp-power' )
                ));
            $this->slplus->database->extension->add_field( __( 'Contact'          ,'slp-power' ), 'varchar', array( 'slug' => 'contact'       , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'First Name'       ,'slp-power' ), 'varchar', array( 'slug' => 'first_name'    , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Last Name'        ,'slp-power' ), 'varchar', array( 'slug' => 'last_name'     , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Title'            ,'slp-power' ), 'varchar', array( 'slug' => 'title'         , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Department'       ,'slp-power' ), 'varchar', array( 'slug' => 'department'    , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Training'         ,'slp-power' ), 'varchar', array( 'slug' => 'training'      , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Facility Type'    ,'slp-power' ), 'varchar', array( 'slug' => 'facility_type' , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Office Phone'     ,'slp-power' ), 'varchar', array( 'slug' => 'office_phone'  , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Mobile Phone'     ,'slp-power' ), 'varchar', array( 'slug' => 'mobile_phone'  , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Contact Fax'      ,'slp-power' ), 'varchar', array( 'slug' => 'contact_fax'   , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Contact Email'    ,'slp-power' ), 'varchar', array( 'slug' => 'contact_email' , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Office Hours'     ,'slp-power' ), 'text'   , array( 'slug' => 'office_hours'  , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Contact Address'  ,'slp-power' ), 'text'   , array( 'slug' => 'contact_address','addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Notes'            ,'slp-power' ), 'text'   , array( 'slug' => 'notes'         , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Introduction'     ,'slp-power' ), 'text'   , array( 'slug' => 'introduction'  , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Year Established' ,'slp-power' ), 'varchar', array( 'slug' =>'year_established','addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'County'           ,'slp-power' ), 'varchar', array( 'slug' => 'county'        , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'District'         ,'slp-power' ), 'varchar', array( 'slug' => 'district'      , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Region'           ,'slp-power' ), 'varchar', array( 'slug' => 'region'        , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->add_field( __( 'Territory'        ,'slp-power' ), 'varchar', array( 'slug' => 'territory'     , 'addon' => $this->addon->short_slug ) );
	        $this->slplus->database->extension->add_field( __( 'Image'            ,'slp-power' ), 'varchar', array( 'slug' => 'contact_image' , 'addon' => $this->addon->short_slug ) );
            $this->slplus->database->extension->update_data_table();

        }

        /**
         * Clean out duplicate report table indexes.
         */
        private function clean_duplicate_indexes() {
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            $this->clean_duplicate_index_from_table( 'slp_rep_query'         , 'slp_repq_time' );
            $this->clean_duplicate_index_from_table( 'slp_rep_query_results' , 'slp_repq_id'   );
        }


        /**
         * Clean the duplicate index for each table.
         *
         * @param string $table
         * @param string $column
         */
        private function clean_duplicate_index_from_table( $table , $column ) {
            global $wpdb;

            $table = $wpdb->prefix . $table;
            $tableindices = $wpdb->get_results("SHOW INDEX FROM {$table};");

            $the_list = wp_list_filter( $tableindices, array( 'Column_name' => $column ) );

            if ( count( $the_list ) > 1 ) {
                $first = true;
                foreach ( $the_list as $list_item ) {
                    if ( $first ) { $first = false; continue; }
                    drop_index( $list_item->Table , $list_item->Key_name );
                }
            }
        }

        /**
         * Install or update the main table
         * @global object $wpdb
         */
        function create_MoreInfoTable() {
            $this->dbupdater(
                $this->addon->category_data->get_SQL( 'create_tagalong_helper' ),
                $this->addon->category_data->plugintable['name']
            );
        }

        /**
         * Set the tag selector value for the show_tag_radio from 4.0.019
         *
         * @param $legacy_value
         * @return string
         */
        static function set_tag_selector( $legacy_value ) {
            return ( $legacy_value === '1' ) ? 'radiobutton' : 'none';
        }

        /**
         * Set the tag selector value for show_tag_search from 4.1.02.
         *
         * @param $legacy_value
         * @return string
         */
        static function set_tag_selector_search( $legacy_value ) {
            if ( $legacy_value === false                            ) { return 'none';      }
            if ( ($legacy_value === '0'  )                          ) { return 'none';      }

            $options = get_option( 'csl-slplus-PRO-options' , array() );
            if ( isset( $options['tag_selections'] ) && ! empty( $options['tag_selections'] ) ) { return 'dropdown';  }

            return 'textinput';
        }

	    /**
	     * Migrate an option slug to this add on and delete the old option.
	     *
	     * @param string $option_slug
	     *
	     * @return boolean  true if migrated, false otherwise
	     */
        private function migrate_legacy_options( $option_slug ) {
	        $options     = get_option( $option_slug );
	        if ( $options === false ) {
	        	return false;
	        }

	        if ( is_array( $options ) ) {
		        array_walk( $options, array( $this->addon, 'set_ValidOptions' ) );
		        array_merge( $this->addon->options, $options );
	        }

	        delete_option( $option_slug );

	        return true;
        }

        /**
         * Update legacy settings.
         */
        function update() {

            // Never installed since 4.5 was our first product release.
            //
            // Import settings from other add-ons that are still valid.
            //
            if ( version_compare( $this->updating_from , '4.5' , '<=' ) ) {
                $this->migrate_legacy_options( 'csl-slplus-PRO-options' );
	            if ( $this->migrate_legacy_options( 'slp_storepages-options' ) ) {
		            $this->addon->options['use_pages'] = '1';
	            }
	            $this->migrate_legacy_options( 'slp-directory-builder-options' );
	            $this->migrate_legacy_options( 'csl-slplus-TAGALONG-options' );

                update_option( $this->addon->option_name , $this->addon->options );
            }

            parent::update();

	        $this->convert_to_smart_options();

            $this->install_reporting_tables();

            $this->create_MoreInfoTable();

            // Do this after tables created to avoid warnings/errors.
            //
            if ( version_compare( $this->updating_from , '4.4' , '<=' ) ) {
                $this->clean_duplicate_indexes();
            }

            if ( $this->slplus->is_CheckTrue( $this->addon->options['use_contact_fields'] ) ) {
                $this->add_data_extensions();
            }
        }

        /*************************************
         * Install reporting tables
         *
         * Update the plugin version in config.php on every structure change.
         */
        private function install_reporting_tables() {
            global $wpdb;

            $charset_collate = '';
            if ( ! empty($wpdb->charset) )
                $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
            if ( ! empty($wpdb->collate) )
                $charset_collate .= " COLLATE $wpdb->collate";

            // Reporting: Queries
            //
            $table_name = $wpdb->prefix . "slp_rep_query";
            $sql = "CREATE TABLE $table_name (
                    slp_repq_id         bigint(20) unsigned NOT NULL auto_increment,
                    slp_repq_time       timestamp NOT NULL default current_timestamp,
                    slp_repq_query      varchar(255) NOT NULL,
                    slp_repq_tags       varchar(255),
                    slp_repq_address    varchar(255),
                    slp_repq_radius     varchar(5),
                    meta_value          longtext,
                    PRIMARY KEY  (slp_repq_id),
                    KEY slp_repq_time (slp_repq_time)
                    )
                    $charset_collate
                    ";
            $this->dbupdater($sql,$table_name);

            // Reporting: Query Results
            //
            $table_name = $wpdb->prefix . "slp_rep_query_results";
            $sql = "CREATE TABLE $table_name (
                    slp_repqr_id    bigint(20) unsigned NOT NULL auto_increment,
                    slp_repq_id     bigint(20) unsigned NOT NULL,
                    sl_id           mediumint(8) unsigned NOT NULL,
                    PRIMARY KEY  (slp_repqr_id),
                    KEY slp_repq_id (slp_repq_id)
                    )
                    $charset_collate
                    ";

            // Install or Update the slp_rep_query_results table
            //
            $this->dbupdater($sql,$table_name);
        }

        /**
         * Update the data structures on new db versions.
         *
         * @global object $wpdb
         * @param type $sql
         * @param type $table_name
         * @return string
         */
        private function dbupdater($sql,$table_name) {
            global $wpdb;
            $retval = ( $wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name ) ? 'new' : 'updated';

            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
            global $EZSQL_ERROR;
            $EZSQL_ERROR = array();

            return $retval;
        }

	    /**
	     * Convert legacy add on options to smart options.
	     *
	     * The parent base_class.admin.php from SLP will auto-call update_option( ) for addon->options.
	     */
	    private function convert_to_smart_options() {
		    $option_list = array(
			    'log_import_messages',
			    'log_schedule_messages',
		    );
		    foreach ( $option_list as $option_slug ) {
			    if ( isset( $this->addon->options[ $option_slug ] ) ) {
			    	switch ( $option_slug ) {
					    case 'log_import_messages':
					    	$this->addon->create_object_import_messages();
					    	break;
					    case 'log_schedule_messages':
					    	$this->addon->create_object_schedule_messages();
					    	break;
				    }
				    $this->slplus->smart_options->{$option_slug}->value = $this->addon->options[ $option_slug ];
				    unset ( $this->addon->options[ $option_slug ] );
			    }
		    }
	    }
    }
}