<?php
if ( ! class_exists( 'SLPPower_Admin_Locations_Actions' ) ) {

    /**
     * Handle location actions.
     *
     * @package   StoreLocatorPlus\SLPPower\Admin\Locations
     * @author    Lance Cleveland <lance@storelocatorplus.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * Text Domain: slp-power
     *
     * @property        SLPPower                       $addon
     * @property        SLPPower_Admin_Locations       $location_handler
     * @property-read   int                            $offset                 The action loop SQL offset for locations
     * @property        SLPlus_Admin_Locations_Actions $slp_action_handler
     */
    class SLPPower_Admin_Locations_Actions extends SLPlus_BaseClass_Object {
        public  $addon;
        public  $location_handler;
        private $offset;
        public  $slp_action_handler;

        /**
         * Setup the show uncoded filter for manage locations.
         *
         * @param string $where
         *
         * @return string
         */
        function add_where_uncoded_to_sql( $where ) {
            return
                $where .
                ( empty( $where ) ? '' : ' AND ' ) .
                ' (NOT (' . $this->slplus->database->filter_SetWhereValidLatLong( '' ) . ') ' .
                "or sl_latitude IS NULL or sl_longitude IS NULL)";
        }

        /**
         * Perform the manage locations action for bulk categorization.
         */
        private function categorize_locations() {
            if ( ! $this->slp_action_handler->set_locations() ) {
                return;
            }

            // Extract the categories
            //
            $this->addon->create_object_category_manager();
            $inputCats = $this->addon->category_manager->get_categories_from_input();
            if ( count( $inputCats ) <= 0 ) {
                return;
            }

            add_action( 'slp_location_add', array( $this, 'update_category_and_page_id' ) );
            add_action( 'slp_location_save', array( $this, 'update_category_and_page_id' ) );

            $id = $this->slp_action_handler->get_next_location();
            while ( ! is_null( $id ) ) {
                $this->slplus->currentLocation->set_PropertiesViaDB( $id );

                // The Action
                $this->update_category_and_page_id();
                $this->slplus->currentLocation->MakePersistent();

                if ( $this->slp_action_handler->all ) {
                    $this->offset = 0;
                }
                $id = $this->slp_action_handler->get_next_location();
            }
        }

        /**
         * Export locations to a local CSV file that to be fetched manually.
         */
        private function export_locations_to_local_csv_file() {

            $this->slplus->set_php_timeout();

            $this->addon->create_CSVLocationExporter();

            $csv_dir = SLPLUS_UPLOADDIR . 'csv';
            if ( ! is_dir( $csv_dir ) ) {
                mkdir( $csv_dir, 0755 );
            }

            $this->addon->csvExporter->do_WriteFile( 'file://' . SLPLUS_UPLOADDIR . 'csv/exported_slp_locations.csv' );
        }

        /**
         * Create Pages
         */
        private function create_pages() {
            if ( ! $this->slp_action_handler->set_locations() ) {
                return;
            }

            add_filter( 'slp_location_page_attributes', array( $this , 'set_taxonomy_for_location' ) );


            $id = $this->slp_action_handler->get_next_location();
            while ( ! is_null( $id ) ) {
                $this->slplus->currentLocation->set_PropertiesViaDB( $id );

                // The Action
                $this->slplus->currentLocation->crupdate_Page();

                if ( $this->slp_action_handler->all ) {
                    $this->offset = 0;
                }
                $id = $this->slp_action_handler->get_next_location();
            }
        }

        /**
         * Delete Pages
         */
        private function delete_pages() {
            if ( ! $this->slp_action_handler->set_locations() ) {
                return;
            }

            $id = $this->slp_action_handler->get_next_location();
            while ( ! is_null( $id ) ) {
                $this->slplus->currentLocation->set_PropertiesViaDB( $id );

                // The Action
                $post = get_post( $this->slplus->currentLocation->linked_postid );
                if ( ( $post !== null ) && ( $post->post_type === SLPlus::locationPostType ) ) {
                    wp_delete_post( $this->slplus->currentLocation->linked_postid, true );
                }

                if ( $this->slp_action_handler->all ) {
                    $this->offset = 0;
                }
                $id = $this->slp_action_handler->get_next_location();
            }
        }

        /**
         * Additional location processing on manage locations admin page.
         *
         * @param    string $action
         */
        public function process( $action ) {

            switch ( $action ) {
                // Categorize Locations
                //
                case 'categorize' :
                    $this->categorize_locations();
                    break;

                case 'createpage':
                    $this->create_pages();
                    break;

                case 'deletepage':
                    $this->delete_pages();
                    break;

                // Export Locations To CSV
                // Also filter by property
                //
                case 'export':
                    $this->slplus->notifications->enabled = false;
                    $this->addon->admin->create_object_admin_location_filters();
                    add_action( 'slp_manage_location_where', array(
                        $this->location_handler,
                        'action_ManageLocations_ByProperty',
                    ) );
                    $this->slplus->notifications->enabled = true;
                    break;

                case 'export_local':
                    $this->slplus->notifications->enabled = false;
                    $this->export_locations_to_local_csv_file();
                    $this->addon->admin->create_object_admin_location_filters();
                    add_action( 'slp_manage_location_where', array(
                        $this->location_handler,
                        'action_ManageLocations_ByProperty',
                    ) );
                    $this->slplus->notifications->enabled = true;
                    break;

                // Filter Locations on Manage Locations List
                //
                case 'filter_by_category' :

                    // Taxonomies are in REQUEST, filter by categories.
                    //
                    if ( isset( $_REQUEST['tax_input'] ) && isset( $_REQUEST['tax_input']['stores'] ) ) {
                        $this->addon->admin->create_object_admin_location_filters();
                        add_filter( 'slp_manage_location_where', array(
                            $this->addon->admin->admin_location_filters,
                            'filter_locations_by_category',
                        ) );
                    }
                    break;

                // Import a CSV File
                //
                case 'import':
                    $_REQUEST['selected_nav_element'] = '#wpcsl-option-import';
                    $this->addon->admin->save_my_settings();
                    $this->addon->create_CSVLocationImporter();
                    $this->addon->csvImporter->start_import();
                    break;

                // Add tags to locations
                case 'add_tag':
                    $this->tag_locations( 'add' );
                    break;

                // Remove tags from locations
                case 'remove_tag':
                    $this->tag_locations( 'remove' );
                    break;

                // Recode The Selected Locations
                case 'recode_all':
                    $this->addon->recode_all_uncoded_locations();
                    break;

                // Recode The Address
                case 'recode':
                    $this->slplus->notifications->delete_all_notices();
                    if ( isset( $_REQUEST['sl_id'] ) ) {

                        // Process SL_ID Array
                        // TODO: use where clause in database property
                        //
                        foreach ( (array) $_REQUEST['sl_id'] as $location_id ) {
                            $this->slplus->currentLocation->set_PropertiesViaDB( $location_id );
                            $this->slplus->currentLocation->do_geocoding();
                            if ( $this->slplus->currentLocation->dataChanged ) {
                                $this->slplus->currentLocation->MakePersistent();
                            }
                        }
                    }
                    break;

                // Filter to show uncoded locations only.
                case 'show_uncoded':
                    add_action( 'slp_manage_location_where', array( $this, 'add_where_uncoded_to_sql' ) );
                    break;

                // Filter with specific location properties
                case 'add':
                case 'save':
                case 'filter_by_property':
                    $this->addon->admin->create_object_admin_location_filters();
                    add_action( 'slp_manage_location_where', array(
                        $this->location_handler,
                        'action_ManageLocations_ByProperty',
                    ) );
                    add_filter( 'slp_manage_locations_actionbar_ui', array(
                        $this->addon->admin->admin_location_filters,
                        'createstring_FilterDisplay',
                    ) );
                    break;

                // Reset on show_all
                case 'show_all':
                    $this->addon->admin->create_object_admin_location_filters();
                    $this->addon->admin->admin_location_filters->reset();
                    break;

                default:
                    break;
            }
        }

        /**
         * Set the Tagalong categories for the new store page.
         *
         * SLP Filter: slp_location_page_attributes
         *
         * @param mixed[] $pageAttributes - the wp_insert_post page attributes
         *
         * @return mixed[] - pageAttributes with tax_input set
         */
        function set_taxonomy_for_location( $pageAttributes ) {
            $this->addon->set_LocationCategories();
            return array_merge( $pageAttributes, array( 'tax_input' =>  array( SLPlus::locationTaxonomy => $this->addon->current_location_categories ) ) );
        }

        /**
         * Tag a location
         *
         * @param string $action = add or remove
         */
        private function tag_locations( $action ) {
            if ( ! $this->slp_action_handler->set_locations() ) {
                return;
            }

            $id = $this->slp_action_handler->get_next_location();
            while ( ! is_null( $id ) ) {
                $this->slplus->currentLocation->set_PropertiesViaDB( $id );

                //adding tags
                if ( $action === 'add' ) {
                    $new_tags = empty( $this->slplus->currentLocation->tags ) ? '' : ',';
                    $new_tags .= $_REQUEST['sl_tags'];
                    $this->slplus->currentLocation->tags .= $new_tags;

                    //removing tags
                } else {
                    $this->slplus->currentLocation->tags = '';
                }
                $this->slplus->currentLocation->MakePersistent();

                if ( $this->slp_action_handler->all ) {
                    $this->offset = 0;
                }
                $id = $this->slp_action_handler->get_next_location();
            }

        }

        /**
         * Attach our category data to the update string.
         *
         * Put it in the sl_option_value field as a seralized string.
         *
         * Assumes currentLocation is set.
         *
         * @return string the modified SQL set syntax
         */
        function update_category_and_page_id() {
            $this->addon->create_object_category_manager();
            $this->addon->category_manager->set_categories_from_input();
            $this->addon->category_manager->update_location_category();

        }
    }
}