<?php
if ( ! class_exists( 'SLPPower_Admin_Locations_Import' ) ) {

	/**
	 * The things that modify the Admin / Locations / Import UX.
	 *
	 *
	 * @package StoreLocatorPlus\SLPPower\Admin\Locations\Import
	 * @author Lance Cleveland <lance@storelocatorplus.com>
	 * @copyright 2016 Charleston Software Associates, LLC
	 *
	 * Text Domain: slp-power
	 *
	 * @property        SLPPower                        $addon
	 *
     * @property-read   SLP_Settings                    $settings
	 *
	 * @property        SLPPower_Text_Modifier          TextModifier
	 */
	class SLPPower_Admin_Locations_Import  extends SLP_Object_With_Objects {
		public  $addon;

		protected $class_prefix = 'SLPPower_';
		protected $objects = array(
			'Text_Modifier' => array(
				'subdir' => 'include/slp_mods/', 'object' => null ,
			),
		);
        private $settings;


        /**
         * Add the bulk upload form to add locations.
         *
         * @param  SLP_Settings    $settings
         */
        function create_import_subtab( $settings ) {
            $this->settings = $settings;
            $section_params['name'] = __( 'Import', 'slp-power' );
            $section_params['slug'] = 'import';
            $section_params['opening_html'] =
                "<form id='importForm' name='importForm' method='post' enctype='multipart/form-data' autocomplete='off'>" .
                "<input type='hidden' name='act' id='act' value='import' />"
            ;
            $section_params['closing_html'] = '</form>';

            $this->settings->add_section( $section_params );

            $this->new_import_upload_csv_group    ( $section_params['slug'] );
	        $this->import_remote_file_group       ( $section_params['slug'] );
	        $this->import_schedule_group          ( $section_params['slug'] );
            $this->import_file_settings_group     ( $section_params['slug'] );
            $this->import_messages_group          ( $section_params['slug'] );
        }

        /**
         * Get the cron schedule as a formatted HTML string.
         *
         * @return string
         */
        private function create_string_cron_schedule() {
            if ( ! isset( $this->file_meta ) ) { return; }
            $html     = '';
            $schedule = wp_get_schedule( 'cron_csv_import', array( 'import_csv', $this->file_meta ) );
            if ( ! empty( $schedule ) ) {
                $html =
                    sprintf( __( 'CSV file imports are currently scheduled to run %s.', 'slp-power' ), $schedule ) .
                    '<br/><br/>';
            } else {
                $html = __( 'There are no scheduled imports.' , 'slp-power' );
            }
            return $html;
        }

        /**
         * Create the new upload form.
         *
         * @return string
         */
        private function create_string_upload_form() {
            $html =
                '<div class="upload-type-container">'                                   .
                    '<a id="select-files" href="javascript:void(0);" class="new_button">'          .
                        '<span class="icon-label dashicons dashicons-upload">' . __('Upload a file', 'slp-power') . '</span>' .
                    '</a>'.
                    '<div id="upload_status" class="upload_status">'                    .
                        '<div id="file_list" class="slp_upload_file_list"></div>'       .
                    '</div>'                                                            .
                    '<div id="progress" class="slp_progress">'                          .
                        '<div id="upload_process" class="slp_upload_process"></div>'    .
                    '</div>'                                                            .
                    '<div id="upload_info" class="slp_upload_info"></div>'              .
                '</div>'                                                                ;

            return $html;
        }

        /**
         * File Settings Group
         *
         * @param string $section_slug
         */
        private function import_file_settings_group( $section_slug ) {
            $group_params['header'      ] = __( 'File Settings', 'slp-power' );
            $group_params['group_slug'  ] = 'file_settings';
            $group_params['section_slug'] = $section_slug;
            $group_params['intro'       ] = __( 'These settings apply to both Upload CSV File and Remote File Retrieval.' , 'slp-power' );
            $group_params['plugin'      ] = $this->addon;
            $this->settings->add_group(  $group_params );

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'type'          => 'checkbox',
                'option'        => 'csv_skip_geocoding',
                'label'         => __( 'Skip Geocoding', 'slp-power' ),
                'description'   => __( 'Do not check with the Geocoding service to get latitude/longitude.  Locations without a latitude/longitude will NOT appear on map base searches.', 'slp-power' ),
                'onChange'      => 'SLP_ADMIN.options.change_option(this);'
            ));

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'type'          => 'checkbox',
                'option'        => 'load_data',
                'label'         => __( 'Load Data', 'slp-power' ),
                'description'   => __( 'If checked use the faster MySQL Load Data method of file processing.', 'slp-power' ) . ' ' .
                                   sprintf( __( 'Only base plugin data can be loaded, see the <a href="%s">approved field name list</a>.', 'slp-power' ), $this->slplus->support_url ),
                'onChange'      => 'SLP_ADMIN.options.change_option(this);'
            ) );

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'type'          => 'checkbox',
                'option'        => 'csv_clear_messages_on_import',
                'label'         => __( 'Clear Messages', 'slp-power' ),
                'description'   => __( 'Clear import messages at the start of each new import.', 'slp-power' ),
                'onChange'      => 'SLP_ADMIN.options.change_option(this);'
            ));

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'option'        => 'csv_duplicates_handling',
                'type'          => 'dropdown',
                'label'         => __( 'Duplicates Handling', 'slp-power' ),
                'description'   => __( 'How should duplicates be handled? ', 'slp-power' ) .
                                   __( 'Duplicates are records that match on name and complete address with country. ', 'slp-power' ) .
                                   __( 'Add (default) will add new records when duplicates are encountered. ', 'slp-power' ) . '<br/>' .
                                   __( 'Skip will not process duplicate records. ', 'slp-power' ) . '<br/>' .
                                   __( 'Update will update duplicate records. ', 'slp-power' ) .
                                   __( 'To update name and address fields the CSV must have the ID column with the ID of the existing location.', 'slp-power' ),
                'custom'        => array(
                    array( 'label' => __( 'Add', 'slp-power' ), 'value' => 'add' ),
                    array( 'label' => __( 'Skip', 'slp-power' ), 'value' => 'skip' ),
                    array( 'label' => __( 'Update', 'slp-power' ), 'value' => 'update' ),
                ),
                'onChange'      => 'SLP_ADMIN.options.change_option(this);'
            ) );

        }

        /**
         * Import Messages Group
         *
         * @param string $section_slug
         */
        private function import_messages_group( $section_slug ) {
            $group_params['header'      ] = __( 'Import Messages', 'slp-power' );
            $group_params['group_slug'  ] = 'import_messages';
            $group_params['section_slug'] = $section_slug;
            $group_params['plugin'      ] = $this->addon;
            $this->settings->add_group(  $group_params );

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'type'          => 'details',
                'label'         => 'Import Messages',
                'custom'        => $this->addon->csvImporter->messages->get_message_string()
            ));

            if ( $this->addon->csvImporter->messages->exist() ) {
                $clear_text = __( 'Clear import messages.', 'slp-power' );
                $this->settings->add_ItemToGroup( array(
                    'group_params'  => $group_params,
                    'type'         => 'hyperbutton',
	                'button_label'  => $clear_text,
	                'id'            => 'import_messages_clear',
                    'onClick'       => 'SLPPOWER_ADMIN_LOCATIONS.messages.clear_import_messages()'
	                )
                );
            }
        }

        /**
         * Remote File Retrieval Group
         *
         * @param string    $section_slug
         */
        private function import_remote_file_group( $section_slug ) {
            $group_params['header'      ] = __( 'Remote File Retrieval', 'slp-power' );
            $group_params['group_slug'  ] = 'remote_file';
            $group_params['section_slug'] = $section_slug;
            $group_params['plugin'      ] = $this->addon;
            $this->settings->add_group(  $group_params );


            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'type'          => 'details',
                'label'         => '' ,
                'description'   =>  sprintf( __( 'The current WordPress time (GMT) is %s.', 'slp-power' ), current_time( 'mysql', true ) )
            ));

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'option'        => 'csv_file_url',
                'label'         => __( 'CSV File URL'                                       , 'slp-power' ),
                'description'   => __( 'Enter a full URL for a CSV file you wish to import' , 'slp-power' )
            ) );

            $this->settings->add_ItemToGroup( array(
                    'group_params'  => $group_params,
                    'option'        => 'cron_import_recurrence',
                    'type'          => 'dropdown',
                    'label'         => __( 'Import Schedule', 'slp-power' ),
                    'description'   => __( 'How often to fetch the file from the URL. '                             , 'slp-power' ) .
                                       __( 'None loads the remote file immediately with no background processing. ' , 'slp-power' ) .
                                       __( 'At loads the file one time on or after the time specified. '            , 'slp-power' ) .
                                       __( 'Set to none and leave the URL blank to clear the cron job.  '           , 'slp-power' )
                ,
                    'custom'        => array(
                        array( 'label' => __( 'None'        , 'slp-power' ), 'value' => 'none'      ),
                        array( 'label' => __( 'At'          , 'slp-power' ), 'value' => 'at'        ),
                        array( 'label' => __( 'Hourly'      , 'slp-power' ), 'value' => 'hourly'    ),
                        array( 'label' => __( 'Twice Daily' , 'slp-power' ), 'value' => 'twicedaily'),
                        array( 'label' => __( 'Daily'       , 'slp-power' ), 'value' => 'daily'     ),
                    ))
            );

            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'option'        => 'cron_import_timestamp',
                'label'         => __( 'Import Time'                                                                                , 'slp-power' ),
                'description'   => __( 'What time to run the recurring import from this URL.  '                                     , 'slp-power' ) .
                                   __( 'WordPress cron is not exact, it executes the next time a visitor comes to your site.  '     , 'slp-power' ) .
                                   __( 'WordPress times are UTC/GMT time NOT local time.  '                                         , 'slp-power' ) .
                                   sprintf( '<a href="%s" target="slp">%s"</a>', 'http://php.net/manual/en/datetime.formats.php', __('PHP Date and Time formats', 'slp-power' ) ) .
                                   __( 'are acceptable, such as "now" or "14:00" or "sunday"' , 'slp-power')
            ));


            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'type'          => 'hyperbutton',
	            'button_label'  => __( 'Import Locations', 'slp-power' ),
	            'onClick'       => 'jQuery("#importForm").submit()'
            ));


        }

        /**
         * Add the import messages group.
         *
         * @param string $section_slug
         */
        private function import_schedule_group( $section_slug ) {
            $group_params['header'      ] = __( 'Scheduled Activity', 'slp-power' );
            $group_params['group_slug'  ] = 'schedule';
            $group_params['section_slug'] = $section_slug;
            $group_params['plugin'      ] = $this->addon;
            $this->settings->add_group(  $group_params );


            $this->settings->add_ItemToGroup( array(
                'group_params'  => $group_params,
                'type'          => 'details',
                'description'   =>  $this->create_string_cron_schedule()
            ));
        }

        /**
         * Upload CSV Group
         *
         * @param   string   $section_slug
         */
        private function new_import_upload_csv_group( $section_slug ) {
	        $this->instantiate( 'Text_Modifier' );

            $group_params['header'      ] = __( 'Upload A File', 'slp-power' );
            $group_params['group_slug'  ] = 'upload_a_file';
            $group_params['section_slug'] = $section_slug;
            $group_params['plugin'      ] = $this->addon;
            $this->settings->add_group(  $group_params );

            $this->settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'type'          => 'subheader'  ,
                'label'         => ''           ,
                'description'   => __( 'CSV files need a header row to define the columns. ' , 'slp-power' ) .
                                   __( 'Files should be in UTF-8 format with comma delimiters. ' , 'slp-power' ) .
                                   __( 'Imports run in the background via a detached process. ', 'slp-power' ).
                                   __( 'Importing a file twice before the first file is done processing can result in duplicate entries. ' , 'slp-power' ) .
                                   $this->slplus->text_manager->get_web_link( 'docs_for_csv_import' )
            ));

            $this->settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'type'          => 'details',
                'description'   => $this->create_string_upload_form()
            ));
        }
	}
}