<?php
if ( ! class_exists( 'SLPPower_Admin_Locations' ) ) {

	/**
	 * The things that modify the Admin / Locations UX.
	 *
	 *
	 * @package StoreLocatorPlus\SLPPower\Admin\Locations
	 * @author Lance Cleveland <lance@storelocatorplus.com>
	 * @copyright 2016 Charleston Software Associates, LLC
	 *
	 * Text Domain: slp-power
	 *
     * @property        SLPPower_Admin_Locations_Actions    $action_handler
     * @property        SLPPower                            $addon
     * @property        SLPPower_Admin_Locations_Import     $import_subtab
     * @property-read   SLP_Settings                        $settings
	 */
	class SLPPower_Admin_Locations  extends SLPlus_BaseClass_Object {
        private $action_handler;
		public  $addon;
        public  $import_subtab;
        private $settings;

        /**
         * Start things for admin locations.
         */
        public function initialize() {
            $this->create_object_locations_import_subtab();
            add_action( 'slp_manage_locations_action'           , array( $this ,'handle_actions' ) );
            add_action( 'slp_build_locations_panels'            , array( $this ,'action_AddLocationsPanels' ) , 30 );
        }

        /**
         * Add the import/export panels to the Locations tab.
         *
         * @param  SLP_Settings    $settings
         */
        function action_AddLocationsPanels( $settings ) {
            if ( ! current_user_can('manage_slp_admin') ) { return; }
            $this->settings = $settings;

            $this->addon->admin->create_object_admin_location_filters();

            $this->addon->create_CSVLocationImporter();
            $this->import_subtab->create_import_subtab( $this->settings );
        }

        /**
         * Filter the manage locations list by property matches in the filter form.
         *
         * @param string $where
         * @return string
         */
        function action_ManageLocations_ByProperty($where) {
            // Get the SQL command and where params
            //
            list($sqlCommands, $sqlParams) = $this->addon->admin->admin_location_filters->create_LocationSQLCommand( $this->create_LocationFilterInputArray() );
            $whereclause = $this->slplus->database->get_SQL('where_default');
            if ( strpos( $whereclause, '%' ) !== false ) {
                $whereclause = $this->slplus->database->db->prepare( $whereclause , $sqlParams );
            }
            $whereclause = preg_replace('/^\s+WHERE /i','',$whereclause);

            return $whereclause;
        }

        /**
         * Create an array of location filter inputs from $_REQUEST.
         *
         * @return string[] named array
         */
        private function create_LocationFilterInputArray() {
            $formdata_array = array();
            $location_filter_inputs =
                array(
                    'name'              ,
                    'state_filter'      ,
                    'state_joiner'      ,
                    'zip_filter'        ,
                    'zip_filter_joiner' ,
                    'country_joiner'    ,
                    'country_filter'
                );
            foreach ($location_filter_inputs as $input) {
                if ( ! empty( $_REQUEST[$input] ) ) {
                    $formdata_array[$input] = $_REQUEST[$input];
                }
            }
            return $formdata_array;
        }

        /**
         * Create the action handler object.
         *
         * @param   SLPlus_Admin_Locations_Actions$slp_action_handler
         */
        private function create_object_action_handler( $slp_action_handler ) {
            if ( ! isset( $this->action_handler ) ) {
                require_once( 'class.admin.locations.actions.php' );
                $this->action_handler = new SLPPower_Admin_Locations_Actions( array(
                    'addon' => $this->addon       , 
                    'location_handler'   => $this ,
                    'slp_action_handler' => $slp_action_handler , 
                ) );
            }
        }
        /**
         * Create and attach the admin locations import object.
         */
        private function create_object_locations_import_subtab() {
            if ( ! isset( $this->import_subtab ) ) {
                require_once('class.admin.locations.import.php');
                $this->import_subtab = new SLPPower_Admin_Locations_Import( array( 'addon' => $this->addon ) );
            }
        }

        /**
         * Additional location processing on manage locations admin page.
         *
         * @param   SLPlus_Admin_Locations_Actions  $slp_action_handler
         */
        public function handle_actions( $slp_action_handler ) {
            $this->create_object_action_handler( $slp_action_handler );
            $this->action_handler->process( $_REQUEST['act'] );
        }
	}
}