<?php
if ( ! class_exists( 'SLPPower_Admin' ) ) {

	require_once( SLPLUS_PLUGINDIR . 'include/base_class.admin.php' );

	/**
	 * Holds the admin-only code.
	 *
	 * This allows the main plugin to only include this file in admin mode
	 * via the admin_menu call.   Reduces the front-end footprint.
	 *
	 * @package   StoreLocatorPlus\Power\Admin
	 * @author    Lance Cleveland <lance@charlestonsw.com>
	 * @copyright 2016 Charleston Software Associates, LLC
	 *
	 * Text Domain: slp-power
	 *
	 * @property        SLPPower                          $addon
	 * @property        SLPPower_Admin_Location_Filters   $admin_location_filters     Admin Location Filters UI Object
	 * @property-read   SLPPower_Admin_Categories         $categories_tab             The categories tab.
	 * @property-read   SLPPower_Admin_EditTags           $edit_tags                  The WP tag editor object.
	 * @property-read   SLPPower_Admin_ExperienceSettings $experience
	 * @property        string                            $export_fileurl             The export fileurl for CSV location export.
	 * @property-read   SLPPower_Admin_General            $Admin_General
	 * @property        SLPPower_Admin_Locations          $locations
	 * @property        SLPPower_Admin_Pages              $pages                      The pages admin class.
	 * @property        SLPPower_Admin_Reports            $reports_tab                The reports tab.
	 * @property        SLP_Settings                      $settings                   The tag list settings object.
	 */
	class SLPPower_Admin extends SLP_BaseClass_Admin {
		public  $addon;
		public  $admin_location_filters;
		private $AdminUI;
		public  $categories_tab;
		private $data;
		private $edit_tags;
		private $experience;
		private $export_fileurl;
		public  $js_requirements = array( 'jquery' );
		public  $locations;
		public  $pages;
		public  $reports_tab;
		public  $settings;
		public  $settings_interface;
		public  $settings_pages  = array(
			'slp-categories'       => array(
				'ajax_orderby_catcount',
				'default_icons',
				'hide_empty',
				'show_icon_array',
				'show_legend_text',
			),
			'slp_experience'       => array(
				'tag_autosubmit',
				'tag_show_any',
			),
			'slp_general'          => array(
				'highlight_uncoded',
				'use_contact_fields',
				'use_nonces',
				'use_pages',
				'use_sensor',
			),
			'slp_manage_locations' => array(
				'csv_skip_geocoding',
				'csv_clear_messages_on_import',
				'load_data',
			),
			'slp-pages'            => array(
				'default_comments',
				'default_trackbacks',
				'pages_replace_websites',
				'prevent_new_window',
				'prepend_permalink_blog',
			),
			'slp_reports'          => array(
				'reporting_enabled',
			),
		);

		protected $class_prefix = 'SLPPower_';
		protected $objects = array(
			'Admin_General'    => array( 'subdir' => 'include/module/admin_tabs/', 'object' => null,    'options' => array() ),
		);

		/**
		 * Add our SLP hooks and Filters for Admin Mode
		 */
		public function add_hooks_and_filters() {
			$this->js_pages = array( $this->slplus->admin_page_prefix . 'slp_manage_locations' );
			parent::add_hooks_and_filters();

			// Load objects based on which admin page we are on.
			//
			if ( isset( $_REQUEST['page'] ) ) {
				switch ( $_REQUEST['page'] ) {
					case 'slp_experience':
						$this->create_object_experience();
						break;
					case 'slp_general':
						$this->instantiate( 'Admin_General' );
						break;
					case 'slp_info':
						$this->create_object_info();
						break;
					case 'slp_manage_locations':
						$this->create_object_locations();

						add_action( 'slp_build_locations_panels', array( $this, 'add_location_addedit_settings' ), 30 );
						add_filter( 'slp_column_data', array( $this, 'add_column_data_to_manage_locations' ), 90, 3 );
						add_filter( 'slp_locations_manage_bulkactions', array(
							$this,
							'extend_manage_locations_bulk_actions',
						) );
						add_filter( 'slp_locations_manage_filters', array( $this, 'filter_LocationsFilters' ) );
						add_filter( 'slp_invalid_highlight', array( $this, 'filter_HighlightUncoded' ), 10, 1 );
						add_filter( 'slp_manage_expanded_location_columns', array(
							$this,
							'filter_AddFieldHeadersToManageLocations',
						) );

						$this->addon->create_object_category_manager();
						if ( $this->addon->category_manager->get_category_count() > 0 ) {
							add_filter( 'slp_manage_location_columns', array(
								$this,
								'filter_AddCategoriesHeaderToManageLocations',
							) );
						}

						break;
				}
			}

			// WP Taxonomy Edit for SLPlus::locationTaxonomy
			add_action( 'stores_add_form_fields', array( $this, 'add_slp_settings_to_wp_add_category' ) );
			add_action( 'stores_edit_form', array( $this, 'add_slp_settings_to_wp_edit_category' ) );

			// Admin skinning and scripts
			//
			add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

			// Location Added
			//
			add_action( 'slp_location_added', array( $this, 'assign_location_categories' ) );
			add_action( 'slp_location_saved', array( $this, 'assign_location_categories' ) );

			// Taxonomy Interface
			//
			if ( isset( $_REQUEST['taxonomy'] ) && ( $_REQUEST['taxonomy'] === SLPlus::locationTaxonomy ) ) {
				$this->create_object_admin_edit_tags();
				$this->edit_tags->add_wp_filters();
			}
		}

		/**
		 * Render the extra tagalong category fields for the add form.
		 */
		function add_slp_settings_to_wp_add_category() {
			$this->addon->create_object_category_manager();
			$this->addon->category_manager->render_ExtraCategoryFields();
		}

		/**
		 * Render the extra tagalong category fields for the edit form.
		 *
		 * @param WP_Term $tag
		 */
		function add_slp_settings_to_wp_edit_category( $tag ) {
			$this->addon->create_object_category_manager();
			print '<div id="tagalong_editform" class="form-wrap">';
			$this->addon->category_manager->render_ExtraCategoryFields( $tag->term_id, $this->addon->get_TermWithTagalongData( $tag->term_id ) );
			print '</div>';
		}

		/**
		 * Extend the edit location form to show store pages data.
		 *
		 * @param   SLP_Settings $settings
		 */
		public function add_location_addedit_settings( $settings ) {
			$current_action = $this->slplus->AdminUI->ManageLocations->current_action;
			$section_name   =
				( $current_action === 'edit' ) ?
					__( 'Edit', 'slp-power' ) :
					__( 'Add', 'slp-power' );
			$group_name     = $this->addon->name;

			$settings->add_ItemToGroup( array(
				'setting'     => 'tags-' . $this->slplus->currentLocation->id,
				'label'       => __( 'Tags', 'slp-power' ),
				'description' =>
					__( 'List of tags assigned to location. ', 'slp-power' ) .
					__( 'Separate with commas.', 'slp-power' ),
				'value'       => $this->slplus->currentLocation->tags,
				'data_field'  => 'tags',
				'use_prefix'  => false,
				'section'     => $section_name,
				'group'       => $group_name,
			) );

			// Only need to do this if some WP categories exist.
			//
			if ( $this->addon->category_manager->get_category_count() > 0 ) {
				$category_check_list = $this->get_CheckList( $this->slplus->currentLocation->linked_postid );

				if ( ! empty( $category_check_list ) ) {
					$settings->add_ItemToGroup( array(
						'label'   => __( 'Categories', 'slp-power' ),
						'type'    => 'subheader',
						'section' => $section_name,
						'group'   => $group_name,
					) );

					$categories_html = '<div id="slp_tagalong_fields" class="slp_editform_section">' .
					                   '<ul>' .
					                   $category_check_list .
					                   '</ul></div>';

					$settings->add_ItemToGroup( array(
						'setting'    => 'slp_tagalong_fields',
						'custom'     => $categories_html,
						'type'       => 'custom',
						'show_label' => false,
						'section'    => $section_name,
						'group'      => $group_name,
					) );
				}
			}

			// Submit
			$settings->add_ItemToGroup( array(
				'type'       => 'details',
				'custom'     => $this->slplus->AdminUI->ManageLocations->add->submit_button(),
				'show_label' => false,
				'section'    => $section_name,
				'group'      => $group_name,
			) );

			if ( $this->slplus->AdminUI->ManageLocations->current_action !== 'edit' ) {
				return;
			}
		}

		/**
		 * Assign categories to a given location via the data relation table.
		 *
		 */
		public function assign_location_categories() {
			$this->addon->create_object_category_manager();
			$this->addon->category_manager->set_categories_from_input();
			$this->addon->category_manager->update_location_category();
		}

		/**
		 * Create the edit tags object.
		 */
		private function create_object_admin_edit_tags() {
			if ( ! isset( $this->edit_tags ) ) {
				require_once( 'class.admin.edit-tags.php' );
				$this->edit_tags = new SLPPower_Admin_EditTags( array( 'addon' => $this->addon ) );
			}
		}

		/**
		 * Create and attach the \CSVExportLocations object
		 */
		public function create_object_admin_location_filters() {
			if ( ! isset( $this->admin_location_filters ) ) {
				require_once( 'class.admin.location.filters.php' );
				$this->admin_location_filters = new SLPPower_Admin_Location_Filters( array( 'addon' => $this->addon ) );
			}
		}

		/**
		 * Create and attach the admin experience object.
		 */
		private function create_object_experience() {
			if ( ! isset( $this->experience ) ) {
				require_once( 'class.admin.experience.php' );
				$this->experience = new SLPPower_Admin_ExperienceSettings( array( 'addon' => $this->addon ) );
			}
		}

		/**
		 * Create and attach the admin info object.
		 */
		private function create_object_info() {
			if ( ! isset( $this->info ) ) {
				require_once( 'class.admin.info.php' );
				$this->info = new SLPPower_Admin_Info( array( 'addon' => $this->addon ) );
			}
		}

		/**
		 * Create and attach the admin locations object.
		 */
		private function create_object_locations() {
			if ( ! isset( $this->locations ) ) {
				require_once( 'class.admin.locations.php' );
				$this->locations = new SLPPower_Admin_Locations( array( 'addon' => $this->addon ) );
			}
		}

		/**
		 * Create categories tab object.
		 */
		private function create_object_categories_tab() {
			if ( ! isset( $this->categories_tab ) ) {
				require_once( 'module/category/SLPPower_Admin_Categories.php' );
				$this->categories_tab = new SLPPower_Admin_Categories( array( 'addon' => $this->addon ) );
			}
		}

		/**
		 * Create and attach the admin processing object.
		 */
		function create_object_pages() {
			if ( ! $this->addon->using_pages ) {
				return;
			}
			if ( isset( $this->pages ) ) {
				return;
			}
			require_once( 'module/pages/class.admin.pages.php' );
			$this->pages = new SLPPower_Admin_Pages( array( 'addon' => $this->addon, 'admin' => $this ) );
		}

		/**
		 * Create the reports interface object and attach to this->reports
		 */
		function create_object_reports_tab() {
			if ( ! isset( $this->reports_tab ) ) {
				require_once( 'class.admin.reports.php' );
				$this->reports_tab = new SLPPower_Admin_Reports( array( 'addon' => $this->addon ) );
			}
		}

		/**
		 * Create the filter by categories div.
		 *
		 * @return string
		 */
		function createstring_FilterByCategoriesDiv() {
			$this->create_object_admin_location_filters();
			$HTML =
				'<div id="extra_filter_by_category" class="filter_extras">' .
				$this->admin_location_filters->createstring_LocationFilterForm() .
				'</div>';

			return $HTML;
		}

		/**
		 * Create the filter by properties div.
		 *
		 * @return string
		 */
		function createstring_FilterByPropertiesDiv() {
			$this->create_object_admin_location_filters();
			$HTML =
				"<iframe id='power_csv_download' src='' style='display:none; visibility:hidden;'></iframe>" .
				'<div id="extra_filter_by_property" class="filter_extras">' .
				$this->admin_location_filters->createstring_LocationFilterForm() .
				'</div>' .
				'<div id="slp-power_message_board" class="popup_message_div ui-dialog ui-widget ui-widget-content ui-corner-all ui-front ui-draggable ui-resizable">' .
				'<div class="ui-dialog-titlebar ui-widget-header">' .
				__( 'Location Processing Info', 'slp-power' ) .
				'</div>' .
				'<div id="slp-power_messages" class="ui-dialog-content ui-widget-content"></div>' .
				'</div>';

			return $HTML;
		}

		/**
		 * Save the Pro Pack options to the database in serialized format.
		 *
		 * Make sure the options are valid first.
		 *
		 * @param string[] $cbArray The checkbox array need to handle for the null to false operation
		 */
		function data_SaveOptions( $cbArray ) {
			$_POST[ $this->addon->option_name ] = array();
			array_walk( $_REQUEST, array( $this, 'set_ValidOptions' ) );

			// AdminUI->save_SerializedOption stores the $_POST[<option_name>] values plus the checkbox options (3rd param)
			// in the persistent store of wp_options
			//
			$this->slplus->AdminUI->save_SerializedOption( $this->addon->option_name, $this->addon->options, $cbArray );

			$this->addon->init_options();
		}

		/**
		 * Deactivate any plugins that this add-on replaces.
		 */
		private function deactivate_replaced_addons() {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			$this->slplus->createobject_AddOnManager();
			$replaced_addons = array(
				'slp-pro',
				'slp-tagalong',
				'slp-pages',
				'slp-contact-extender',
				'slp-directory-builder',
			);
			foreach ( $replaced_addons as $addon_slug ) {
				if ( $this->slplus->is_AddonActive( $addon_slug ) ) {
					deactivate_plugins( $this->slplus->add_ons->instances[ $addon_slug ]->file );
					$this->slplus->helper->add_wp_admin_notification(
						sprintf(
							__( 'The %s add-on deactivated the conflicting %s add-on. ', 'slp-power' ),
							$this->addon->name,
							$this->slplus->add_ons->instances[ $addon_slug ]->name
						)
					);
				}
			}
		}

		/**
		 * Execute some admin startup things for this add-on pack.
		 */
		function do_admin_startup() {
			$this->addon->create_object_category_data();
			$this->addon->create_object_import_messages();


			parent::do_admin_startup();

			$this->create_object_pages();

			$this->create_object_categories_tab();
			$this->create_object_reports_tab();

			$this->export_fileurl = SLPLUS_UPLOADURL . 'csv/exported_slp_locations.csv';
		}

		/**
		 * Enqueue the admin scripts.
		 *
		 * @param string $hook
		 */
		function enqueue_admin_scripts( $hook ) {
			$scriptData = array(
				'plugin_url' => SLPLUS_PLUGINURL,
			);

			switch ( $hook ) {
				case $this->slplus->admin_page_prefix . 'slp_reports':
					$scriptData['message_nodata']           =
						__( 'No data recorded yet. ', 'slp-power' );
					$scriptData['message_chartaftersearch'] =
						__( 'Chart will be available after a Store Locator Plus search has been performed.', 'slp-power' );
					$scriptData['total_searches']           = $this->reports_tab->data->total_searches;
					$scriptData['count_dataset']            = $this->reports_tab->data->counts_dataset;
					$scriptData['chart_type']               = $this->reports_tab->data->google_chart_type;

					wp_enqueue_script( 'google_jsapi', 'https://www.google.com/jsapi' );
					wp_enqueue_script( 'jquery_tablesorter', $this->addon->url . '/js/jquery.tablesorter.js', array( 'jquery' ) );

					wp_enqueue_script( 'slp_reporting', $this->addon->url . '/js/reporting.js' );
					wp_localize_script( 'slp_reporting', 'slp_power', $scriptData );

					break;

				case $this->slplus->admin_page_prefix . 'slp_manage_locations':
					$manage_locations_vars = array_merge(
						$_REQUEST,
						array(
							'download_file_message' => sprintf( __( 'Download your %s', 'slp-power' ), sprintf( '<a href="%s">%s</a>', $this->export_fileurl, __( 'locations CSV file.', 'slp-power' ) ) ),
							'text_uploading'        => __( 'Uploading', 'slp-power' ),
							'text_loading'          => __( 'Loading locations from ', 'slp-power' ),
							'text_loaded'           => __( 'Finished loading ', 'slp-power' ),
						)
					);
					wp_register_script( 'slp_power_manage_locations', $this->addon->url . '/js/manage_locations.js', array(
						'jquery',
						'jquery-ui-progressbar',
						'plupload-all',
					) );
					wp_localize_script( 'slp_power_manage_locations', 'request_vars', $manage_locations_vars );
					wp_enqueue_script( 'slp_power_manage_locations' );

					break;

				default:
					break;
			}
		}

		/**
		 * Add the categories column header to the manage locations table.
		 *
		 * SLP Filter: slp_manage_location_columns
		 *
		 * @param mixed[] $currentCols column name + column label for existing items
		 *
		 * @return mixed[] column name + column labels, extended with our categories data
		 */
		function filter_AddCategoriesHeaderToManageLocations( $currentCols ) {
			return array_merge( $currentCols,
				array(
					'sl_option_value' => __( 'Categories', 'slp-power' ),
				)
			);

		}

		/**
		 * Render the categories column in the manage locations table.
		 *
		 * SLP Filter: slp_column_data
		 *
		 * @param string $data  the option_value field data from the database
		 * @param string $field the name of the field from the database (should be sl_option_value)
		 * @param string $label the column label for this column (should be 'Categories')
		 *
		 * @return string
		 */
		function add_column_data_to_manage_locations( $data, $field, $label ) {
			if (
				( $this->addon->category_manager->get_category_count() > 0 ) &&
				( $field === 'sl_option_value' ) &&
				( $label === __( 'Categories', 'slp-power' ) )
			) {
				$data = $this->addon->createstring_IconArray( array( 'show_label' => true, 'add_edit_link' => true ) );
			}

			if ( $field === 'sl_tags' ) {
				$data = ( $this->slplus->currentLocation->tags != '' ) ?
					$this->slplus->currentLocation->tags :
					"";
			}

			return $data;
		}

		/**
		 * Add the images column header to the manage locations table.
		 *
		 * SLP Filter: slp_manage_location_columns
		 *
		 * @param mixed[] $currentCols column name + column label for existing items
		 *
		 * @return mixed[] column name + column labels, extended with our extra fields data
		 */
		function filter_AddFieldHeadersToManageLocations( $currentCols ) {
			return array_merge( $currentCols,
				array(
					'sl_tags' => __( 'Tags', 'slp-power' ),
				)
			);
		}

		/**
		 * Add our admin pages to the valid admin page slugs.
		 *
		 * @used-by SLPlus_AdminUI->is_our_admin_page()
		 *
		 * @param string[] $slugs admin page slugs
		 *
		 * @return string[] modified list of admin page slugs
		 */
		function filter_AddOurAdminSlug( $slugs ) {
			$slugs = parent::filter_AddOurAdminSlug( $slugs );
			foreach ( $this->settings_pages as $slug => $checkbox_array ) {
				if ( ! in_array( $slug, $slugs ) ) {
					$slugs[] = $slug;
					$slugs[] = $this->slplus->admin_page_prefix . $slug;
				}
			}
			if ( isset( $_REQUEST['taxonomy'] ) && ( $_REQUEST['taxonomy'] === 'stores' ) ) {
				$slugs[] = 'edit-tags.php';
				$slugs[] = 'edit_tags-stores';
				$slugs[] = 'term.php';
			}

			return $slugs;
		}

		/**
		 * Remove the invalid class from non-geocoded locations if highlight uncoded is off.
		 *
		 * @param $class_string
		 *
		 * @return mixed
		 */
		function filter_HighlightUncoded( $class_string ) {
			if ( ! $this->slplus->is_CheckTrue( $this->addon->options['highlight_uncoded'] ) ) {
				$class_string = str_replace( ' invalid', '', $class_string );
			}

			return $class_string;
		}

		/**
		 * Add more actions to the Bulk Action drop down on the admin Locations/Manage Locations interface.
		 *
		 * @param mixed[] $items
		 *
		 * @return mixed[]
		 */
		function extend_manage_locations_bulk_actions( $items ) {
			$power_actions = array();

			// Only add categorize if categories exist.
			// TODO: the category checklist should be pulled by AJAX
			//
			if ( $this->addon->category_manager->get_category_count() > 0 ) {
				$power_actions[] = array(
					'label'  => __( 'Categorize', 'slp-power' ),
					'value'  => 'categorize',
					'extras' =>
						'<script type="text/javascript">' .
						'var hiddenInputs=new Array();' .
						"jQuery(document).ready(function(){" .
						"jQuery('#slp_tagalong_fields input:checkbox').change(function(){" .
						'theCats="";' .
						'hiddenInputs=new Array();' .
						'jQuery("#slp_tagalong_fields li").' .
						'find("input[type=checkbox]:checked" ).' .
						'each(function(idx) {' .
						'theCats += jQuery(this).parent().text() + ", ";' .
						'hiddenInputs.push("<input type=\'hidden\' name=\'tax_input[stores][]\' value=\'"+jQuery(this).val()+"\'/>");' .
						'})' .
						';' .
						"});" .
						"});" .
						'</script>' .
						'<div id="extra_categorize" class="bulk_extras">' .
						__( 'Select your categories: ', 'slp-power' ) .
						'<div id="slp_tagalong_selector">' .
						'<div id="slp_tagalong_fields" class="slp_editform_section">' .
						'<ul>' .
						$this->get_CheckList( 0 ) .
						'</ul>' .
						'</div>' .
						'</div>' .
						'</div>',
				);
			}

			$power_actions[] = array(
				'label' => __( 'Export, Download CSV', 'slp-power' ),
				'value' => 'export',
			);
			$power_actions[] = array(
				'label' => __( 'Export, Hosted CSV', 'slp-power' ),
				'value' => 'export_local',
			);
			$power_actions[] = array(
				'label' => __( 'Geocode Selected', 'slp-power' ),
				'value' => 'recode',
			);
			$power_actions[] = array(
				'label' => __( 'Geocode All Uncoded', 'slp-power' ),
				'value' => 'recode_all',
			);
			$power_actions[] = array(
				'label'  => __( 'Tag', 'slp-power' ),
				'value'  => 'add_tag',
				'extras' =>
					'<div id="extra_add_tag" class="bulk_extras">' .
					'<label for="sl_tags">' . __( 'Enter your comma-separated tags:', 'slp-power' ) . '</label>' .
					'<input name="sl_tags">' .
					'</div>',
			);
			$power_actions[] = array(
				'label' => __( 'Tag, Remove', 'slp-power' ),
				'value' => 'remove_tag',
			);

			return array_merge( $items, $power_actions );
		}

		/**
		 * Add more actions to the Bulk Action drop down on the admin Locations/Manage Locations interface.
		 *
		 * @param mixed[] $items
		 *
		 * @return mixed[]
		 */
		function filter_LocationsFilters( $items ) {
			$power_filters   = array();
			$power_filters[] = array(
				'label' => __( 'Show Uncoded', 'slp-power' ),
				'value' => 'show_uncoded',
			);
			$power_filters[] = array(
				'label'  => __( 'With These Properties', 'slp-power' ),
				'value'  => 'filter_by_property',
				'extras' => $this->createstring_FilterByPropertiesDiv(),
			);

			// Only add categorize if categories exist.
			//
			if ( $this->addon->category_manager->get_category_count() > 0 ) {
				$power_filters[] = array(
					'label'  => __( 'In These Categories', 'slp-power' ),
					'value'  => 'filter_by_category',
					'extras' => $this->createstring_FilterByCategoriesDiv(),
				);
			}

			return array_merge( $items, $power_filters );
		}

		/**
		 * Generate the categories check list.
		 *
		 * @param int $postID the ID for the store page related to the location.
		 *
		 * @return string HTML of the checklist.
		 */
		public function get_CheckList( $post_id ) {
			return wp_terms_checklist(
				intval( $post_id ),
				array(
					'checked_ontop' => false,
					'taxonomy'      => 'stores',
					'echo'          => false,
				)
			);
		}

		/**
		 * Extend base class save_my_settings to check if contact fields have been added.
		 *
		 * Triggered via do_admin_startup when an update action is found.
		 */
		function save_my_settings() {
			$use_contact_fields_before_save = $this->addon->options['use_contact_fields'];

			parent::save_my_settings();

			if (
				( $use_contact_fields_before_save !== $this->addon->options['use_contact_fields'] ) &&
				$this->slplus->is_CheckTrue( $this->addon->options['use_contact_fields'] )
			) {
				require_once( 'class.activation.php' );
				$activation = new SLPPower_Activation( array( 'addon' => $this->addon ) );
				$activation->add_data_extensions();
				$this->slplus->notifications->add_notice( 'info', __( 'Extended contact fields have been activated.', 'slp-power' ) );
			}
		}


		/**
		 * Our default object options.
		 */
		protected function set_default_object_options() {
			$this->objects[ 'Admin_General'    ][ 'options' ] = array( 'addon' => $this->addon , 'admin' => $this );
		}

		/**
		 * Set the Tagalong categories for the new store page.
		 *
		 * SLP Filter: slp_location_page_attributes
		 *
		 * @param mixed[] $pageAttributes - the wp_insert_post page attributes
		 *
		 * @return mixed[] - pageAttributes with tax_input set
		 */
		function set_taxonomy_for_location( $pageAttributes ) {
			$this->addon->category_manager->set_categories_from_input();

			return $this->addon->category_manager->set_page_taxonomy( $pageAttributes );
		}

		/**
		 * Set valid options from the incoming REQUEST
		 *
		 * @param mixed  $val - the value of a form var
		 * @param string $key - the key for that form var
		 */
		function set_ValidOptions( $val, $key ) {
			$simpleKey = str_replace( $this->slplus->prefix . '-', '', $key );

			if ( array_key_exists( $simpleKey, $this->addon->options ) ) {
				$_POST[ $this->addon->option_name ][ $simpleKey ] = stripslashes_deep( $val );
			}
		}

		/**
		 * Deactivate the competing add-on packs.
		 */
		function update_install_info() {
			parent::update_install_info();
			$this->deactivate_replaced_addons();
		}
	}
}
