<?php
if ( !class_exists( 'SLPPower_AJAX' ) ) {
    require_once( SLPLUS_PLUGINDIR . '/include/base_class.ajax.php' );


    /**
     * Holds the ajax-only code.
     *
     * This allows the main plugin to only include this file in AJAX mode
     * via the slp_init when DOING_AJAX is true.
     *
     * @package   StoreLocatorPlus\SLPPro\AJAX
     * @author    Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * text-domain: slp-power
     *
     * @property        SLPPower                        $addon
     * @property        SLPPower_Category_Data          $category_data   The data helper object.
     * @property-read   SLPPower_AJAX_Location_Manager  $location_manager
     *
     */
    class SLPPower_AJAX extends SLP_BaseClass_AJAX {
        public $category_data;
	    private $location_manager;
        public $query_params_valid = array( 'options' );
        private $upload_transient = 'slp_uploads_path';
        public $valid_actions = array(
            'csl_ajax_onload',
            'csl_ajax_search',
            'slp_background_location_download',
            'slp_change_option',
            'slp_clear_import_messages',
            'slp_download_report_csv',
            'slp_download_locations_csv',
	        'slp_get_country_list',
	        'slp_get_state_list',
            'slp_new_upload',
        );

        /**
         * Add our specific AJAX filters.
         */
        function add_ajax_hooks() {
            $this->addon->create_object_category_data();
            $this->category_data = $this->addon->category_data;

            add_filter( 'slp_ajaxsql_where', array( $this, 'filter_JSONP_SearchFilters' ), 20 );
            add_filter( 'slp_location_filters_for_AJAX', array( $this, 'filter_JSONP_SearchByCategory' ) );
            add_filter( 'slp_location_having_filters_for_AJAX', array( $this, 'filter_AJAX_AddHavingClause' ), 55 );
            add_filter( 'slp_location_filters_for_AJAX', array( $this, 'createstring_TagSelectionWhereClause' ) );
            add_action( 'slp_orderby_default', array( $this, 'add_category_count_to_orderby' ), 5 );
            add_filter( 'slp_results_marker_data', array( $this, 'modify_ajax_markers' ) );

            add_action( 'wp_ajax_slp_background_location_download', array( $this, 'location_download_in_background' ) );
            add_action( 'wp_ajax_slp_download_report_csv', array( $this, 'download_report_csv' ) );
            add_action( 'wp_ajax_slp_download_locations_csv', array( $this, 'download_locations_csv' ) );

	        add_action( 'wp_ajax_slp_get_country_list', array( $this, 'get_country_list' ) );
	        add_action( 'wp_ajax_slp_get_state_list', array( $this, 'get_state_list' ) );

            add_action( 'wp_ajax_slp_new_upload', array( $this, 'process_new_upload' ) );

            if ( $this->slplus->is_CheckTrue( $this->addon->options[ 'reporting_enabled' ] ) ) {
                add_action( 'slp_report_query_result', array( $this, 'log_search_queries_and_results' ), 10, 2 );
            }

        }

        /**
         * Change the results order.
         *
         * Precedence is given to the order by category count option over all other extensions that came before it.
         * This is enacted by placing the special category count clause as the first parameter of extend_OrderBy,
         * and by setting the filter to a high priority (run last).
         *
         */
        function add_category_count_to_orderby() {
            if ( empty( $this->addon->options[ 'ajax_orderby_catcount' ] ) ) {
                return;
            }
            $this->slplus->database->extend_order_array( '(' . $this->category_data->get_SQL( 'select_categorycount_for_location' ) . ') DESC ' );
        }

        /**
         * Clean the $_REQUEST array of some things we don't want to track.
         *
         * @return array[string]string
         */
        private function clean_request() {
            $clean_request = $_REQUEST;

            unset( $clean_request[ 'options' ][ 'bubblelayout' ] );

            // Remove all label* options
            foreach ( $clean_request[ 'options' ] as $key => $value ) {
                if ( strpos( $key, 'label' ) === 0 ) {
                    unset( $clean_request[ 'options' ][ $key ] );
                }
            }

            return $clean_request;
        }

	    /**
	     * Attach the location_manager object.
	     */
	    private function create_location_manager() {
		    if ( ! isset( $this->location_manager ) ) {
			    require_once( 'module/ajax/SLPPower_AJAX_Location_Manager.php' );
			    $this->location_manager = new  SLPPower_AJAX_Location_Manager( array( 'ajax' => $this ) );
		    }
	    }

        /**
         * Add the tags condition to the MySQL statement used to fetch locations with JSONP.
         *
         * @param string[] $currentFilters
         * @return string[]
         */
        public function createstring_TagSelectionWhereClause( $currentFilters ) {
            if ( !isset( $_POST[ 'tags' ] ) || ( $_POST[ 'tags' ] == '' ) ) {
                return $currentFilters;
            }

            $posted_tag = preg_replace( '/^\s+(.*?)/', '$1', $_POST[ 'tags' ] );
            $posted_tag = preg_replace( '/(.*?)\s+$/', '$1', $posted_tag );

            return array_merge(
                $currentFilters, array( " AND ( sl_tags LIKE '%%" . $posted_tag . "%%') " )
            );
        }


        /**
         * Process incoming AJAX request to download the CSV file.
         * TODO: use locations extended class
         */
        function download_locations_csv() {
            $this->slplus->set_php_timeout();


            $this->addon->create_CSVLocationExporter();
            $this->addon->csvExporter->do_SendFile();
        }


        /**
         * Process incoming AJAX request to download the CSV file.
         */
        function download_report_csv() {
            $this->slplus->set_php_timeout();


            // CSV Header
            header( 'Content-Description: File Transfer' );
            header( 'Content-Disposition: attachment; filename=slplus_' . $_REQUEST[ 'filename' ] . '.csv' );
            header( 'Content-Type: application/csv;' );
            header( 'Pragma: no-cache' );
            header( 'Expires: 0' );

            // Setup our processing vars
            //
            global $wpdb;
            $query = $_REQUEST[ 'query' ];

            // All records - revise query
            //
            if ( isset( $_REQUEST[ 'all' ] ) && ( $_REQUEST[ 'all' ] == 'true' ) ) {
                $query = preg_replace( '/\s+LIMIT \d+(\s+|$)/', '', $query );
            }

            $slpQueryTable = $wpdb->prefix . 'slp_rep_query';
            $slpResultsTable = $wpdb->prefix . 'slp_rep_query_results';
            $slpLocationsTable = $wpdb->prefix . 'store_locator';

            $expr = "/,(?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))/";
            $parts = preg_split( $expr, trim( html_entity_decode( $query, ENT_QUOTES ) ) );
            $parts = preg_replace( "/^\"(.*)\"$/", "$1", $parts );

            // Return the address in CSV format from the reports
            //
            if ( $parts[ 0 ] === 'addr' ) {
                $slpReportStartDate = $parts[ 1 ];
                $slpReportEndDate = $parts[ 2 ];

                // Only Digits Here Please
                //
                $slpReportLimit = preg_replace( '/[^0-9]/', '', $parts[ 3 ] );

                $query = "SELECT slp_repq_address, count(*)  as QueryCount FROM $slpQueryTable " .
                    "WHERE slp_repq_time > %s AND " .
                    "      slp_repq_time <= %s " .
                    "GROUP BY slp_repq_address " .
                    "ORDER BY QueryCount DESC " .
                    "LIMIT %d";
                $queryParms = array(
                    $slpReportStartDate,
                    $slpReportEndDate,
                    $slpReportLimit,
                );

                // Return the locations searches in CSV format from the reports
                //
            } else if ( $parts[ 0 ] === 'top' ) {
                $slpReportStartDate = $parts[ 1 ];
                $slpReportEndDate = $parts[ 2 ];

                // Only Digits Here Please
                //
                $slpReportLimit = preg_replace( '/[^0-9]/', '', $parts[ 3 ] );

                $query = "SELECT sl_store,sl_city,sl_state, sl_zip, sl_tags, count(*) as ResultCount " .
                    "FROM $slpResultsTable res " .
                    "LEFT JOIN $slpLocationsTable sl " .
                    "ON (res.sl_id = sl.sl_id) " .
                    "LEFT JOIN $slpQueryTable qry " .
                    "ON (res.slp_repq_id = qry.slp_repq_id) " .
                    "WHERE slp_repq_time > %s AND slp_repq_time <= %s " .
                    "GROUP BY sl_store,sl_city,sl_state,sl_zip,sl_tags " .
                    "ORDER BY ResultCount DESC " .
                    "LIMIT %d";
                $queryParms = array(
                    $slpReportStartDate,
                    $slpReportEndDate,
                    $slpReportLimit,
                );

                // Not Locations (top) or addresses entered in search
                // short circuit...
                //
            } else {
                die( __( "Cheatin' huh!", 'slp-power' ) );
            }

            // No parms array?  GTFO
            //
            if ( !is_array( $queryParms ) ) {
                die( __( "Cheatin' huh!", 'slp-power' ) );
            }


            // Run the query & output the data in a CSV
            $thisDataset = $wpdb->get_results( $wpdb->prepare( $query, $queryParms ), ARRAY_N );


            // Sorting
            // The sort comes in based on the display table column order which
            // matches the query output column order listed here.
            //
            // It is a paired array, first number is the column number (zero offset)
            // second number is the sort order [0=ascending, 1=descending]
            //
            // The sort needs to happen AFTER the select.
            //

            // Get our sort array
            //
            $thisSort = explode( ',', $_REQUEST[ 'sort' ] );

            // Build our array_multisort command and our sort index/sort order arrays
            // we will need this later for helping do a multi-dimensional sort
            //
            $sob = 'sort';
            $amsstring = '';
            $sortarrayindex = 0;
            foreach ( $thisSort as $sl_value ) {
                if ( $sob == 'sort' ) {
                    $sort[] = $sl_value;
                    $amsstring .= '$s[' . $sortarrayindex++ . '], ';
                    $sob = 'order';
                } else {
                    $order[] = $sl_value;
                    $amsstring .= ( $sl_value == 0 ) ? 'SORT_ASC, ' : 'SORT_DESC, ';
                    $sob = 'sort';
                }
            }
            $amsstring .= '$thisDataset';

            // Now that we have our sort arrays and commands,
            // build the indexes that will be used to do the
            // multi-dimensional sort
            //
            foreach ( $thisDataset as $key => $row ) {
                $sortarrayindex = 0;
                foreach ( $sort as $column ) {
                    $s[ $sortarrayindex++ ][ $key ] = $row[ $column ];
                }
            }

            // Now do the multidimensional sort
            //
            // This will sort using the first array ($s[0] we built in the above 2 steps)
            // to determine what order to put the "records" (the outter array $thisDataSet)
            // into.
            //
            // If there are secondary arrays ($s[1..n] as built above) we then further
            // refine the sort using these secondary arrays.  Think of them as the 2nd
            // through nth columns in a multi-column sort on a spreadsheet.
            //
            // This exactly mimics the jQuery sorts that manage our tables on the HTML
            // page.
            //

            //array_multisort($amsstring);
            // Output the sorted CSV strings
            // This simply iterates through our newly sorted array of records we
            // got from the DB and writes them out in CSV format for download.
            //
            foreach ( $thisDataset as $thisDatapoint ) {
                print SLPPro::array_to_CSV( $thisDatapoint );
            }

            // Get outta here
            die();
        }

        /**
         * Add to the AJAX having clause
         *
         * @param mixed[] having clause array
         * @return mixed[]
         */
        function filter_AJAX_AddHavingClause( $clauseArray ) {

            // Ignore Radius Is On
            //
            if ( $this->slplus->options[ 'ignore_radius' ] === '1' ) {
                array_push( $clauseArray, ' OR (sl_distance > 0) ' );
            }

            return $clauseArray;
        }

        /**
         * Add the category condition to the MySQL statement used to fetch locations with JSONP.
         *
         * @param string $currentFilters
         * @return string
         */
        function filter_JSONP_SearchByCategory( $currentFilters ) {
            if ( empty( $this->formdata ) ) {
                return $currentFilters;
            }
            if ( !isset( $this->formdata[ 'cat' ] ) ) {
                return $currentFilters;
            }
            if ( !is_array( $this->formdata[ 'cat' ] ) && ( $this->formdata[ 'cat' ] <= 0 ) ) {
                return $currentFilters;
            }

            $this->addon->create_object_category_data();

            // Single Category
            //
            if ( !is_array( $this->formdata[ 'cat' ] ) ) {
                $sql_select_stores_in_cats = sprintf( 'AND ' . $this->addon->category_data->get_SQL( 'where_location_has_category' ), $this->formdata[ 'cat' ] );

                // Array Of Categories
            } else {
                $sql_select_stores_in_cats = '';
                foreach ( $this->formdata[ 'cat' ] as $term_id ) {
                    if ( $term_id > 0 ) {
                        $sql_select_stores_in_cats .= sprintf( 'AND ' . $this->addon->category_data->get_SQL( 'where_location_has_category' ), $term_id );
                    }
                }
            }

            // Setup and clause to select stores by a specific category
            //
            return array_merge( $currentFilters, array( $sql_select_stores_in_cats ) );
        }

        /**
         * Add the selected filters to the search results.
         *
         */
        function filter_JSONP_SearchFilters( $where ) {
            if ( !isset( $this->slplus->AjaxHandler ) ) {
                return $where;
            }

            $ajax_options = $this->addon->options;

            foreach ( $this->addon->location_fields as $field => $shorthand ) {
                if (
                    !empty( $ajax_options[ $shorthand ] ) &&
                    ( $ajax_options[ $shorthand . '_selector' ] === 'hidden' )
                ) {
                    return $this->slplus->database->extend_WhereFieldMatches( $where, $field, $ajax_options[ $shorthand ] );
                }

            }

            return $where;
        }

	    /**
	     * Return a list of current countries
	     */
	    public function get_country_list() {
		    $this->create_location_manager();
		    $this->location_manager->get_country_list();
	    }

	    /**
	     * Return a list of current states
	     */
        public function get_state_list() {
        	$this->create_location_manager();
	        $this->location_manager->get_state_list();
        }

        /**
         * Log the search query and results into the reporting tables.
         *
         * <code>
         * $query_params['QUERY_STRING'] => 'the query string';
         * $query_params['tags'] => 'tags,used,for,this,query';
         * $query_params['address'] => 'address for, the search';
         * $query_params['radius'] => 'radius_of_search';
         * </code>
         *
         * @param          array [string]string Contain query sql, tags, address and radius
         * @param string[] Query result row id (integers) array.
         */
        function log_search_queries_and_results( $query_params, $results ) {
            $inserted_query_id = $this->log_search_query( $query_params );
            $this->log_search_results( $results, $inserted_query_id );
        }

        /**
         * Log the search query that was used.
         *
         * <code>
         * $query_params['QUERY_STRING'] => 'the query string';
         * $query_params['tags'] => 'tags,used,for,this,query';
         * $query_params['address'] => 'address for, the search';
         * $query_params['radius'] => 'radius_of_search';
         * </code>
         *
         * @param array [string]string Contain query sql, tags, address and radius
         *
         * @return int the insert ID for this record.
         */
        private function log_search_query( $query_params ) {

            $this->slplus->db->insert(
                "{$this->slplus->db->prefix}slp_rep_query",
                array(
                    'slp_repq_query'   => $query_params[ 'QUERY_STRING' ],
                    'slp_repq_tags'    => $query_params[ 'tags' ],
                    'slp_repq_address' => $query_params[ 'address' ],
                    'slp_repq_radius'  => $query_params[ 'radius' ],
                    'meta_value'       => serialize( array( 'REQUEST' => $this->clean_request(), 'SERVER' => $_SERVER ) ),
                ),
                '%s'
            );

            return $this->slplus->db->insert_id;
        }

        /**
         * Log the search results that were returned.
         *
         * @param string[] $results
         * @param int      $inserted_query_id
         */
        private function log_search_results( $results, $inserted_query_id ) {
            foreach ( $results as $row_id ) {
                $this->slplus->db->insert(
                    "{$this->slplus->db->prefix}slp_rep_query_results",
                    array(
                        'slp_repq_id' => $inserted_query_id,
                        'sl_id'       => $row_id,
                    ),
                    '%d'
                );
            }
        }

        /**
         * Start the process to download the locations in the background.
         */
        function location_download_in_background() {

            // TODO: Fire off the process to get the CSV written to disk.
            // Do this is an AJAX post from here?
            //

            // Tell the user the CSV creation process has started.
            //
            die(
            json_encode(
                array(
                    'message' => __( 'Creating the location export CSV file.', 'slp-power' ),
                )
            )
            );
        }

        /**
         * Modify the marker data.
         *
         * @param array [string]string $marker the current marker data
         * @return array[string]string
         */
        function modify_ajax_markers( $marker ) {

			$marker = $this->set_contact_fields( $marker );

            $marker = $this->set_location_tags( $marker );

            $marker = $this->set_location_pages_properties( $marker );

            $marker = $this->set_location_category_properties( $marker );

            return $marker;
        }

        /**
         * Clear the import messages transient.
         *
         * Runs via do_ajax_startup() in the base class.
         */
        function process_slp_clear_import_messages() {
        	$this->addon->create_CSVLocationImporter();
	        $this->addon->csvImporter->messages->clear_messages();
            die( 'ok' );
        }

        /**
         * New upload.
         */
        function process_new_upload() {
            $this->upload();
            die( '{"OK": 1}' );
        }

	    /**
	     * Set the special contact fields to be returned by AJAX for this location.
	     *
	     * @param  array $marker
	     *
	     * @return array
	     */
	    private function set_contact_fields( $marker ) {
		    if ( ! empty( $marker[ 'data' ][ 'contact_image' ] ) ) {
			    $contact_image_html = sprintf( '<img src="%s" alt="%s">', $marker[ 'data' ][ 'contact_image' ], $marker[ 'data' ][ 'contact' ] );
		    }  else {
			    $contact_image_html = '';
		    }

		    return array_merge(
			    $marker,
			    array(
				    'contact_image_html'     => $contact_image_html,
			    )
		    );
	    }

        /**
         * Set the location marker based on categories.
         */
        private function set_LocationMarker() {

            $locationMarker = '';

            // Location Marker from Tagalong
            //
            if (
                ( !$this->slplus->is_CheckTrue( $this->addon->options[ 'default_icons' ] ) ) &&
                ( count( $this->addon->current_location_categories ) > 0 )
            ) {

                $best_rank = 999999;
                foreach ( $this->addon->current_location_categories as $term_id ) {
                    $category_details = $this->addon->get_TermWithTagalongData( $term_id );
                    if
                    (
                        ( (int)$category_details[ 'rank' ] < $best_rank ) &&
                        isset( $category_details[ 'map-marker' ] )
                    ) {
                        $best_rank = (int)$category_details[ 'rank' ];
                        $locationMarker = $category_details[ 'map-marker' ];
                    }
                }

                // No category marker - use the default marker.
                //
            } else {
                $locationMarker = ( isset( $mapData[ 'attributes' ][ 'marker' ] ) ? $mapData[ 'attributes' ][ 'marker' ] : '' );
            }

            return $locationMarker;
        }

        /**
         * Set the category properties to be returned by AJAX for this location.
         *
         * @param  array $marker
         *
         * @return array
         */
        private function set_location_category_properties( $marker ) {
            if ( ! $this->slplus->currentLocation->isvalid_ID( $marker[ 'id' ] ) ) {
                return $marker;
            }

            $this->addon->set_LocationCategories();

            // If we are looking for a specific category,
            // check to see if it is assigned to this location
            // Category searched for not in array, Skip this one.
            //
            //
            $filterOut = isset( $_POST[ 'formflds' ] ) && isset( $_POST[ 'formflds' ][ 'cat' ] ) && ( $_POST[ 'formflds' ][ 'cat' ] > 0 );
            if ( $filterOut ) {
                $selectedCat = (int)$_POST[ 'formflds' ][ 'cat' ];
                if ( !in_array( $selectedCat, $this->addon->current_location_categories ) ) {
                    return array();
                }
            }

            // Category Count
            //
            $category_count = count( $this->addon->current_location_categories );

            // Category Details
            // If category details is enabled (on by default), return them in the AJAX string.
            //
            $category_names = '';
            if ( $category_count > 0 ) {
                $category_name_array = array();
                foreach ( $this->addon->current_location_categories as $term_id ) {
                    $category_info = $this->addon->get_TermWithTagalongData( $term_id );
                    $category_name_array[] = isset( $category_info[ 'name' ] ) ? $category_info[ 'name' ] : '';
                }

                /**
                 * FILTER: slp_category_name_separator
                 * Change the default ', ' separator used between category names.
                 *
                 * @param   string  the character string to be used between each category on the category name results output
                 *
                 * @return string   the character string to be used between each category on the category name results output
                 */
                $category_name_separator = apply_filters( 'slp_category_name_separator', ', ' );
                $category_names = implode( $category_name_separator, $category_name_array );
            }

            // Return our modified array
            //
            return array_merge(
                $marker,
                array(
                    'attributes'     => $this->slplus->currentLocation->attributes,
                    'categories'     => $this->addon->current_location_categories,
                    'category_count' => $category_count,
                    'category_names' => $category_names,
                    'icon'           => $this->set_LocationMarker(),
                    'iconarray'      => $this->addon->createstring_IconArray(),
                )
            );
        }

        /**
         * Modify the marker if pages is active.
         *
         * @param   array $marker
         *
         * @return  array
         */
        private function set_location_pages_properties( $marker ) {
            if ( ! $this->slplus->is_CheckTrue( $this->addon->options['use_pages'] ) ) {
                return $marker;
            }

            // Pages SEO processing
            //
            $use_pages_link = $this->slplus->is_CheckTrue( $this->addon->options[ 'pages_replace_websites' ] );
            $use_same_window = $this->slplus->is_CheckTrue( $this->addon->options[ 'prevent_new_window' ] );

            $marker[ 'sl_pages_url' ] = $this->set_pages_url( $marker );

            // No using pages link, just update the target window and leave.
            //
            if ( !$use_pages_link ) {
                if ( $use_same_window ) {
                    $marker[ 'web_link' ] = str_replace( "target='_blank'", "target='_self'", $marker[ 'web_link' ] );
                    $marker[ 'url_link' ] = str_replace( "target='_blank'", "target='_self'", $marker[ 'url_link' ] );
                }

                return $marker;
            }

            // Using Pages Link and a public page exists, create link.
            //
            if ( $use_pages_link && ( !empty( $marker[ 'sl_pages_url' ] ) ) ) {
                $marker[ 'url' ] = $marker[ 'sl_pages_url' ];
                $marker[ 'web_link' ] =
                    sprintf(
                        "<a href='%s' target='%s' class='storelocatorlink'>%s</a><br/>",
                        $marker[ 'url' ],
                        ( $use_same_window ) ? '_self' : '_blank',
                        $this->slplus->WPML->get_text( 'label_website' )
                    );
                $marker[ 'url_link' ] =
                    sprintf(
                        "<a href='%s' target='%s' class='storelocatorlink'>%s</a><br/>",
                        $marker[ 'url' ],
                        ( $use_same_window ) ? '_self' : '_blank',
                        $marker[ 'url' ]
                    );

            } else {
                $marker[ 'url' ] = '';
                $marker[ 'web_link' ] = '';
                $marker[ 'url_link' ] = '';
            }

            return $marker;
        }

        /**
         * Set the tags property to be returned by AJAX for this location.
         *
         * @param array $marker
         * @return array mixed
         */
        private function set_location_tags( $marker ) {
            switch ( $this->addon->options[ 'tag_output_processing' ] ) {
                case 'hide':
                    $marker[ 'tags' ] = '';
                    break;

                case 'replace_with_br':
                    $marker[ 'tags' ] = str_replace( ',', '<br/>', $marker[ 'tags' ] );
                    $marker[ 'tags' ] = str_replace( '&#044;', '<br/>', $marker[ 'tags' ] );
                    break;

                case 'as_entered':
                default:
                    break;
            }
            return $marker;
        }

        /**
         * Set the pages URL for the given location, only if the page status is PUBLISH.
         *
         * @param array [string]string $marker
         * @return string
         */
        private function set_pages_url( $marker ) {
        	$url = '';

            if (
            	! empty( $marker[ 'sl_pages_url' ] ) &&
	            ( get_post_status( $this->slplus->currentLocation->linked_postid ) === 'publish' )
            ) {
            	$url = $this->slplus->currentLocation->pages_url;
            }

            return apply_filters( 'slp_pages_url' , $url );
        }


        /**
         * upload.php
         *
         * Copyright 2009, Moxiecode Systems AB
         * Released under GPL License.
         *
         * License: http://www.plupload.com/license
         * Contributing: http://www.plupload.com/contributing
         */
        private function upload() {

//            if ( ! check_ajax_referer( 'wp_all_import_secure', '_wpnonce', false )){
//                exit(json_encode(array("jsonrpc" => "2.0", "error" => array("code" => 100, "message" => __('Security check', 'wp_all_import_plugin')), "id" => "id")));
//            }

            // HTTP headers for no cache etc
            // header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
            // header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
            // header("Cache-Control: no-store, no-cache, must-revalidate");
            // header("Cache-Control: post-check=0, pre-check=0", false);
            // header("Pragma: no-cache");

            // Settings
            //$targetDir = ini_get("upload_tmp_dir") . DIRECTORY_SEPARATOR . "plupload";
            //$uploads = wp_upload_dir();

            //$targetDir = self::$path;
            $targetDir = SLPLUS_UPLOADDIR;

            if ( !is_dir( $targetDir ) || !is_writable( $targetDir ) ) {
                delete_transient( $this->upload_transient );
                exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 100, "message" => __( "Uploads folder is not writable.", 'slp-power' ) ), "id" => "id" ) ) );
            }

            $cleanupTargetDir = true; // Remove old files
            $maxFileAge = 5 * 3600; // Temp file age in seconds

            // 5 minutes execution time
            @set_time_limit( 5 * 60 );

            // Uncomment this one to fake upload time
            // usleep(5000);

            // Get parameters
            $chunk = isset( $_REQUEST[ "chunk" ] ) ? intval( $_REQUEST[ "chunk" ] ) : 0;
            $chunks = isset( $_REQUEST[ "chunks" ] ) ? intval( $_REQUEST[ "chunks" ] ) : 0;
            $fileName = isset( $_REQUEST[ "name" ] ) ? $_REQUEST[ "name" ] : '';

            // Clean the fileName for security reasons
            $fileName = preg_replace( '/[^\w\._]+/', '_', $fileName );

            if ( !preg_match( '%\W(csv)$%i', trim( basename( $fileName ) ) ) ) {
                exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 100, "message" => __( "Uploaded file must be CSV", 'slp-power' ) ), "id" => "id" ) ) );
            }

            // Make sure the fileName is unique but only if chunking is disabled
            if ( $chunks < 2 && file_exists( $targetDir . DIRECTORY_SEPARATOR . $fileName ) ) {
                $ext = strrpos( $fileName, '.' );
                $fileName_a = substr( $fileName, 0, $ext );
                $fileName_b = substr( $fileName, $ext );

                $count = 1;
                while ( file_exists( $targetDir . DIRECTORY_SEPARATOR . $fileName_a . '_' . $count . $fileName_b ) )
                    $count++;

                $fileName = $fileName_a . '_' . $count . $fileName_b;
            }

            $filePath = $targetDir . DIRECTORY_SEPARATOR . $fileName;

            // Create target dir
            if ( !file_exists( $targetDir ) )
                @mkdir( $targetDir );

            // Remove old temp files
            if ( $cleanupTargetDir && is_dir( $targetDir ) && ( $dir = opendir( $targetDir ) ) ) {
                while ( ( $file = readdir( $dir ) ) !== false ) {
                    $tmpfilePath = $targetDir . DIRECTORY_SEPARATOR . $file;

                    // Remove temp file if it is older than the max age and is not the current file
                    if ( preg_match( '/\.part$/', $file ) && ( filemtime( $tmpfilePath ) < time() - $maxFileAge ) && ( $tmpfilePath != "{$filePath}.part" ) ) {
                        @unlink( $tmpfilePath );
                    }
                }

                closedir( $dir );
            } else {
                delete_transient( $this->upload_transient );
                exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 100, "message" => __( "Failed to open temp directory.", 'slp-power' ) ), "id" => "id" ) ) );
            }


            // Look for the content type header
            if ( isset( $_SERVER[ "HTTP_CONTENT_TYPE" ] ) )
                $contentType = $_SERVER[ "HTTP_CONTENT_TYPE" ];

            if ( isset( $_SERVER[ "CONTENT_TYPE" ] ) )
                $contentType = $_SERVER[ "CONTENT_TYPE" ];

            // Handle non multipart uploads older WebKit versions didn't support multipart in HTML5
            if ( strpos( $contentType, "multipart" ) !== false ) {
                if ( isset( $_FILES[ 'slp_location_upload' ][ 'tmp_name' ] ) && is_uploaded_file( $_FILES[ 'slp_location_upload' ][ 'tmp_name' ] ) ) {
                    // Open temp file
                    $out = fopen( "{$filePath}.part", $chunk == 0 ? "wb" : "ab" );
                    if ( $out ) {
                        // Read binary input stream and append it to temp file
                        $in = fopen( $_FILES[ 'slp_location_upload' ][ 'tmp_name' ], "rb" );

                        if ( $in ) {
                            while ( $buff = fread( $in, 4096 ) )
                                fwrite( $out, $buff );
                        } else {
                            delete_transient( $this->upload_transient );
                            exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 101, "message" => __( "Failed to open input stream.", 'slp-power' ) ), "id" => "id" ) ) );
                        }
                        fclose( $in );
                        fclose( $out );
                        @unlink( $_FILES[ 'slp_location_upload' ][ 'tmp_name' ] );
                    } else {
                        delete_transient( $this->upload_transient );
                        exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 102, "message" => __( "Failed to open output stream.", 'slp-power' ) ), "id" => "id" ) ) );
                    }
                } else {
                    delete_transient( $this->upload_transient );
                    exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 103, "message" => __( "Failed to move uploaded file.", 'slp-power' ) ), "id" => "id" ) ) );
                }
            } else {
                // Open temp file
                $out = fopen( "{$filePath}.part", $chunk == 0 ? "wb" : "ab" );
                if ( $out ) {
                    // Read binary input stream and append it to temp file
                    $in = fopen( "php://input", "rb" );

                    if ( $in ) {
                        while ( $buff = fread( $in, 4096 ) )
                            fwrite( $out, $buff );
                    } else {
                        delete_transient( $this->upload_transient );
                        exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 101, "message" => __( "Failed to open input stream.", 'slp-power' ) ), "id" => "id" ) ) );
                    }

                    fclose( $in );
                    fclose( $out );
                } else {
                    delete_transient( $this->upload_transient );
                    exit( json_encode( array( "jsonrpc" => "2.0", "error" => array( "code" => 102, "message" => __( "Failed to open output stream.", 'slp-power' ) ), "id" => "id" ) ) );
                }
            }

            $post_type = false;

            // Check if file has been uploaded
            if ( !$chunks || $chunk == $chunks - 1 ) {

                $file_meta = $_FILES;
                $file_meta[ 'local_file' ] = $filePath;
                $this->addon->create_CSVLocationImporter( array( 'file_meta' => $file_meta, 'output' => 'json' ) );

                // Strip the temp .part suffix off
                rename( "{$filePath}.part", $filePath );
                chmod( $filePath, 0755 );
                delete_transient( $this->upload_transient );

	            $this->addon->csvImporter->messages->add_message(__( 'AJAX direct import.', 'slp-power' ) );
	            $this->addon->csvImporter->prepare_for_import();
                $this->addon->csvImporter->open_file_and_start_importing();

                die(
                json_encode(
                    array(
                        'jsonrps'           => '2.0',
                        'processing_report' => $this->addon->csvImporter->processing_report,
                        'error'             => NULL,
                        'result'            => NULL,
                        'id'                => 'id',
                        'name'              => $file_meta[ 'local_file' ],
                    )
                )
                );

            }

            // Return JSON-RPC response
            exit( json_encode( array( "jsonrpc" => "2.0", "error" => 'no_chunks_found', "result" => NULL, "id" => "id", "name" => $filePath ) ) );

        }
    }
}