<?php
if ( ! class_exists( 'SLPPower_Admin_General' ) ) {

	/**
	 * The things that modify the Admin / General Tab.
	 *
	 * @package StoreLocatorPlus\SLPPower\Admin\General
	 * @author Lance Cleveland <lance@storelocatorplus.com>
	 * @copyright 2016 Charleston Software Associates, LLC
	 *
	 * Text Domain: slp-power
	 *
	 * @property    SLPPower                        $addon
	 * @property    SLPPower_Admin_General_Text   $Admin_General_Text
	 *
	 */
	class SLPPower_Admin_General  extends SLP_Object_With_Objects {
		public  $addon;

		protected $class_prefix = 'SLPPower_';
		protected $objects = array(
			'Admin_General_Text' => array( 'subdir' => 'include/module/admin_tabs/', 'object' => null, 'auto_instantiate' => true,  ),
		);


        /**
         * Things we do at the start.
         */
        public function initialize() {
	        parent::initialize();
            add_action( 'slp_build_general_settings_panels', array( $this , 'extend_admin_subtab'       ), 90 );
            add_action( 'slp_build_general_settings_panels', array( $this , 'extend_user_subtab'        ), 90 );
            add_action( 'slp_build_general_settings_panels', array( $this , 'extend_data_subtab'        ), 90 );
        }

        /**
         * General / Data
         *
         * @param   SLP_Settings    $settings
         */
        public function extend_data_subtab( $settings ) {
            $group_params = array(
                'plugin'       => $this->addon,
                'section_slug' => 'data',
                'group_slug'   => 'packaged_data_extensions',
            );

            $settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'type'          => 'checkbox'   ,
                'option'        => 'use_contact_fields',
                'label'         => __( 'Enable Contact Fields' , 'slp-power' ),
                'description'   => __( 'Add detailed contacts fields to your locations data. ' , 'slp-power' ) .
                                   __( 'Includes an intelligent "Identifier" field that maintains links to external data sources through CSV import.' , 'slp-power' )
            ));

			if ( ! $this->slplus->site_architect_active ) {
				$settings->add_ItemToGroup( array(
					'group_params' => $group_params,
					'type'         => 'checkbox',
					'option'       => 'use_pages',
					'label'        => __( 'Enable Pages', 'slp-power' ),
					'description'  => __( 'Create pages for one or all locations using a template system. ', 'slp-power' ) .
					                  __( 'Pages are search engine friendly and are linked directly to your live location data. ', 'slp-power' )
				) );
			}
        }

        /**
         * Add General / Admin
         *
         * @param SLP_Settings  $settings
         */
        public function extend_admin_subtab( $settings ) {
	        if ( ! $this->slplus->site_architect_active ) {
		        $group_params = array(
			        'plugin'       => $this->addon,
			        'section_slug' => 'admin',
			        'group_slug'   => 'locations',
		        );

		        $settings->add_ItemToGroup( array(
			        'group_params' => $group_params,
			        'type'         => 'checkbox',
			        'option'       => 'highlight_uncoded',
			        'label'        => __( 'Highlight Uncoded', 'slp-power' ),
			        'description'  => __( 'Highlight the uncoded locations in the Manage Locations panel.', 'slp-power' )
		        ) );
	        }
        }

        /**
         * Extend the General / User tab with new settings.
         *
         * @param SLP_Settings  $settings
         */
        public function extend_user_subtab( $settings ) {
            $group_params = array(
                'plugin'       => $this->addon,
                'section_slug' => 'user_interface',
                'group_slug'   => 'javascript',
            );

            // JavaScript
            $settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'type'          => 'checkbox'  ,
                'option'        => 'use_sensor',
                'label'         => __('Use location sensor' , 'slp-power' ),
                'description'   =>
	                __('This can be slow to load and customers are prompted whether or not to allow location sensing.', 'slp-power') .
                    __('Modern web browsers require your site to be using https for the location sensor to work.' , 'slp-power' ) .
                    __('This is true for the latest Chrome and Firefox browsers.' , 'slp-power' )
            ));

            // Directory Builder
            if ( ! $this->slplus->site_architect_active ) {
                $group_params['header']     = __( 'Security', 'slp-power' );
                $group_params['group_slug'] = 'security';

                $settings->add_ItemToGroup( array(
                    'group_params' => $group_params,
                    'type'         => 'checkbox',
                    'option'       => 'use_nonces',
                    'label'        => __( 'Use Nonces On Directory', 'slp-power' ),
                    'description'  => __( 'Use use nonces on [slp_directory] shortcode links. ', 'slp-power' ) .
                                      __( 'Nonces make it harder to build search engine optimized links but harder for competitors to steal data. ', 'slp-power' )
                ) );
            }
        }

		/**
		 * Set default options for objects.  Override in your class.
		 */
		protected function set_default_object_options() {
			$this->objects[ 'Admin_General_Text' ][ 'options' ] = array( 'addon' => $this->addon );
		}

    }
}