<?php
if (! class_exists('SLPPower_Admin_Categories')):

/**
 * Manage the categories tab.
 *
 * @package StoreLocatorPlus\Power\Admin\Categories
 * @author Lance Cleveland <lance@charlestonsw.com>
 * @copyright 2016 Charleston Software Associates, LLC
 *
 * text-domain: slp-power
 *
 * @property        SLPPower                                $addon
 * @property        CSVImportCategories                     $csvImporter
 * @property        SLPPower_CategoryWalker_List            $ListWalker         Connect the list walker object here.
 * @property        SLP_Settings                            $settings
 */
class SLPPower_Admin_Categories extends SLPlus_BaseClass_Object  {
    public $addon;
    public $csvImporter;
    public $ListWalker;
    public $settings;

    /**
     * Things we do to get started.
     */
    function initialize() {

        $this->settings = new SLP_Settings(array(
            'name'              => $this->slplus->name . __(' - Categories','slp-power'),
            'form_action'       => admin_url() . 'admin.php?page=slp-categories',
            'save_text'         => __('Save Settings','slp-power') ,
            'form_name'         => 'category_settings',
            'form_enctype'      => 'multipart/form-data',
        ));
    }

    /**
     * Render the admin panel.
     */
    function render() {
        $this->addon->create_object_category_selector_manager();

        $this->settings->add_section(
            array(
                'name'          => 'Navigation',
                'div_id'        => 'navbar_wrapper',
                'description'   => $this->slplus->AdminUI->create_Navbar(),
                'innerdiv'      => false,
                'is_topmenu'    => true,
                'auto'          => false,
            )
        );

        $panelName  = __('Settings','slp-power');
        $this->settings->add_section(array('name' => $panelName));

        // Group : Search
        $groupName  = __('Search'         ,'slp-power');
        $this->settings->add_ItemToGroup(array(
            'section'       => $panelName                                           ,
            'group'         => $groupName                                           ,
            'use_prefix'    => false,
            'type'          => 'dropdown'                                           ,
            'label'         => __('Show Categories On Search','slp-power')   ,
            'setting'       => $this->addon->setting_name('show_cats_on_search')                       ,
            'custom'        => $this->addon->category_selector_manager->get_selectors( ARRAY_A )          ,
            'description'   =>
                __('How to show the category selector on the search form.','slp-power') . ' ' .
                __('No will not show the category selector.','slp-power') . ' ' .
                __('Single shows the category selector as a single drop down with indents if there are parent/children relations.','slp-power') . ' ' .
                __('Cascading shows a single drop down showing parents then children drop downs on an as-needed basis.','slp-power')
            ));
        $this->settings->add_ItemToGroup(array(
            'section'       => $panelName                                                       ,
            'group'         => $groupName                                                       ,
            'use_prefix'    => false,
            'type'          => 'checkbox'                                                         ,
            'setting'       => $this->addon->setting_name('hide_empty')                                            ,
            'value'          => $this->slplus->is_CheckTrue($this->addon->options['hide_empty']),
            'label'         => __('Hide Unpublished Categories','slp-power')             ,
            'description'   =>
                __('Only works if Store Pages is installed.  Hide the empty categories from the category selector. Store Pages in draft mode are considered "empty".','slp-power'),
            ));
        $this->settings->add_ItemToGroup(array(
            'section'       => $panelName                                   ,
            'group'         => $groupName                                   ,
            'use_prefix'    => false,
            'type'          => 'text'                                       ,
            'setting'       => $this->addon->setting_name('show_option_all')                   ,
            'value'         =>
                $this->slplus->WPML->get_text(
                    'show_option_all'          ,
                    $this->addon->options['show_option_all']   ,
                    'slp-power'
                ),
            'label'         => __('Any Category Label','slp-power')  ,
            'description'   =>
                __('If set, prepends this text to select "any category" as an option to the selector. Set to blank to not provide the any selection.','slp-power'),
                ));
        $this->settings->add_ItemToGroup(array(
            'section'       => $panelName                                       ,
            'group'         => $groupName                                       ,
            'use_prefix'    => false,
            'type'          => 'text'                                           ,
            'setting'       => $this->addon->setting_name('label_category')                        ,
            'value'         =>
                $this->slplus->WPML->get_text(
                    'label_category'          ,
                    $this->addon->options['label_category']   ,
                    'slp-power'
                ),
            'label'         => __('Category Select Label','slp-power')   ,
            'description'   =>
                __('The label for the category selector.','slp-power'),
             ));

        // Group : Map
        //
        $groupName  = __('Map'         ,'slp-power');
        $this->settings->add_ItemToGroup(array(
                'section'       => $panelName                                   ,
                'group'         => $groupName                                   ,
                'use_prefix'    => false,
                'type'          => 'checkbox'                                     ,
                'setting'       => $this->addon->setting_name('default_icons')                     ,
                'value'         => $this->addon->options['default_icons']      ,
                'label'         => __('Use Default Markers','slp-power') ,
                'description'   =>
                    __('Do not use custom category destination markers on map, use default Map Settings markers.','slp-power')
                ));

        // Group : Results
        //
        $groupName  = __('Results'         ,'slp-power');
        $this->settings->add_ItemToGroup(array(
                'section'       => $panelName                                   ,
                'group'         => $groupName                                   ,
                'use_prefix'    => false,
                'type'          => 'checkbox'                                     ,
                'setting'       => $this->addon->setting_name('show_icon_array')                   ,
                'value'         => $this->addon->options['show_icon_array']    ,
                'label'         => __('Show Icon Array','slp-power') ,
                'description'   =>
                    __('When enabled an array of icons will be created in the below map results and info bubble.',
                       'slp-power'
                      )
                ));
        $this->settings->add_ItemToGroup(array(
                'section'       => $panelName                                       ,
                'group'         => $groupName                                       ,
                'use_prefix'    => false,
                'type'          => 'checkbox'                                         ,
                'setting'       => $this->addon->setting_name('ajax_orderby_catcount')                 ,
                'value'         => $this->addon->options['ajax_orderby_catcount']  ,
                'label'         => __('Order By Category Count','slp-power') ,
                'description'   =>
                    __('When enabled the results will be ordered by those with the most categories assigned appearing first.',
                       'slp-power'
                      )
                ));

        // Group : View
        //
        $this->slplus->createobject_AddOnManager();
        $groupName  = __('View'         ,'slp-power');
        $this->settings->add_ItemToGroup(array(
                'section'       => $panelName                                   ,
                'group'         => $groupName                                   ,
                'use_prefix'    => false,
                'type'          => 'checkbox'                                     ,
                'setting'       => $this->addon->setting_name('show_legend_text')                  ,
                'value'         => $this->addon->options['show_legend_text']    ,
                'label'         => __('Show Text Under Legend','slp-power') ,
                'description'   =>
                    __('When enabled text will appear under each category icon in the legend. ','slp-power') .
                    sprintf(__('Add legend to the output with the [tagalong legend] shortcode via the %s view setting.','slp-power'),
                        $this->slplus->add_ons->get_product_url( 'slp-power' ) ) . ' ' .
                    __('Under the User Experience / View tab.' , 'slp-power' )
                )
         );


        //-------------------------
        // Store Categories Panel
        //-------------------------
        $panelName  = __('Taxonomy','slp-power');
        $this->settings->add_section(array('name' => $panelName));

        // Group : Manage
        //
        $groupName  = __('Manage'         ,'slp-power');
        $this->settings->add_ItemToGroup(array(
                'section'       => $panelName   ,
                'group'         => $groupName   ,
                'use_prefix'    => false,
                'type'          => 'subheader'  ,
                'label'         => ''           ,
                'show_label'    => false        ,
                'description'   =>
                    sprintf(
                        __('The <a href="%s">Category Manager</a>','slp-power'),
	                    add_query_arg( array( 'taxonomy' => SLPlus::locationTaxonomy , 'post_type' => SLPlus::locationPostType ), admin_url( 'edit-tags.php' ) )
                        ) .
                    __(' provides a full management interface for Store Categories.', 'slp-power')
                ));

	    $this->addon->create_object_category_manager();
	    if ( $this->addon->category_manager->get_category_count() > 0 ) {
		    $this->settings->add_ItemToGroup( array(
			    'section'     => $panelName,
			    'group'       => $groupName,
			    'use_prefix'  => false,
			    'type'        => 'subheader',
			    'label'       => '',
			    'show_label'  => false,
			    'description' =>
				    __( 'Your store categories:', 'slp-power' ) .
				    $this->createstring_StoreCategoryList()
		    ) );
	    }

        $this->settings->render_settings_page();
    }

    /**
     * Render the list of store categories.
     *
     *  @return string HTML for the category list.
     */
    function createstring_StoreCategoryList() {
    	$this->addon->create_object_category_manager();
	    if ( $this->addon->category_manager->get_category_count() <= 0  ) {
	    	return;
	    }

        $this->create_CategoryWalkerForList();

        return
            '<ul id="tagalong_category_list">' .
            walk_category_tree(
            	$this->addon->category_manager->wp_categories,
                0,
                array(
                	'addon'             => $this->addon       ,
                    'echo'              => '0'                ,
                    'hierarchical'      => '1'                ,
                    'hide_empty'        => '0'                ,
	                'style'             => 'list'             ,
                    'walker'            => $this->ListWalker
                )
            ) .
            '</ul>'
            ;
    }

    /**
     * Setup the List category walker object.
     */
    private function create_CategoryWalkerForList() {
        if ( ! isset( $this->ListWalker ) ) {
            require_once('SLPPower_CategoryWalker_List.php');
            $this->ListWalker = new SLPPower_CategoryWalker_List(array('parent'=>$this));
        }
    }
}

endif;