<?php
if (! class_exists( 'SLPPower_Directory_Landing_Page' )) {

    /**
     * The slp_directory shortcode landing page processor.
     *
     * [slp_directory style="landing_page"]
     *
     * @package StoreLocatorPlus\SLPDirectory\UI\Shortcode\slp_directory\landing_page
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     * @link https://www.storelocatorplus.com/support/documentation/store-locator-plus/directory-builder/
     *
     * @property        SLPPower    $addon
     * @property-read   string      $field      The field we are filtering on.
     * @property-read   string      $filter_by  The shorthand name of the field we are filtering on.
     * @property        string      $html       The landing page HTML.
     * @property-read   string      $value      The value we are filtering on.
     */
    class SLPPower_Directory_Landing_Page  extends SLPlus_BaseClass_Object {
        private $field;
        private $filter_by;
        public $html;
        private $value;

        /**
         * Thing we do at the start.
         */
        function initialize() {
            add_filter( 'slp_ajaxsql_where' , array( $this , 'set_WhereClause' ) );
            $this->slplus->UI->setup_stylesheet_for_slplus();
        }

        /**
         * Set the location URL.
         * @return mixed
         */
        private function set_LocationURL() {
            $url = '';
            if ( $this->slplus->is_CheckTrue( $this->addon->options[ 'pages_replace_websites' ] ) ) {
                $url = $this->slplus->currentLocation->pages_url;

            } else if ( ! empty( $this->slplus->currentLocation->url ) ) {
                $prefix = '';
                if (
                    ( strpos( $this->slplus->currentLocation->url , 'http://' ) === false ) &&
                    ( strpos( $this->slplus->currentLocation->url , 'https://') === false )
                ) {
                    $prefix = 'http://';
                }

                if ( filter_var( $prefix . $this->slplus->currentLocation->url , FILTER_VALIDATE_URL ) ) {
                    $url = $prefix . $this->slplus->currentLocation->url;
                }
            }
            return $url;
        }

        /**
         * Process the slp_location shortcodes.
         */
        public function process_slp_location_shortcode( $attributes , $content , $tag ) {

            $output = '';

            // Field Name
            switch ( $attributes[0] ) {

                // Renamed Fields
                //
                case 'name':
                    $output = $this->slplus->currentLocation->store;
                    break;

                // Standard Fields
                //
                case 'id':
                case 'store':
                case 'address':
                case 'address2':
                case 'city':
                case 'state':
                case 'zip':
                case 'country':
                case 'latitude':
                case 'longitude':
                case 'tags':
                case 'description':
                case 'email':
                case 'url':
                case 'hours':
                case 'phone':
                case 'fax':
                case 'image':
                case 'private':
                case 'neat_title':
                case 'linked_postid':
                case 'pages_url':
                case 'pages_on':
                case 'option_value':
                case 'lastupdated':
                    $output = $this->slplus->currentLocation->$attributes[0];
                    break;

                // Special Processing
                //
                default:
                    $output = $this->process_slp_location_special( $attributes, $content , $tag );
                    break;
            }

            return $output;
        }

        /**
         * Output for special slp_location data elements.
         *
         * These are the non-field data elements, such as distance_1 which is normally
         * calculated in the JavaScript processor.
         *
         * @param $attributes
         * @param $content
         * @param $tag
         * @return string
         */
        private function process_slp_location_special( $attributes, $content , $tag ) {
            $output = '';

            // Field Name
            switch ( $attributes[0] ) {

                // Aggregates
                //
                case 'city_state_zip':
                    $output = $this->slplus->currentLocation->city;
                    if ( ! empty( $this->slplus->currentLocation->city ) && ! empty( $this->slplus->currentLocation->state ) ) { $output .= ', '; }
                    $output .= $this->slplus->currentLocation->state;
                    if ( ! empty( $output ) ) { $output .= ' '; }
                    $output .= $this->slplus->currentLocation->zip;
                    break;

                // Links
                //
                case 'web_link':

                    $url = $this->set_LocationURL();
                    $target = $this->slplus->is_CheckTrue( $this->addon->options[ 'prevent_new_window' ] ) ? '_self' : '_blank' ;

                    $output =
                        "<a href='{$url}' target='{$target}' class='storelocatorlink'>".
                        "{$this->slplus->WPML->get_text('label_website')}".
                        '</a>'.
                        '<br/>'
                        ;

                    break;

                case 'email_link':
                    if ( filter_var( $this->slplus->currentLocation->email , FILTER_VALIDATE_EMAIL ) ) {
                        $output =
                            "<a href='mailto:{$this->slplus->currentLocation->email}' target='_blank'  id='slp_marker_email' class='storelocatorlink'>".
                            "{$this->slplus->currentLocation->email}".
                            '</a>'
                            ;
                    }
                    break;

                // Unknown blank output
                //
                default:
                    $output = '';
                    break;
            }

            return $output;
        }

        /**
         * Generate the landing page HTML from the SLP results layout.
         */
        public function set_html_FromResultsLayout() {
            $this->html = '';

            // Get the map results layout format.
            //
            $results_layout = $this->slplus->UI->set_ResultsLayout( true, true );

            // Check Incoming Request Data
            //
            if ( $this->addon->userinterface->is_a_valid_directory_redirect() ) {

                $this->set_FilterProperties();
                add_shortcode( 'slp_location' , array( $this , 'process_slp_location_shortcode' ) );

                // Loop through each matching location
                //
                $offset = 0;
                while ( ( $location = $this->slplus->database->get_Record( array('selectall','where_default') , array( ) , $offset++) ) !== null) {
                    $this->slplus->currentLocation->set_PropertiesViaArray($location);
                    $this->html .= do_shortcode($results_layout);
                }

                // Wrap the HTML in a div so we can style it easier.
                //
                $this->html = "<div class='slp_directory landing_page by_{$this->addon->userinterface->property_shorthand}' >{$this->html}</div>";

            // Not called from a directory link, nothing to output.
            //
            } else {
                $this->html = '';
            }
        }

        /**
         * Set filter properties, the by, field, and value we are filtering on.
         */
        private function set_FilterProperties() {
            if ( !isset( $this->addon->userinterface->location_property             ) ) { return; }
            $this->field = $this->addon->userinterface->location_property;
            $this->filter_by = $this->addon->userinterface->property_shorthand;

            if ( !isset( $_REQUEST[$this->field]  ) ) { return; }
            $this->value = $_REQUEST[$this->field];
        }

        /**
         * Extend the database where clause to filter by city, state, zip, etc.
         *
         * @param string $where
         * @return string
         */
        public function set_WhereClause( $where ) {
            if ( !isset( $this->field             ) ) { return $where; }
            if ( !isset( $_REQUEST[$this->field]  ) ) { return $where; }

            return
                $this->slplus->database->extend_WhereFieldMatches(
                    $where ,
                    $this->field ,
                    $this->value
                );
        }

    }
}