<?php

if ( !class_exists( 'SLPPower_Admin_Pages' ) ) {

    /**
     * Admin pages stuff.
     *
     * @package StoreLocatorPlus\Power\Admin\Pages
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * @property    SLPPower                    $addon
     * @property    SLPPower_Admin              $admin
     * @property    SLPPower_Admin_Pages_Tab    $tab
     *
     */
    class SLPPower_Admin_Pages extends SLPlus_BaseClass_Object {
        public $addon;
        public $admin;
        public $tab;

        /**
         * Things we do to get started.
         */
        function initialize() {
            $this->add_hooks_and_filters(); // Only called after SLP init and if in admin mode and if pages is active.
            $this->save_pages_settings();
        }

        /**
         * Add a location action button.
         *
         * @param string $theHTML - the HTML of the original buttons in place
         * @param array $locationValues
         * @return string - the augmented HTML
         */
        function add_action_buttons( $theHTML , $locationValues ) {
            if ( !isset( $locationValues[ 'sl_id' ] ) ) {
                return $theHTML;
            }
            if ( $locationValues[ 'sl_id' ] < 0 ) {
                return $theHTML;
            }

            $page_class = (($locationValues[ 'sl_linked_postid' ] > 0) ? 'haspage_icon' : 'createpage_icon');
            $icon_text = ( $page_class === 'createpage_icon' ) ? __( 'create page' , 'slp-power' ) : __( 'recreate page' , 'slp-power' );

            $pageURL = preg_replace(
                           '/&createpage=/' . (isset( $_GET[ 'createpage' ] ) ? $_GET[ 'createpage' ] : '') , '' , $_SERVER[ 'REQUEST_URI' ]
                       ) .
                       '&act=createpage' .
                       '&sl_id=' . $locationValues[ 'sl_id' ] .
                       '&slp_pageid=' . $locationValues[ 'sl_linked_postid' ] .
                       '#a' . $locationValues[ 'sl_id' ]
            ;
            return $theHTML .
                   "<a  class='dashicons dashicons-media-document slp-no-box-shadow $page_class' " .
                   "alt='{$icon_text}' " .
                   "title='{$icon_text}' " .
                   "href='$pageURL'></a>"
                ;
        }

        /**
         * Add pages admin hooks and filters if pages is active.
         */
        private function add_hooks_and_filters() {
            // Pages Content Management
            add_filter( 'slp_pages_content'                     , array( $this , 'replace_id_placeholder'       )           );
            add_filter( 'slp_location_page_attributes'          , array( $this , 'create_content'               )           );
            add_action( 'publish_' . SLPlus::locationPostType   , array( $this , 'update_content_on_publish'    ) , 90 , 2  );
            add_action( 'before_delete_post'                    , array( $this , 'delete_page'                  )           );

            // Locations Tab
            add_action( 'slp_build_locations_panels'            , array( $this , 'extend_location_edit'         ) , 30      );
            add_filter( 'slp_column_data'                       , array( $this , 'extend_location_columns'      ) , 90, 3   );
            add_filter( 'slp_manage_expanded_location_columns'  , array( $this , 'extend_elocation_columns'     )           );
            add_filter( 'slp_locations_manage_bulkactions'      , array( $this , 'extend_bulk_actions'          )           );
            add_filter( 'slp_manage_locations_actionbuttons'    , array( $this , 'add_action_buttons'           ) , 15 , 2  );
        }

        /**
         * Create the tab object for the pages tab.
         */
        private function create_object_tab() {
            if ( ! isset( $this->tab ) ) {
                require_once( 'class.admin.pages.tab.php' );
                $this->tab = new SLPPower_Admin_Pages_Tab( array( 'addon' => $this->addon , 'admin' => $this ) );
            }
        }

        /**
         * Set the page content when location create/update page is called.
         *
         * @param   array $page_data
         *
         * @return  array
         */
        function create_content( $page_data ) {
            if ( ( ! isset( $_REQUEST['action'] ) || ( $_REQUEST['action'] !== 'createpage' ) ) && isset( $page_data['ID'] ) && ! empty ( $page_data[ 'ID' ] ) ) { return $page_data; }
            $new_attributes =
                array_merge( $page_data , array(
                        'post_content' => $this->create_string_content() ,
                        'post_status' => $this->addon->options[ 'default_page_status' ]
                    )
                );
            return $new_attributes;
        }

        /**
         * Create the action entry string based on the params entries.
         *
         * params['label'] - required, the text to show for the link
         * params['ifok']  - boolean, if true only show this item if the pages_url for the current location is set
         *
         * @param mixed[] $params
         * @return string
         */
        private function create_string_action_entry( $params ) {
            // If all these things are true that means we have no page URL
            // and we are not supposed to be showing this... get out
            //
            if ( ($this->slplus->currentLocation->pages_url == '') && isset( $params[ 'ifpageexists' ] ) && $params[ 'ifpageexists' ] ) {
                return;
            }
            if ( !isset( $params[ 'label' ] ) || empty( $params[ 'label' ] ) ) {
                return;
            }

            // Set title if not set
            if ( !isset( $params[ 'title' ] ) ) {
                $params[ 'title' ] = $params[ 'label' ];
            }

            // Set link if not set
            if ( !isset( $params[ 'link' ] ) ) {
                $params[ 'link' ] = '#';
            }

            // Rel used?
            //
            if ( isset( $params[ 'rel' ] ) ) {
                $params[ 'rel' ] = " rel='{$params[ 'rel' ]}' ";
            } else {
                $params[ 'rel' ] = '';
            }

            // Target used?
            //
            if ( isset( $params[ 'target' ] ) ) {
                $params[ 'target' ] = " target='{$params[ 'target' ]}' ";
            } else {
                $params[ 'target' ] = '';
            }

            // Class used?
            //
            if ( isset( $params[ 'class' ] ) ) {
                $params[ 'class' ] = " class='{$params[ 'class' ]}' ";
            } else {
                $params[ 'class' ] = '';
            }

            return "<a {$params[ 'class' ]} {$params[ 'rel' ]} title='{$params[ 'title' ]}' href='{$params[ 'link' ]}' {$params[ 'target' ]}>{$params[ 'label' ]}</a>";
        }

        /**
         * Create the action output for add/edit locations.
         * @return string
         */
        private function create_string_actions() {
            $actions = $this->set_page_actions();
            $action_count = count( $actions );
            $i = 1;
            $out = '<div class="row-actions">';
            foreach ( $actions as $action => $link ) {
                ( $i++ == $action_count ) ? $sep = '' : $sep = ' | ';
                $out .= "<span class='$action'>$link$sep</span>";
            }
            $out .= '</div>';
            return $out;
        }

        /**
         * Create the content for a Store Page.
         *
         * Creates the content for the page.  If plus pack is installed
         * it uses the plus template file, otherwise we use the hard-coded
         * layout.
         *
         * @return string
         */
        private function create_string_content() {

            // Make sure we have a default template.
            //
            if ( empty( $this->addon->options[ 'page_template' ] ) ) {
                $this->addon->options[ 'page_template' ] = $this->create_string_default_template();
            }

            // FILTER: slp_pages_content
            //
            return apply_filters( 'slp_pages_content' , $this->addon->options[ 'page_template' ] );
        }

        /**
         * Create the default Store Page content.
         *
         *
         *
         * @return string - HTML content that is the WordPress page content.
         */
        public function create_string_default_template() {
            $content = '<span class="storename">[storepage field=store]</span>' . "\n" .
                       '[storepage field=image type=image]' . "\n" .
                       '[storepage field=address]' . "\n" .
                       '[storepage field=address2]' . "\n" .
                       '[storepage field=city] [storepage field=state] [storepage field=zip] ' . "\n" .
                       '[storepage field=country]' . "\n" .
                       '<h1>' . __( 'Description' , 'slp-power' ) . '</h1>' . "\n" .
                       '<p>[storepage field=description]</p>' . "\n" .
                       '<h1>' . __( 'Contact Info' , 'slp-power' ) . '</h1>' . "\n" .
                       $this->slplus->WPML->get_text( 'label_phone' ) . '[storepage field=phone]' . "\n" .
                       $this->slplus->WPML->get_text( 'label_fax' ) . '[storepage field=fax]' . "\n" .
                       '[storepage field=email type=mailto]' . "\n" .
                       '[storepage field=url type=hyperlink]' . "\n" .
                       '[storepage map=location]' . "\n";
            ;

            return apply_filters( 'slp_pages_default_content' , $content );
        }

        /**
         * Create a short Store Page URL for use on manage locations interface.
         *
         * @param string $fullURL
         * @return string the short hyperlinked URL
         */
        function create_string_short_url( $fullURL ) {
            $pattern = '/^(.*?)=/';
            $shortURL = preg_replace( $pattern , '' , $fullURL );
            $shortURL = str_replace( get_site_url() , '' , $shortURL );
            return "<a href='$fullURL' target='csa'>$shortURL</a>";
        }

        /**
         * Set the link in store locations table to null when a store page is permanently deleted.
         *
         * @param int $pageID
         * @return boolean
         */
        function delete_page( $pageID ) {
            $locationID = $this->slplus->database->get_Value(
                array( 'selectslid' , 'wherelinkedpostid' ) , $pageID
            );
            $this->slplus->currentLocation->set_PropertiesViaDB( $locationID );

            if ( ($this->slplus->currentLocation->linked_postid !== '') ||
                 ($this->slplus->currentLocation->pages_url !== '' )
            ) {
                $this->slplus->currentLocation->linked_postid = '';
                $this->slplus->currentLocation->pages_url = '';

                $this->slplus->currentLocation->MakePersistent();
            }
            return true;
        }

        /**
         * Add more actions to the Bulk Action drop down on the admin Locations/Manage Locations interface.
         *
         * @param mixed[] $BulkActions
         * @return mixed[]
         */
        function extend_bulk_actions( $BulkActions ) {
            return
                array_merge(
                    $BulkActions , array(
                        array(
                            'label' => __( 'Pages, Create' , 'slp-power' ) ,
                            'value' => 'createpage' ,
                        ) ,
                        array(
                            'label' => __( 'Pages, Delete Permanently' , 'slp-power' ) ,
                            'value' => 'deletepage' ,
                        ) , )
                );
        }

        /**
         * Add the Store Pages URL column.
         *
         * @param array $theColumns - the array of column data/titles
         * @return array - modified columns array
         */
        function extend_elocation_columns( $theColumns ) {
            return array_merge( $theColumns , array(
                    'sl_pages_url' => __( 'Pages URL' , 'slp-power' ) ,
                )
            );
        }

        /**
         * Render the extra fields on the manage location table.
         *
         * SLP Filter: slp_column_data
         *
         * @param string $theData  - the option_value field data from the database
         * @param string $theField - the name of the field from the database (should be sl_option_value)
         * @param string $theLabel - the column label for this column (should be 'Categories')
         * @return string the modified data
         */
        function extend_location_columns ( $theData , $theField , $theLabel ) {
            if ( $theField === 'sl_pages_url' ) {
                $theData = '';
                if ( $this->slplus->currentLocation->pages_url != '' ) {
                    $theData .= '<span class="infoid floater">' . get_post_status( $this->slplus->currentLocation->linked_postid ) . '</span>';
                    $theData .= $this->create_string_short_url( $this->slplus->currentLocation->pages_url );
                }
                $theData .= $this->create_string_actions();
            }
            return $theData;
        }

        /**
         * Extend the location edit form.
         *
         * TODO: Convert to new group params and slug based system.    Called by slp_build_locations_panels.
         */
        function extend_location_edit() {
            if ( empty( $this->slplus->currentLocation->pages_url ) ) {
                return;
            }

            $section_name = __( 'Edit' , 'slp-power' );
            $group_name = $this->addon->name;

            $shortSPurl = preg_replace( '/^.*?store_page=/' , '' , $this->slplus->currentLocation->pages_url );
            if ( $this->slplus->currentLocation->linked_postid >= 0 ) {
                $pageEditLink = sprintf( '<a href="%s" class="action_icon edit_icon" target="_blank"></a>' , admin_url() . 'post.php?post=' . $this->slplus->currentLocation->linked_postid . '&action=edit' );
            }

            $pages_html = '<div id="slp_pages_fields" class="slp_editform_section">' .
                          "<label for='pages_url'>" .
                          sprintf( __( 'Store Page ID %d is at ' , 'slp-power' ) , $this->slplus->currentLocation->linked_postid ) .
                          "</label>" .
                          $pageEditLink . ' ' .
                          "<a name='pages_url' href='{$this->slplus->currentLocation->pages_url}' target='csa'>$shortSPurl</a>" .
                          $this->create_string_actions() .
                          '</div>'
            ;

            $this->slplus->AdminUI->ManageLocations->settings->add_ItemToGroup( array(
                'setting' => 'slp_tagalong_fields' ,
                'custom' => $pages_html ,
                'type' => 'custom' ,
                'show_label' => false ,
                'section' => $section_name ,
                'group' => $group_name ,
            ) );
        }

        /**
         * Render the admin panel.
         */
        function render_pages_tab() {
            $this->create_object_tab();
            $this->tab->render();
        }

        /**
         * Replace the content sl_id to the real id value.
         *
         * @param string $content
         * @return string
         */
        function replace_id_placeholder( $content ) {
            return str_replace( "%sl_id%" , $this->slplus->currentLocation->id , $content );
        }

        /**
         * Save settings on pages, set permalink flush.
         */
        private function save_pages_settings() {
            if ( isset( $_REQUEST['page'] )  && ( $_REQUEST['page'] === 'slp-pages' ) ) {
                $this->addon->options['permalink_flush_needed'] = '1';
	            $this->slplus->option_manager->update_wp_option( $this->addon->option_name, $this->addon->options );
            }
        }

        /**
         * Set Page Actions.
         *
         * @return array
         */
        private function set_page_actions() {
            $actions = array();

            // Things we only do for posts with a linked post ID
            //
            if ( $this->slplus->currentLocation->linked_postid > 0 ) {
                $pageStatus = get_post_status( $this->slplus->currentLocation->linked_postid );

                // Recreate
                //
                $link = $this->create_string_action_entry( array(
                    'ifpageexists' => true ,
                    'label' => __( 'Recreate' , 'slp-power' ) ,
                    'link' => $this->slplus->AdminUI->ManageLocations->hangoverURL . '&act=createpage&sl_id=' . $this->slplus->currentLocation->id
                ) );
                if ( !empty( $link ) ) {
                    $actions[ 'create' ] = $link;
                }

                // Edit
                //
                $link = $this->create_string_action_entry( array(
                    'ifpageexists' => true ,
                    'label' => __( 'Edit' , 'slp-power' ) ,
                    'link' => admin_url() . 'post.php?post=' . $this->slplus->currentLocation->linked_postid . '&action=edit'
                ) );
                if ( !empty( $link ) ) {
                    $actions[ 'edit' ] = $link;
                }

                // Trash
                //
                if ( $pageStatus !== 'trash' ) {
                    $link = $this->create_string_action_entry( array(
                        'ifpageexists' => true ,
                        'label' => __( 'Trash' , 'slp-power' ) ,
                        'class' => 'submitdelete' ,
                        'link' => get_delete_post_link( $this->slplus->currentLocation->linked_postid )
                    ) );
                    if ( !empty( $link ) ) {
                        $actions[ 'trash' ] = $link;
                    }
                }

                // Delete Permanently
                //
                $link = $this->create_string_action_entry( array(
                    'ifpageexists' => true ,
                    'label' => __( 'Delete' , 'slp-power' ) ,
                    'class' => 'submitdelete' ,
                    'link' => get_delete_post_link( $this->slplus->currentLocation->linked_postid , '' , true )
                ) );
                if ( !empty( $link ) ) {
                    $actions[ 'delete' ] = $link;
                }


                // View/Preview
                //
                switch ( $pageStatus ) {
                    // View mode for published, private items
                    //
                    case 'private':
                    case 'publish':
                        $link = $this->create_string_action_entry( array(
                            'ifpageexists' => true ,
                            'label' => __( 'View' , 'slp-power' ) ,
                            'rel' => 'permalink' ,
                            'target' => 'csa' ,
                            'link' => $this->slplus->currentLocation->pages_url
                        ) );
                        if ( !empty( $link ) ) {
                            $actions[ 'view' ] = $link;
                        }
                        break;

                    // All others - preview mode
                    //
                    default:
                        $link = $this->create_string_action_entry( array(
                            'ifpageexists' => true ,
                            'label' => __( 'Preview' , 'slp-power' ) ,
                            'rel' => 'permalink' ,
                            'target' => 'csa' ,
                            'link' => get_site_url() . '?post_type=store_page&p=' . $this->slplus->currentLocation->linked_postid . '&preview=true'
                        ) );
                        if ( !empty( $link ) ) {
                            $actions[ 'view' ] = $link;
                        }
                        break;
                }
            }
            return $actions;
        }

        /**
         * Check if the content of Store Page post is blank when it's being published.
         * If it is, set content to default template value.
         *
         * @param mixed $post_id
         * @param mixed $post
         */
        function update_content_on_publish( $post_id , $post ) {
            if ( trim( $post->post_content ) == '' ) {
                $post->post_content = $this->create_string_default_template();
                wp_update_post( $post );
            }
        }
    }

}
