<?php

if ( !class_exists( 'SLPPower_UI_Shortcode_Storepage' ) ) {

    /**
     * UI pages stuff on loaded of using pages is enabled.
     *
     * @package StoreLocatorPlus\Power\UserInterface\Shortcode\Storepage
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * @property    SLPPower    $addon
     *
     */
    class SLPPower_UI_Shortcode_Storepage extends SLPlus_BaseClass_Object {
        public $addon;

        /**
         * Create an url hyperlink base on the type field provided.
         *
         * @param string $field
         * @param string $content
         *
         * @return string
         */
        private function create_string_hyperlink( $field , $content ) {
            if ( is_null( $field ) ) {
                return '';
            }
            if ( empty( $content ) ) {
                return '';
            }
            return
                sprintf( '<a href="%s" target="store_locator_plus">%s</a>' , esc_url( $content ) , $content
                );
        }

        /**
         * Create an image string if the field and content are valid.
         *
         * @param string $field
         * @param string $content
         * @param string $title
         *
         * @return string
         */
        private function create_string_image( $field , $content , $title ) {
            if ( is_null( $this->slplus->currentLocation->id ) ) { return ''; }

            if ( is_null( $field ) ) {
                return '';
            }
            if ( empty( $content ) ) {
                return '';
            }
            if ( empty( $title ) ) {
                $title = $this->slplus->currentLocation->store;
            }
            $title = sprintf( 'title="%s" alt="%s"' , $title , $title );

            return
                sprintf( '<img src="%s" class="store_page_image field-%s" %s>' , esc_url( $content ) , $field , $title
                );
        }

        /**
         * Create an email hyperlink base on the type field provided.
         *
         * @param string $field
         * @param string $content
         *
         * @return string
         */
        private function create_string_mailto( $field , $content ) {
            if ( is_null( $field ) ) {
                return '';
            }
            if ( empty( $content ) ) {
                return '';
            }
            return
                sprintf( '<a href="%s" target="store_locator_plus">%s</a>' , 'mailto:' . $content , $content
                );
        }

        /**
         * Generate a map for the current location.
         */
        private function generate_map() {
            if ( is_null( $this->slplus->currentLocation->id ) ) { return ''; }

            $map_style = 'style="' .
                         "height: {$this->slplus->options_nojs[ 'map_height' ]}{$this->slplus->options_nojs[ 'map_height_units' ]}; " .
                         "width: {$this->slplus->options_nojs[ 'map_width' ]}{$this->slplus->options_nojs[ 'map_width_units' ]}; " .
                         '"';

            $html = "<div id='storepage-{$this->slplus->currentLocation->id}-map' title='location_map' data-location_id='{$this->slplus->currentLocation->id}' class='map-canvas-box' {$map_style}>" .
                    "<div id='map-canvas-{$this->slplus->currentLocation->id}' class='map-canvas'></div>" .
                    "</div>";

            return $html;
        }

        /**
         * Return Post value for the linked post.
         *
         * @param $value
         *
         * @return string
         */
        private function get_post_data( $value ) {
            $args = array(
                'echo'      => false,
                'post_type' => SLPLus::locationPostType,
                'include'   => array( $this->slplus->currentLocation->linked_postid ),
            );
            $page = get_pages( $args );

            if ( is_array( $page ) && isset( $page[0] ) ) {

                /**
                 * @var WP_Post $the_post
                 */
                $the_post = $page[0];

                switch ( $value ) {

                    case 'post_content':
                        $cleaned_content = preg_replace( '/post\W+post_content\W+/i' , '' , $the_post->post_content ); // try not to loop this
                        return do_shortcode( $cleaned_content );

                    case 'permalink':
                        return get_permalink( $the_post->ID );

                    default:
                        return isset( $the_post->$value ) ? $the_post->$value : '';
                }
            }

            return '';
        }

        /**
         * Manage the shortcode
         *
         * @param array $attributes named array of attributes set in shortcode
         * @param string $content the existing content that we will modify
         *
         * @return string the modified HTML content
         */
        function process_shortcode( $attributes , $content = null ) {

            // Pre-process the attributes.
            //
            // This allows third party plugins to man-handle the process by
            // tweaking the attributes.  If, for example, they were to return
            // array('hard_coded_value','blah blah blah') that is all we would return.
            //
            // FILTER: shortcode_storepage
            //
            $attributes = (array) $attributes;
            $attributes = apply_filters( 'shortcode_storepage' , $attributes );

            // Process the attributes
            //
            $field = null;
            $title = null;
            foreach ( $attributes as $key => $value ) {
                $key = strtolower( $key );
                switch ( $key ) {

                    // Field attribute: output specified field
                    //
                    case 'field':
                        if ( ! is_null( $this->slplus->currentLocation->id ) ) {
                            $field   = preg_replace( '/\W/', '', htmlspecialchars_decode( $value ) );
                            $field   = preg_replace( '/^sl_/', '', strtolower( $field ) );
                            $content = $this->slplus->currentLocation->$field;
                        } else {
                            $content = '';
                        }
                        break;

                    case 'title':
                        $title = $value;
                        break;

                    case 'type':
                        switch ( $value ) {
                            case 'hyperlink':
                                $content = $this->create_string_hyperlink( $field , $content );
                                break;

                            case 'image':
                                $content = $this->create_string_image( $field , $content , $title );
                                break;

                            case 'mailto':
                                $content = $this->create_string_mailto( $field , $content );
                                break;

                            default:
                                break;
                        }
                        break;

                    case 'hard_coded_value':
                        $content = $value;
                        break;

                    case 'map':
                        $content = $this->generate_map();
                        break;

                    case 'post':
                        $content = $this->get_post_data( $value );
                        break;

                    default:
                        break;
                }
            }

            return $content;
        }
    }

}