<?php

if ( !class_exists( 'SLPPower_UI_Pages' ) ) {

    /**
     * UI pages stuff on loaded of using pages is enabled.
     *
     * @package StoreLocatorPlus\Power\UserInterface\Pages
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     * @property    SLPPower                            $addon
     * @property    SLPPower_UI_Shortcode_Storepage     $shortcode_storepage
     * @property    SLPPower_UI                         $ui
     *
     */
    class SLPPower_UI_Pages extends SLPlus_BaseClass_Object {
        public $addon;
        public $shortcode_storepage;
        public $ui;

        /**
         * Startup stuff.
         */
        public function initialize() {
            $this->add_hooks_and_filters();
        }

        /**
         * Add WP Hooks and Filters.
         */
        private function add_hooks_and_filters() {
            add_shortcode( 'storepage' , array( $this , 'process_storepage_shortcode' ) );

            // TODO: only if Premier Subscription is active   if (  $this->slplus->add_ons->is_premier_subscription_valid() ) { ... }
            add_shortcode( 'slp_pages' , array( $this , 'process_slp_pages_shortcode' ) );

            if ( $this->has_pages_content() ) {
                add_action( 'the_post' , array( $this, 'fetch_location_for_post' ) );
                $this->setup_css_and_js();
            }
        }

        /**
         * Attach the storepage shortcode process.
         */
        function create_object_shortcode_storepage( ) {
            if ( ! isset( $this->shortcode_storepage ) ) {
                require_once( 'class.shortcode.storepage.php' );
                $this->shortcode_storepage = new SLPPower_UI_Shortcode_Storepage( array( 'addon' => $this->addon ) );
            }
        }

        /**
         * Attach the slp_pages shortcode process.
         */
        function create_object_shortcode_slp_pages( ) {
            if ( ! isset( $this->shortcode_slp_pages ) ) {
                require_once( 'class.shortcode.slp_pages.php' );
                $this->shortcode_slp_pages = new SLPPower_UI_Shortcode_SLP_Pages( array( 'addon' => $this->addon ) );
            }
        }

        /**
         * Set the location data for each run through the custom post types in a loop (or a single display).
         *
         * @param type $post_object
         */
        function fetch_location_for_post( $post_object ) {
            $current_location_array = $this->slplus->database->get_Record( array( 'selectall' , 'wherelinkedpostid' ) , $post_object->ID );
            $this->slplus->currentLocation->set_PropertiesViaArray( $current_location_array );
        }

        /**
         * Returns true if current page has Pages content.
         */
        private function has_pages_content() {
            if ( get_post_type() === SLPlus::locationPostType ) { return true; }

            $post = get_post();
            if ( ! is_a( $post , 'WP_Post' ) ) { return false; }
            $content = $post->post_content;
            if ( has_shortcode( $content , 'slp_pages' ) ) { return true; }
            if ( has_shortcode( $content , 'storepage' ) ) { return true; }

            return false;
        }

        /**
         * Manage the storepage shortcode
         *
         * @param array $attributes named array of attributes set in shortcode
         * @param string $content the existing content that we will modify
         * @return string the modified HTML content
         */
        function process_storepage_shortcode( $attributes , $content = null ) {
            $this->create_object_shortcode_storepage();
            return $this->shortcode_storepage->process_shortcode( $attributes, $content );
        }

        /**
         * Manage the slp_pages shortcode
         *
         * @param array $attributes named array of attributes set in shortcode
         * @param string $content the existing content that we will modify
         *
         * @return string the modified HTML content
         */
        function process_slp_pages_shortcode( $attributes , $content = null ) {
            $this->create_object_shortcode_slp_pages();
            $this->setup_css_and_js();
            return $this->shortcode_slp_pages->process_shortcode( $attributes, $content );
        }

        /**
         * Enqueue the CSS and JS as needed.
         */
        private function setup_css_and_js() {
            $this->ui->js_settings = array_merge( $this->slplus->options , $this->addon->options );
            $this->ui->js_settings['ajaxurl'] = admin_url( 'admin-ajax.php' );
            $this->ui->js_settings['resturl'] = rest_url( 'store-locator-plus/v1/' );
            $this->slplus->enqueue_google_maps_script();
            $this->ui->enqueue_ui_javascript();
            $this->ui->enqueue_ui_css();
        }
    }

}