/**
 * A base class that helps add-on packs separate ui functionality.
 *
 * Add on packs should include and extend this class.
 *
 * This allows the main plugin to only include this file when rendering the front end.
 * Reduces the front-end footprint.
 *
 * @package StoreLocatorPlus\Power\UserInterfaceJS
 * @author Lance Cleveland <lance@storelocatorplus.com>
 * @copyright 2016 Charleston Software Associates, LLC
 *
 * @type {SLPPOWER}
 */

// Setup the Premier namespace
var SLPPOWER = SLPPOWER || {};


/**
 * The Location List Class
 */
var slp_power_map = function () {

    /**
     * Initialize the map.
     */
    this.initialize = function () {
        var location_id;
        jQuery("[title='location_map']").each(function () {
            var location_id = jQuery(this).attr('data-location_id');
            SLPPOWER.map.get_location_data(location_id);
        });
    };

    /**
     * Add a location to the map.
     */
    this.add_this_location = function ( data ) {
        var center = new google.maps.LatLng( data.sl_latitude , data.sl_longitude );
        var options = {
            center: center,
            zoom: 14
        };

        var map = new google.maps.Map(document.getElementById('map-canvas-'+data.sl_id), options);

        var marker_settings = {
            position: center,
            map: map,
            icon: slppower_settings.map_end_icon,
        }
        marker = new google.maps.Marker( marker_settings );
    };

    /**
     * Request location data from the SLP REST service.
     *
     * @param location_id
     */
    this.get_location_data = function ( location_id ) {
        jQuery.ajax(
            slppower_settings.resturl + 'locations/' + location_id ,
            {
                success: SLPPOWER.map.add_this_location
            }
        );
    };

    /**
     * Show a placeholder in the map area until we get data back from the server.
     */
    this.show_map_placeholder = function () {

    };


}


// Document Ready
jQuery(document).ready(function () {
    SLPPOWER.map = new slp_power_map();
    SLPPOWER.map.initialize();
});
