/**
 * Power add-on administrative JavaScript stuff.
 *
 * @package StoreLocatorPlus\Power\Admin\Locations\JS
 * @author Lance Cleveland <lance@storelocatorplus.com>
 * @copyright 2016 Charleston Software Associates, LLC
 */

// Setup the Power namespace
var SLPPOWER_ADMIN_LOCATIONS = SLPPOWER_ADMIN_LOCATIONS || {};

/**
 * Location Filters.
 */
var slp_power_filters = function () {
    var state_list_populated = false;
    var country_list_populated = false;

    /**
     * Initialize.
     */
    this.initialize = function () {
        jQuery('#state_filter').click( this.get_state_list );
        jQuery('#country_filter').click( this.get_country_list );
    }

    /**
     * Get the list of countries and populate the drop down.
     */
    this.get_country_list = function() {
        if ( SLPPOWER_ADMIN_LOCATIONS.filters.country_list_populated ) {
            return;
        }

        jQuery('div#country_filter_spinner').addClass('is-active');

        var post_data = new Object();
        post_data['action']     = 'slp_get_country_list';

        var ajax_settings = new Object();
        ajax_settings['url']    = ajaxurl;
        ajax_settings['method'] = 'POST';
        ajax_settings['dataType'] = 'jsonp';
        ajax_settings['data'] = post_data;

        var request = jQuery.ajax( ajax_settings );

        request.always( function( response ) { SLPPOWER_ADMIN_LOCATIONS.filters.populate_country_list( response.responseText ); } );
    }

    /**
     *
     * @param response
     */
    this.populate_country_list = function( response ) {
        jQuery('div#country_filter_spinner').removeClass( 'is-active' );

        var json_response = JSON.parse( response );
        if ( json_response.success ) {
            var country_dropdown = jQuery( 'select#country_filter' );
            for ( var cnt = 0 ; cnt < json_response.data.count ; cnt++ ) {
                country_dropdown.append(jQuery('<option />').attr( 'value' , json_response.data.states[cnt] ).text( json_response.data.states[cnt] ));
            }

            SLPPOWER_ADMIN_LOCATIONS.filters.country_list_populated = true;
        } else {
            SLPPOWER_ADMIN_LOCATIONS.filters.state_list_populated = false;
        }
    }

    /**
     * Get the list of states and populate the drop down.
     */
    this.get_state_list = function() {
        if ( SLPPOWER_ADMIN_LOCATIONS.filters.state_list_populated ) {
            return;
        }

        jQuery('div#state_filter_spinner').addClass('is-active');

        var post_data = new Object();
        post_data['action']     = 'slp_get_state_list';

        var ajax_settings = new Object();
        ajax_settings['url']    = ajaxurl;
        ajax_settings['method'] = 'POST';
        ajax_settings['dataType'] = 'jsonp';
        ajax_settings['data'] = post_data;

        var request = jQuery.ajax( ajax_settings );

        request.always( function( response ) { SLPPOWER_ADMIN_LOCATIONS.filters.populate_state_list( response.responseText ); } );
    }

    /**
     *
     * @param response
     */
    this.populate_state_list = function( response ) {
        jQuery('div#state_filter_spinner').removeClass('is-active');

        var json_response = JSON.parse( response );
        if ( json_response.success ) {
            var state_dropdown = jQuery( 'select#state_filter' );
            for ( var cnt = 0 ; cnt < json_response.data.count ; cnt++ ) {
                state_dropdown.append(jQuery('<option />').attr( 'value' , json_response.data.states[cnt] ).text( json_response.data.states[cnt] ));
            }

            SLPPOWER_ADMIN_LOCATIONS.filters.state_list_populated = true;
        } else {
            SLPPOWER_ADMIN_LOCATIONS.filters.state_list_populated = false;
        }
    }




};

/**
 * Location Messages.
 */
var slp_power_messages = function () {

    /**
     * Clear the import messages list.
     */
    this.clear_import_messages = function () {
        jQuery( '.import_message_block').empty();

        var post_data = new Object();
        post_data['action']     = 'slp_clear_import_messages';
        jQuery.post( ajaxurl, post_data , this.process_clear_import_messages_response );
    };

    /**
     * Handle the clear response.
     *
     * @param response
     */
    this.process_clear_import_messages_response = function( response ) {
        if ( response !== 'ok' ) {
            jQuery('.import_message_block').html( '<span class="clear failed">***</span>' );
        }
    };
};

// Document Ready
jQuery( document ).ready(
    function () {
        SLPPOWER_ADMIN_LOCATIONS.filters = new slp_power_filters();
        SLPPOWER_ADMIN_LOCATIONS.filters.initialize();
        SLPPOWER_ADMIN_LOCATIONS.messages = new slp_power_messages();
    }
);
