/*
 *
 * manage_locations.js
 *
 * Power Manage Locations page scripts.
 *
 * @requires jQuery v1.2.3
 *
 * GPL 2 licensed.
 * http://www.gnu.org/licenses/gpl.html
 *
 */


var SLP_POWER_LOCATION_MANAGER = SLP_POWER_LOCATION_MANAGER || {};

/**
 * Infobox class.
 */
var slp_power_infobox = function() {
    var current_message = '';

    /**
     * Update the info box.
     */
    this.update = function( json_data ) {

        // Show alerts returned in json data.
        //
        if ( json_data.alert ) {
            alert( json_data.alert);
            return;
        }

        // No message? Leave.
        //
        if ( ! json_data.message ) {
            return;
        } else {
            this.current_message = json_data.message;
        }
        jQuery('#slp-power_messages').append( this.create_string_message_div() );
        jQuery('#slp-power_message_board').show();
    } ,

    /**
     * Create the message string div.
     */
    this.create_string_message_div = function() {
        return (
            '<div class="slp-power_message">' +
            this.current_message +
            '</div>'
        );
    }
};

/**
 * PL Uploader Class
 */
var slp_power_pluploader = function() {
    var uploader;

    /**
     * Initialize the object.
     */
    this.initialize = function () {
        var uploader_options = {
            url: ajaxurl + '?action=slp_new_upload',
            browse_button: 'select-files',
            multi_selection: false,
            file_data_name: 'slp_location_upload',
            container: 'wpcsl_settings_group-upload_a_file'
        };

        this.uploader = new plupload.Uploader( uploader_options );
        this.uploader.refresh();
        this.uploader.init();
        this.add_uploader_features();
    };

    /**
     * Add features to the uploader.
     */
    this.add_uploader_features = function () {

        /**
         * File has been added, show it on the progress area.
         */
        this.uploader.bind('FilesAdded', function(up, files) {
            var html = '';
            plupload.each(files, function(file) {
                html += '<span id="' + file.id + '">' + file.name + ' (' + plupload.formatSize(file.size) + ')</span>';
            });
            jQuery('#file_list').html('');
            jQuery('#file_list').append(html);
            SLP_POWER_LOCATION_MANAGER.upload_ux.uploader.start();
        });

        /**
         * Upload Progress Bar
         */
        this.uploader.bind('UploadProgress', function(up, file) {
            jQuery('#file_list').html('<span id="' + file.id + '">' + request_vars.text_uploading + ' ' + file.name + ' (' + plupload.formatSize(file.size) + ')</span>' );
            jQuery('#upload_process').progressbar({value:file.percent});
        });

        /**
         * File upload complete.
         *
         * Fires when AJAX exits.
         */
        this.uploader.bind('FileUploaded', function(up, file, server_reply ) {
            var response = server_reply.response;
            var parsed_response = JSON.parse( response );

            // Hide progress bar
            //
//            jQuery('#upload_process').hide();

            // Hide SLP Errors On Admin Page
            //
            jQuery('#setting-error-settings_updated').hide();

            // Change Manage Subtab click to actual page load
            //
            var location_destination = jQuery('li#toplevel_page_csl-slplus a.current').attr('href');
            jQuery('#wpcsl-option-current_locations_sidemenu').attr('href',location_destination);

            // File Status
            //
            jQuery('#file_list').html('<span id="' + file.id + '">' + request_vars.text_loaded + ' ' + file.name + '</span>' );

            // Processing Report
            //
            html = '';
            for ( num = 0 ; num < parsed_response.processing_report.length; num++ ) {
                html += '<span class="upload_report">' + parsed_response.processing_report[num] + '<span>';
            };

            jQuery('#upload_info').html( html );

        });

        /**
         * Upload Errors
         */
        this.uploader.bind('Error', function(up, error) {
            var html = "Error #" + err.code + ": " + err.message;
            jQuery('#upload_info').append(html);
        });

    }
};

// Document Is Ready...
//
jQuery(document).ready(
    function() {

        // Process incoming request actions
        //
        switch (request_vars.action) {

            // Export immediate mode, load CSV into iFrame.
            //
            case 'export':
                jQuery('#power_csv_download').attr('src',
                    ajaxurl + '?' +
                    jQuery.param(
                        {
                            action: 'slp_download_locations_csv',
                            filename: 'locations',
                            formdata: jQuery('#locationForm').serialize()
                        }
                    )
                );
                break;

            // Export, locally hosted.
            //
            case 'export_local':
                var infobox = new slp_power_infobox();
                infobox.update({'message': request_vars.download_file_message});
                break;
        }


        SLP_POWER_LOCATION_MANAGER.upload_ux = new slp_power_pluploader();
        SLP_POWER_LOCATION_MANAGER.upload_ux.initialize();
    }
);