<?php
if (! class_exists('SLPPremier_Activation')) {
    require_once(SLPLUS_PLUGINDIR.'/include/base_class.activation.php');

    /**
     * Manage plugin activation.
     *
     * @package StoreLocatorPlus\Premier\Activation
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2016 Charleston Software Associates, LLC
     *
     *
     * @property    SLPPremier      $addon
     */
    class SLPPremier_Activation  extends SLP_BaseClass_Activation {
		public $addon;

        /**
         * Update or create the data tables.
         *
         * This can be run as a static function or as a class method.
         */
        function update() {
			parent::update();

	        $this->convert_to_smart_options();

	        $this->addon->find_minmax_latlng();

			$this->add_woo_fields();

            if ( $this->slplus->is_CheckTrue( $this->addon->options['use_territory_bounds'] ) ) {
                $this->add_territory_bounds();
            }
        }

        /**
         * Add Territory Bounds field.
         */
        public function add_territory_bounds() {
            $this->slplus->database->extension->add_field(
                __( 'Boundary Unit'     ,'slp-premier' ),
                'text'   ,
                array(
                    'slug'              => 'territory_distance_unit'      ,
                    'addon'             => $this->addon->short_slug       ,
                    'display_type'      => 'list'                         ,
                    'custom'            => array( __('No Territory' , 'slp-premier' ) => '' , __( 'Miles' , 'slp-premier' ) => 'miles' , __('Kilometers' , 'slp-premier' ) => 'km' ),
                    'help_text'         => __( 'Should kilometers or miles be used when setting territory bounds when using distance from location?' , 'slp-premier' )
                )
            );
            $this->slplus->database->extension->add_field(
                __( 'North Boundary'     ,'slp-premier' ),
                'text'   ,
                array(
                    'slug'              => 'territory_distance_north'      ,
                    'addon'             => $this->addon->short_slug       ,
                    'display_type'      => 'text'                         ,
                    'help_text'         => __( 'Distance north of the location that is considered part of the covered territory.' , 'slp-premier' )
                )
            );
            $this->slplus->database->extension->add_field(
                __( 'South Boundary'     ,'slp-premier' ),
                'text'   ,
                array(
                    'slug'              => 'territory_distance_south'      ,
                    'addon'             => $this->addon->short_slug       ,
                    'display_type'      => 'text'                         ,
                    'help_text'         => __( 'Distance south of the location that is considered part of the covered territory.' , 'slp-premier' )
                )
            );
            $this->slplus->database->extension->add_field(
                __( 'East Boundary'     ,'slp-premier' ),
                'text'   ,
                array(
                    'slug'              => 'territory_distance_east'      ,
                    'addon'             => $this->addon->short_slug       ,
                    'display_type'      => 'text'                         ,
                    'help_text'         => __( 'Distance east of the location that is considered part of the covered territory.' , 'slp-premier' )
                )
            );
            $this->slplus->database->extension->add_field(
                __( 'West Boundary'     ,'slp-premier' ),
                'text'   ,
                array(
                    'slug'              => 'territory_distance_west'      ,
                    'addon'             => $this->addon->short_slug       ,
                    'display_type'      => 'text'                         ,
                    'help_text'         => __( 'Distance west of the location that is considered part of the covered territory.' , 'slp-premier' )
                )
            );
            $this->slplus->database->extension->add_field(
                __( 'Territory Bounds'     ,'slp-premier' ),
                'text'   ,
                array(
                    'slug'              => 'territory_bounds'             ,
                    'addon'             => $this->addon->short_slug       ,
                    'display_type'      => 'callback'
                )
            );

            $this->slplus->database->extension->update_data_table( array( 'mode' => 'force' ) );
        }

	    /**
	     * Add woo fields if WooCommerce is running.
	     */
	    private function add_woo_fields() {
		    if ( $this->addon->is_woo_running() ) {
			    $this->addon->instantiate('WooGlue');
			    $this->addon->WooGlue->add_extended_data_fields();
		    }
	    }

	    /**
	     * Convert legacy add on options to smart options.
	     *
	     * The parent base_class.admin.php from SLP will auto-call update_option( ) for addon->options.
	     */
	    private function convert_to_smart_options() {
		    $option_list = array(
		    	'clusters_enabled',
		    );
		    foreach ( $option_list as $option_slug ) {
			    if ( isset( $this->addon->options[ $option_slug ] ) ) {
				    $this->slplus->smart_options->{$option_slug}->value = $this->addon->options[ $option_slug ];
				    unset ( $this->addon->options[ $option_slug ] );
			    }
		    }
	    }
    }
}