<?php
if (! class_exists('SLPPremier_UI')) {

    require_once(SLPLUS_PLUGINDIR.'/include/base_class.userinterface.php');

    /**
     * Holds the UI-only code.
     *
     * This allows the main plugin to only include this file in the front end
     * via the wp_enqueue_scripts call.   Reduces the back-end footprint.
     *
     * @property        SLPPremier                  $addon
     *
     * @package StoreLocatorPlus\SLPPremier\UI
     * @author Lance Cleveland <lance@storelocatorplus.com>
     * @copyright 2015 - 2016 Charleston Software Associates, LLC
     */
    class SLPPremier_UI  extends SLP_BaseClass_UI  {
        public $addon;

	    /**
	     * Add WordPress and SLP hooks and filters.
	     */
	    public function add_hooks_and_filters() {
		    parent::add_hooks_and_filters();

			$this->js_settings = $this->addon->options;

            $this->addon->create_object_tagalong();
            if ( is_object( $this->addon->Tagalong ) ) {
                $this->addon->Tagalong->create_object_ui_tagalong();
            }

            add_filter( 'shortcode_slp_option'          , array( $this , 'augment_slp_option_shortcode'     )       );
		    add_action( 'slp_directory_shortcode_init' , array( $this , 'extend_slp_directory_shortcode' ) );
	    }

        /**
         * Manage the UI CSS enqueue process.
         *
         */
        function enqueue_ui_css() {

            // Pagination or Woo
            //
            if (
                $this->slplus->is_CheckTrue( $this->addon->options['pagination_enabled'] ) ||
                $this->addon->is_woo_running()
            ) {
                parent::enqueue_ui_css();
                wp_enqueue_style('dashicons');
            }

            if ( $this->addon->options['dropdown_style'] !== 'none' ) {
                $this->load_jquery_style();
            }
        }

        /**
         * Manage the UI JavaScript enqueue process.
         *
         */
        function enqueue_ui_javascript() {

            // Extend js_settings to include bounds 'boundaries_influence_type'] !== 'none'
            //
            if ( ( $this->addon->options['boundaries_influence_type'] !== 'none' ) ) {
                $this->js_settings['bounds'] = $this->addon->set_location_search_boundaries();
            }

            // Extend js_settings to include region if 'boundaries_influence_type' !== 'none'
            //
            if ( $this->slplus->is_CheckTrue( $this->addon->options['region_influence_enabled'] ) ) {
                $this->js_settings['region'] = $this->set_geocoding_region();
            }

            $this->js_settings['map_option_hide_streetview'] = $this->slplus->is_CheckTrue( $this->addon->options['map_option_hide_streetview'] );
            $this->js_settings['url'] = $this->addon->url;

            parent::enqueue_ui_javascript();

            // Add cluster marker scripts if clusters enabled.
            //
            if ( $this->slplus->smart_options->clusters_enabled->is_true ) {
                wp_enqueue_script( 'slp_marker_clusters', $this->addon->url . '/js/markerclusterer.js', $this->js_requirements );
            }

            // jQuery Selectmenu
            //
            if ( $this->addon->options['dropdown_style'] !== 'none' ) {
                wp_enqueue_script('jquery-ui-selectmenu');
            }
        }

	    /**
	     * Things we do when the slp_directory shortcode processing is started by the Power add on.
	     */
        public function extend_slp_directory_shortcode() {
			add_filter( 'slp_directory_entry_text' , array( $this, 'process_slp_directory_text_if_blank' ) , 10 , 2 );
        }

	    /**
	     * Add the text_if_blank shortcode attributet to slp_directory
	     *
	     * @param string $text
	     * @param string[] $shortcode_attributes
	     *
	     * @return string
	     */
        public function process_slp_directory_text_if_blank( $text, $shortcode_attributes ) {
	        if ( empty( $text ) && ! empty( $shortcode_attributes[ 'text_if_blank' ] ) ) {
		        return $shortcode_attributes[ 'text_if_blank' ];
	        }
	        return $text;
        }


        /**
         * Attached defined jQuery themes.
         *
         * They live in the SLPlus admin directory so they can be shared amongst add-ons.
         */
        private function load_jquery_style() {
            if ( $this->addon->options['dropdown_style'] === 'none' ) { return; }

            $style = $this->addon->options['dropdown_style'];
            if ( file_exists( SLPLUS_PLUGINDIR . '/css/admin/jquery-ui-' . $style . '.css' ) ) {
                wp_enqueue_style( 'jquery-ui-' . $style , SLPLUS_PLUGINURL . '/css/admin/jquery-ui-'.$style.'.css' );
            }
        }

	    /**
	     * Set the geocoding region based on the selected Map Domain cctld.
	     *
	     * @return string
	     */
	    private function set_geocoding_region() {
		    $this->slplus->create_object_CountryManager();
			$country = $this->slplus->CountryManager->countries[ $this->slplus->options_nojs['default_country'] ];
		    return $country->cctld;
	    }

    }
}