<?php
if ( !class_exists( 'SLPPremier_Admin_Experience' ) ) {

    /**
     * Class SLPPremier_Admin_UXSettings
     *
     * @property        SLPPremier                          $addon
     * @property        SLPPremier_Admin_Experience_Text    $Admin_Experience_Text
     * @property-read   array                               $group_params
     * @property-read   SLP_Settings                        $Settings
     *
     * Text Domain: slp-premier
     */
    class SLPPremier_Admin_Experience extends SLP_Object_With_Objects {
        public $addon;
	    protected $class_prefix = 'SLPPremier_';
	    private $group_params;
	    private $Settings;

	    protected $objects = array(
		    'Admin_Experience_Text' => array(
			    'subdir' => 'include/module/admin_tabs/',
			    'object' => null,
			    'auto_instantiate' => true,
		    ),
	    );

        /**
         * Instantiate the admin experience object.
         */
        function initialize() {
	        parent::initialize();

            $this->slplus->enqueue_google_maps_script();

	        $this->group_params = array( 'plugin' => $this->addon, 'section_slug' => null, 'group_slug' => null );
	        add_action( 'slp_build_map_settings_panels', array( $this, 'modify_tab' ), 99 );

            add_filter( 'slp_radius_behavior_description' , array( $this , 'extend_radius_behavior_description' ) );
            add_filter( 'slp_radius_behavior_selections'  , array( $this , 'extend_radius_behavior_selections'  ) );
        }

	    /**
	     * Modify entries on the tab.
	     *
	     * @param SLPlus_Settings $Settings
	     */
	    public function modify_tab( $Settings ) {
		    $this->Settings = $Settings;
		    $this->modify_map();
		    $this->modify_results();
		    $this->modify_search();
	    }

	    /**
	     * Map
	     */
	    private function modify_map() {
		    $this->group_params['section_slug'] = 'map';
		    $this->modify_map_appearance();
		    $this->modify_map_at_startup();
	    }

	    /**
	     * Map / Appearance
	     */
	    function modify_map_appearance( ) {
		    $this->group_params['group_slug'] = 'appearance';

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'       => __( 'Hide Street View', 'slp-premier' ),
			    'description' => __( 'Hide the street view man on the Google map interface.', 'slp-premier' ),
			    'option'     => 'map_option_hide_streetview',
			    'type'        => 'checkbox',
		    ) );


		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'       => __( 'Bubble Footnote', 'slp-premier' ),
			    'description' =>
				    __( 'The text entered here will appear in the map info bubble where the [slp_option bubble_footnote] shortcode appears. ', 'slp-premier' ),
			    'option'     => 'bubble_footnote',
			    'type'        => 'textarea',
		    ) );

	    }

	    /**
	     * Map / At Startup
	     */
	    function modify_map_at_startup() {
		    $this->group_params['group_slug'] = 'at_startup';
		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'custom'      => $this->create_map_center_div(),
			    'description' =>
				    __( 'The current position of Center Lat/Long Fallback.', 'slp-premier' ) .
				    __( 'This what will be used to display the initial locations when Google Geocoding is offline.', 'slp-premier' ) .
				    __( 'Only locations within the Results / Radius To Search Initially setting will be shown.', 'slp-premier' ),
			    'show_label'  => false,
			    'type'        => 'custom',
		    ) );

	    }

	    /**
	     * Results
	     */
	    private function modify_results() {
		    $this->group_params['section_slug'] = 'results';
		    $this->modify_results_appearance();
		    $this->modify_results_labels();
	    }

	    /**
	     * Results / Appearance
	     */
	    private function modify_results_appearance() {
		    $this->group_params['group_slug'] = 'appearance';
		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'        => __( 'Do Not Wrap Results In Div', 'slp-premier' ),
			    'description'  =>
				    __( 'When checked do not wrap the individual location search results in a div.', 'slp-premier' ) .
				    __( 'Useful for plugin styles that provide their own non-div wrappers such as tr.', 'slp-premier' )
		    ,
			    'option'       => 'results_no_wrapper',
			    'type'         => 'checkbox',
		    ) );

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'        => __( 'Results Header', 'slp-premier' ),
			    'description'  =>
				    __( 'If set, output the HTML + shortcodes before displaying results. ', 'slp-premier' ) .
				    __( 'If you wish to attach the locations to an HTML element other than the default #map_sidebar div, ', 'slp-premier' ) .
				    __( 'add id="add_locations_here" to one of your Results Header elements such as a <tbody> tag. ', 'slp-premier' )
		    ,
			    'option'       => 'results_header',
			    'type'         => 'textarea',
		    ) );
		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'        => __( 'Category Name Separator', 'slp-premier' ),
			    'description'  =>
				    __( 'The string to be used to separate Tagalong category names in the map results.  ', 'slp-premier' ) .
				    __( 'Utilized with Taglong and the [slp_location category_names] shortcode. ', 'slp-premier' ) .
				    __( 'Can include HTML such as <br>. ', 'slp-premier' )
		    ,
			    'option'       => 'category_name_separator',
		    ) );

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'        => __( 'Enable Pagination', 'slp-premier' ),
			    'description'  =>
				    __( 'When checked the pagination label and previous/next page button is shown on results.', 'slp-premier' ),
			    'option'       => 'pagination_enabled',
			    'type'         => 'checkbox',
		    ) );

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'        => __( 'Pagination Label', 'slp-premier' ),
			    'description'  => __( 'Enter a label to appear in front of the previous/next page icons in the results.', 'slp-premier' ),
			    'option'       => 'pagination_label',
		    ) );
	    }

	    /**
	     * Results / Labels
	     */
	    private function modify_results_labels() {
		    $this->group_params['group_slug'] = 'labels';
		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'       => __( 'Results Header 1', 'slp-premier' ),
			    'description' =>
				    __( 'Output a header on the results for plugin styles that support it. ', 'slp-premier' ) .
				    __( 'Use the settings [slp_option name = "results_header_1"] in the results layout. ', 'slp-premier' )
		    ,
			    'option'     => 'results_header_1',
		    ) );
		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'       => __( 'Results Header 2', 'slp-premier' ),
			    'description' =>
				    __( 'Output a header on the results for plugin styles that support it. ', 'slp-premier' ) .
				    __( 'Use the settings [slp_option name = "results_header_2"] in the results layout. ', 'slp-premier' )
		    ,
			    'option'     => 'results_header_2',
		    ) );
		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'       => __( 'Results Header 3', 'slp-premier' ),
			    'description' =>
				    __( 'Output a header on the results for plugin styles that support it. ', 'slp-premier' ) .
				    __( 'Use the settings [slp_option name = "results_header_3"] in the results layout. ', 'slp-premier' )
		    ,
			    'option'     => 'results_header_3',
		    ) );
		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'label'       => __( 'Results Header 4', 'slp-premier' ),
			    'description' =>
				    __( 'Output a header on the results for plugin styles that support it. ', 'slp-premier' ) .
				    __( 'Use the settings [slp_option name = "results_header_4"] in the results layout. ', 'slp-premier' )
		    ,
			    'option'     => 'results_header_4',
		    ) );
	    }

	    /**
	     * Search
	     */
	    private function modify_search() {
		    $this->group_params['section_slug'] = 'search';
		    $this->modify_search_appearance();
		    $this->modify_search_functionality();
		    $this->modify_search_labels();
	    }

	    /**
	     * Search / Appearance
	     */
	    private function modify_search_appearance() {
		    $this->group_params['group_slug'] = 'appearance';

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'option'      => 'dropdown_style',
			    'type'         => 'dropdown',
			    'label'        => __( 'Drop Down Style', 'slp-premier' ),
			    'description'  => __( 'Which style should be used for search form drop down menus? ', 'slp-premier' ),
			    'custom'       => array(
				    array( 'label' => __( 'None', 'slp-premier' ), 'value' => 'none'),
				    array( 'label' => __( 'Smoothness', 'slp-premier' ), 'value' => 'smoothness' ),
				    array( 'label' => __( 'Vader', 'slp-premier' ), 'value' => 'vader' ),
			    )
		    ) );
	    }

	    /**
	     * Search / Functionality
	     */
	    private function modify_search_functionality() {
		    $this->group_params['group_slug'] = 'functionality';

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'option'      => 'dropdown_autosubmit',
			    'type'         => 'checkbox',
			    'label'        => __( 'Dropdown Autosubmit', 'slp-premier' ),
			    'description'  => __( 'If checked, automatically submit searches when a dropdown changes.', 'slp-premier' )
		    ) );

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'type'         => 'subheader',
			    'label'        => __( 'Search Form Address Processing', 'slp-premier' ),
		    ) );

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'option'      => 'show_address_guess',
			    'type'         => 'checkbox',
			    'label'        => __( 'Show Address Guess', 'slp-premier' ),
			    'description'  => __( 'If checked, replace the address the user typed into the address box with what Google thought they meant.', 'slp-premier' )
		    ) );

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'option'      => 'region_influence_enabled',
			    'type'         => 'checkbox',
			    'label'        => __( 'Country Influences Guess', 'slp-premier' ),
			    'description'  => __( 'When checked us the selected Map Domain to influence how Google guesses what the user meant when entering a search address.', 'slp-premier' )
		    ) );

		    // Boundary Influence Guess
		    $this->Settings->add_ItemToGroup( array(
			    'group_params'  => $this->group_params,
			    'option'        => 'boundaries_influence_type',
			    'type'          => 'dropdown',
			    'label' => __( 'Boundaries Influence Guess', 'slp-premier' ),
			    'description' =>
				    __( 'Use a search boundary to influence how Google guesses what the user meant when entering a search address.', 'slp-premier' ) .
				    __( 'Locations is a rectangle that surrounds all of your locations.', 'slp-premier' ) .
				    __( 'Defined Boundary is a rectangle that you define with a map that will appear below.', 'slp-premier' ) .
				    __( 'The default is none, using standard Google address lookup rules.', 'slp-premier' ),
			    'custom' => array(
				    array( 'label' => __( 'None', 'slp-premier' ), 'value' => 'none' ),
				    array( 'label' => __( 'Locations', 'slp-premier' ), 'value' => 'locations' ),
				    array( 'label' => __( 'Defined Boundary', 'slp-premier' ), 'value' => 'boundary' ),
			        ),
			    'onChange' => 'SLPPREMIER_ADMIN.boundaries_map.set_map_visibility_based_on_dropdown();'
		    ) );

		    // The details and map of the boundary influence.
		    $this->Settings->add_ItemToGroup( array(
		    	'group_params'  => $this->group_params,
			    'description' => $this->create_string_boundaries_influence(),
			    'type'        => 'details',
		    ) );
	    }

	    /**
	     * Search / Labels
	     */
	    private function modify_search_labels() {
		    $this->group_params['group_slug'] = 'labels';

		    $this->Settings->add_ItemToGroup( array(
			    'group_params' => $this->group_params,
			    'option'      => 'search_box_subtitle',
			    'label'        => __( 'Search Box Subtitle', 'slp-premier' ),
			    'description'  => __( 'The label that goes in the search form box header, for plugin themes that support it. ', 'slp-premier' ) .
			                      __( 'Put this in a search form using the [slp_option name="search_box_subtitle"] shortcode. ', 'slp-premier' ),
		    ) );
	    }

        /**
         * Create the boundaries influence description.
         */
        private function create_map_center_div() {
            return
                '<div class="input-group">'.
                '<label for="center_map">' . __( 'Map Center Fallback', 'slp-premier' ) . '</label>' .
                '<div id="center_map" name="center_map"></div>'.
	            '</div>'
	            ;
        }

        /**
         * Create the boundaries influence description.
         */
        private function create_string_boundaries_influence() {
            $ne_coordinates = sprintf(
                '<span id="ne_boundary">(%s,%s)</span>',
                $this->addon->options[ 'boundaries_influence_max_lat' ],
                $this->addon->options[ 'boundaries_influence_max_lng' ]
            );
            $sw_coordinates = sprintf(
                '<span id="sw_boundary">(%s,%s)</span>',
                $this->addon->options[ 'boundaries_influence_min_lat' ],
                $this->addon->options[ 'boundaries_influence_min_lng' ]

            );

            return
                "<div id='boundaries_map_wrapper' style='width:100%; height: 100%; display: block;'>" .

                "<input type='hidden' id='{$this->addon->option_name}[boundaries_influence_min_lat]' name='{$this->addon->option_name}[boundaries_influence_min_lat]' value='{$this->addon->options['boundaries_influence_min_lat']}' /> " .
                "<input type='hidden' id='{$this->addon->option_name}[boundaries_influence_max_lat]' name='{$this->addon->option_name}[boundaries_influence_max_lat]' value='{$this->addon->options['boundaries_influence_max_lat']}' /> " .
                "<input type='hidden' id='{$this->addon->option_name}[boundaries_influence_min_lng]' name='{$this->addon->option_name}[boundaries_influence_min_lng]' value='{$this->addon->options['boundaries_influence_min_lng']}' /> " .
                "<input type='hidden' id='{$this->addon->option_name}[boundaries_influence_max_lng]' name='{$this->addon->option_name}[boundaries_influence_max_lng]' value='{$this->addon->options['boundaries_influence_max_lng']}' /> " .

                "<span id='current_boundaries'>" .
                sprintf(
                    __( 'Boundaries are currently set to %s southwest and %s northeast.', 'slp-premier' ),
                    $sw_coordinates,
                    $ne_coordinates
                ) .
                '</span>' .

                '<div id="boundaries_map"></div>' .
                '</div>';
        }

        /**
         * Extend Radius Behavior Selections
         * @param string $description
         * @return array
         */
        public function extend_radius_behavior_description( $description ) {
            if ( ! $this->slplus->is_CheckTrue( $this->addon->options['use_territory_bounds'] ) ) {
                return $description;
            }

            $description .= __( '<strong>Always Use, address entered must be in territory</strong> - location must be within radius AND address entered but be in the location territory.<br/>' , 'slp-premier' );

            $description .= __( '<strong>Use for locations without a territory, Do not use for territories</strong> - show locations within radius if they have no territory as well as locations with a territory that covers the entered address.'  , 'slp-premier' );

            return $description;
        }

        /**
         * Extend Radius Behavior Selections
         * @param array $selections
         * @return array
         */
        public function extend_radius_behavior_selections( $selections ) {
            if ( ! $this->slplus->is_CheckTrue( $this->addon->options['use_territory_bounds'] ) ) {
                return $selections;
            }
            $selections[] = array( 'label' => __( 'Always Use, address entered must be in territory' , 'slp-premier' ), 'value' => 'in_radius_and_in_territory' );
            $selections[] = array( 'label' => __( 'Use for locations without a territory, Do not use for territories'  , 'slp-premier' ), 'value' => 'in_radius_no_terr_set_or_in_territory' );
            return $selections;
        }
    }
}