<?php

if ( ! class_exists( 'SLPPremier_Admin_Experience_Text' ) ):

/**
 * SLP Text Modifier
 *
 * Used by this MySLP Dashboard Base Class on as well as the REST API handler.
 *
 * @var array    text    array of our text modifications key => SLP text manager slug, value = our replacement text
 *
 */
class SLPPremier_Admin_Experience_Text extends SLPlus_BaseClass_Object {
	private $text;

	/**
	 * SLP Text Manager Hooks
	 */
	private function add_hooks() {
		add_filter( 'slp_get_text_string'    , array( $this , 'augment_text_string' ) , 10 , 2 );
	}

	/**
	 * Replace the SLP Text Manager Strings at startup.
	 *
	 * @param string $text  the original text
	 * @param string $slug  the slug being requested
	 *
	 * @return array            the new SLP text manager strings
	 */
	public function augment_text_string( $text , $slug ) {
		$this->init_text();
		if ( ! is_array( $slug ) ) {
			if ( isset( $this->text[ $slug ] ) ) {
				return $this->text[ $slug ];
			}
		} else {
			if ( isset( $this->text[ $slug[0] ] ) && isset( $this->text[ $slug[0] ][ $slug[1] ] ) ){
				return $this->text[ $slug[0] ][ $slug[1] ];
			}
		}
		return $text;
	}

	/**
	 * Initialize our text modification array.
	 */
	private function init_text() {
		if ( isset( $this->text ) ) {
			return;
		}
		$this->text['settings_group']['results_interaction'] = __('Results Interaction' , 'slp-premier' );

		$this->text['label'      ]['map_marker_tooltip'] = __( 'Marker Tooltip', 'slp-premier' );
		$this->text['description']['map_marker_tooltip'] = __( 'Shows a tool tip of the location name when a user hovers over a map marker.', 'slp-premier' );

		$this->text['label'      ]['results_click_animate_marker'] = __('Animate Marker' , 'slp-premier' );
		$this->text['description']['results_click_animate_marker'] = __('How to animate the map marker when a user clicks the corresponding result.' , 'slp-premier' );

		$this->text['label'      ]['results_click_label_marker'] = __('Label Marker' , 'slp-premier' );
		$this->text['description']['results_click_label_marker'] = __('How to label the map marker when a user clicks the corresponding result.' , 'slp-premier' );

		$this->text['label'      ]['results_click_map_movement'] = __('Map Movement' , 'slp-premier' );
		$this->text['description']['results_click_map_movement'] = __('How should the map behave when a location is clicked?' , 'slp-premier' );

		$this->text['label'      ]['results_click_marker_icon_behavior'] = __('Marker Icon Behavior' , 'slp-premier' );
		$this->text['description']['results_click_marker_icon_behavior'] = __('How should the marker icon change when a result is clicked?' , 'slp-premier' );

		$this->text['label'      ]['results_click_marker_icon'] = __('Active Marker Icon' , 'slp-premier' );
		$this->text['description']['results_click_marker_icon'] = __('What to change the marker to when clicked.' , 'slp-premier' );

	}

	/**
	 * Things we do at the start.
	 */
	public function initialize() {
		$this->add_hooks();
	}
}
endif;