<?php
if ( ! class_exists( 'SLPPremier_Admin_General' ) ) {

	/**
     * The things that modify the Admin / General Tab.
     *
     * @package StoreLocatorPlus\SLPPremier\Admin\General
     * @author Lance Cleveland <lance@storelocatorplus.com>
     * @copyright 2016 Charleston Software Associates, LLC
	 *
     * Text Domain: slp-premier
     *
	 * @property    SLPPremier                      $addon
	 * @property    SLPPremier_Admin                $admin
	 * @property    SLPPremier_Admin_General_Text   $Admin_General_Text
	 */
	class SLPPremier_Admin_General extends SLP_Object_With_Objects {
		public $addon;
		public $admin;
		private $group_params;

		protected $class_prefix = 'SLPPremier_';
		protected $objects = array(
			'Admin_General_Text' => array( 'subdir' => 'include/module/admin_tabs/', 'object' => null, 'auto_instantiate' => true,  ),
		);

		/**
		 * Things we do at the start.
		 */
		function initialize() {
			parent::initialize();
			$this->group_params = array( 'plugin' => $this->addon , 'section_slug' => null , 'group_slug' => null );

            // WooCommerce
            //
            if ( $this->addon->is_woo_running() ) {
                add_action( 'slp_generalsettings_modify_userpanel'  , array( $this , 'add_woo_group' ), 10, 3 );
            }

            add_action( 'slp_build_general_settings_panels'     , array( $this , 'add_data_subtab'               ), 90    );
			add_action( 'slp_build_general_settings_panels'     , array( $this , 'add_schedule_subtab'           ), 90    );
		}

        /**                        
         * If Territory Bounds settings is enabled post-activation, add the fields.
         */
        public function activate_territory_bounds( ) {
            if ( $this->admin->option_changed( 'use_territory_bounds' , true ) ){
                require_once( $this->addon->dir . 'include/class.activation.php');
                $activation = new SLPPremier_Activation( array( 'addon' => $this->addon ) );
                $activation->add_territory_bounds();
                $this->slplus->notifications->add_notice( 'info' , __( 'Territory Bounds have been activated.' , 'slp-premier' ) );
            }            
        }

        /**
         * General / Data
         *
         * @param   SLP_Settings    $settings
         */
        public function add_data_subtab( $settings ) {
            $group_params = array(
                'plugin'       => $this->addon,
                'group_slug'   => 'packaged_data_extensions',
                'section_slug' => 'data',
            );

            $settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'type'          => 'checkbox'   ,
                'option'        => 'use_territory_bounds',
                'label'         => __( 'Enable Territory Bounds' , 'slp-premier' ),
                'description'   => __( 'Add fields to describe coverage areas for locations. ' , 'slp-premier' )
            ));
        }


		/**
		 * General / Schedule
		 *
		 * @param   SLP_Settings    $settings
		 */
		public function add_schedule_subtab( $settings ) {
			$this->group_params[ 'section_slug' ] = 'schedule';
			$this->group_params[ 'group_slug'   ] = 'tasks';
			$settings->add_group( $this->group_params );


		}

        /**
         * Add General / UI
         *
         * @param SLP_Settings  $settings
         * @param string        $section_name
         * @param array         $section_params
         */
        function add_woo_group( $settings , $section_name , $section_params ) {
            $group_params['group_slug']     = 'general_woocommerce';
            $group_params['section_slug']   = $section_params['slug'];
            $group_params['plugin']         = $this->addon;
            $group_params['header']         = __( 'WooCommerce' , 'slp-premier' );
            $group_params['intro']          = __( 'These settings impact how Store Locator Plus interacts with Woocommerce. ' , 'slp-premier' );

            $settings->add_group( $group_params );

            $settings->add_ItemToGroup(array(
                'group_params'  => $group_params,
                'type'          => 'checkbox'  ,
                'option'        => 'show_location_on_order_email',
                'label'         => __('Order Email Location Info' , 'slp-premier' ),
                'description'   => __( 'Shows location address, phone, and email on order email to customer.' , 'slp-premier' )
            ));
        }

		/**
		 * Set default options for objects.  Override in your class.
		 */
		protected function set_default_object_options() {
			$this->objects[ 'Admin_General_Text' ][ 'options' ] = array( 'addon' => $this->addon );
		}

	}
}