<?php

if ( ! class_exists( 'SLPPremier_Admin_General_Text' ) ):

/**
 * SLP Text Modifier
 *
 * @var array    text    array of our text modifications key => SLP text manager slug, value = our replacement text
 *
 * @property    SLPPremier $addon
 */
class SLPPremier_Admin_General_Text extends SLPlus_BaseClass_Object {
	public $addon;
	private $text;

	/**
	 * SLP Text Manager Hooks
	 */
	private function add_hooks() {
		add_filter( 'slp_get_text_string'    , array( $this , 'augment_text_string' ) , 10 , 2 );
	}

	/**
	 * Replace the SLP Text Manager Strings at startup.
	 *
	 * @param string $text  the original text
	 * @param string $slug  the slug being requested
	 *
	 * @return array            the new SLP text manager strings
	 */
	public function augment_text_string( $text , $slug ) {
		$this->init_text();
		if ( ! is_array( $slug ) ) {
			if ( isset( $this->text[ $slug ] ) ) {
				return $this->text[ $slug ];
			}
		} else {
			if ( isset( $this->text[ $slug[0] ] ) && isset( $this->text[ $slug[0] ][ $slug[1] ] ) ){
				return $this->text[ $slug[0] ][ $slug[1] ];
			}
		}
		return $text;
	}

	/**
	 * Initialize our text modification array.
	 */
	private function init_text() {
		if ( isset( $this->text ) ) {
			return;
		}

		$this->addon->instantiate( 'Schedule_Manager' );

		$this->text['settings_section']['schedule' ] = __( 'Schedule', 'slp-premier' );

		$this->text['settings_group'  ]['tasks'    ] = __( 'Tasks'   , 'slp-premier' );
		$this->text['settings_group_header' ] = $this->text[ 'settings_group' ];

		$this->text['label'      ]['schedule_for_geocoding'] = __('Geocoding' , 'slp-premier' );
		$this->text['base_desc']['schedule_for_geocoding'] =
			__('When should uncoded locations be automatically geocoded? ' , 'slp-premier' ) .
		    __('Will run on the hour and repeat every hour, 12 hour or 24 hours. ', 'slp-premier' );
		$this->set_schedule_description( 'schedule_for_geocoding' );

		$this->text['label'      ]['schedule_for_initial_distance'] = __('Initial Distance' , 'slp-premier' );
		$this->text['base_desc']['schedule_for_initial_distance'] =
			__('When should locations have their initial distance automatically calculated? ' , 'slp-premier' ) .
			__('Will run on the hour and repeat every hour, 12 hour or 24 hours. ', 'slp-premier' );
		$this->set_schedule_description( 'schedule_for_initial_distance' );

	}

	/**
	 * Set the schedule info for the help text on scheduled options.
	 *
	 * @param string $slug
	 */
	public function set_schedule_description( $slug ) {
		$extended_text =
			empty( $this->addon->Schedule_Manager->$slug->next_event ) ?
				__( 'This event is not scheduled.' , 'slp-premier' ) :
				sprintf (
					__('Next Event happens in %s on %s' , 'slp-premier') ,
					$this->addon->Schedule_Manager->$slug->next_event_time_to_text,
					$this->addon->Schedule_Manager->$slug->next_event_text
				);
		$this->text['description'][ $slug ] = $this->text[ 'base_desc' ][ $slug ] . '<span class="scheduled_time">' .  $extended_text . '</span>';
	}

	/**
	 * Things we do at the start.
	 */
	public function initialize() {
		$this->add_hooks();
	}
}
endif;