<?php

if ( ! class_exists( 'SLPPremier_Options' ) ):

	/**
	 * Class SLPPremier_Options
	 *
	 * The options management for the premier add on
	 *
	 * @package   StoreLocatorPlus\Premier\Options
	 * @author    Lance Cleveland <lance@charlestonsw.com>
	 * @copyright 2016 Charleston Software Associates, LLC
	 *
	 * @since     4.6.4
	 *
	 * @property SLPPremier     $addon
	 * @property SLP_Option     $admin_notice_dismissed
	 *
	 * @property-read   boolean $initialized
	 *
	 */
	class SLPPremier_Options extends SLPlus_BaseClass_Object {
		public  $addon;
		public  $admin_notice_dismissed;
		private $initialized;
		private $our_options=array();

		/**
		 * Things we do at the start.
		 */
		public function initialize() {
			if ( $this->initialized ) {
				return;
			}

			$this->create_experience_options();
			$this->create_general_options();

			$this->initialized = true;
		}

		/**
		 * Experience
		 */
		private function create_experience_options() {
			$this->create_experience_map_options();
			$this->create_experience_results_options();
		}

		/**
		 * Experience / Map
		 */
		private function create_experience_map_options() {
			// Appearance
			//
			$smart_options['map_options_clickableIcons'] = array(
				'page'     => 'slp_experience',
				'section'           => 'map',
				'group'             => 'appearance',
				'label'             => __( 'Clickable Icons', 'slp-premier' ),
				'description'       => __( "When checked makes Google's default map icons clickable.", 'slp-premier' ),
				'related_to'        => 'map_option_hide_streetview',
				'type'              => 'checkbox',
				'default'           => '1',
				'use_in_javascript' => true,
			);


			// Markers
			//
			$smart_options['clusters_enabled'] = array(
				'page'     => 'slp_experience',
				'section'           => 'map',
				'group'             => 'markers',
				'label'             => __( 'Enable Clusters', 'slp-premier' ),
				'description'       => __( 'When checked employ map clusters when markers are within close proximity.', 'slp-premier' ),
				'related_to'        => 'cluster_gridsize,cluster_minimum',
				'type'              => 'checkbox',
				'default'           => '1',
				'use_in_javascript' => true,
			);
			$smart_options['cluster_gridsize'] = array(
				'page'     => 'slp_experience',
				'section'           => 'map',
				'group'             => 'markers',
				'label'             => __( 'Cluster Radius', 'slp-premier' ),
				'description'       =>
					__( 'How close to markers need to be to cluster together. ', 'slp-premier' ) .
					__( 'The number is "tile spaces" which changes based on the map zoom level. ', 'slp-premier' )
			,
				'related_to'        => 'clusters_enabled,cluster_minimum',
				'type'              => 'dropdown',
				'default'           => '60',
				'use_in_javascript' => true,
				'options'       => array(
					array( 'label' => __( '90'          , 'slp-premier' ), 'value' => '90' , 'description' => __( 'Larger areas cluster together; zoom in more to break clusters.' , 'slp-premier' ) ) ,
					array( 'label' => __( '60 (Default)', 'slp-premier' ), 'value' => '60' , 'description' => __( 'The default value.' , 'slp-premier' ) ),
					array( 'label' => __( '30'          , 'slp-premier' ), 'value' => '30' , 'description' => __( 'Smaller areas cluster together; zoom in less to break clusters. ' , 'slp-premier' ) ) ,
				),
			);
			$smart_options['cluster_minimum'] = array(
				'page'     => 'slp_experience',
				'section'           => 'map',
				'group'             => 'markers',
				'label'             => __( 'Cluster Minimum', 'slp-premier' ),
				'description'       => __( 'The minimum number of markers required to make a cluster.', 'slp-premier' ),
				'related_to'        => 'clusters_enabled,cluster_gridsize',
				'default'           => '3',
				'use_in_javascript' => true,
			);
			$smart_options['map_marker_tooltip'] = array(
				'page'     => 'slp_experience',
				'section'           => 'map',
				'group'             => 'markers',
				'type'              => 'checkbox',
				'default'           => '1',
				'use_in_javascript' => true,
			);
			$this->our_options = array_merge( $this->our_options , array_keys( $smart_options ) );
			$this->slplus->smart_options->create_smart_options( $smart_options, $this->slplus->options, $this->slplus->options_nojs );
		}

		/**
		 * Experience / Results
		 */
		private function create_experience_results_options() {

			// Results Interaction
			//
			$smart_options[ 'results_click_animate_marker' ] = array(
				'page'    => 'slp_experience',
				'section' => 'results',
				'group'   => 'results_interaction',
				'use_in_javascript' => true,
				'type'    => 'dropdown' ,
				'default' => 'none',
				'options'       => array(
					array( 'label' => __( 'None'  , 'slp-premier' ), 'value' => 'none'   , 'description' => __( 'No animation for the marker.' , 'slp-premier' ) ),
					array( 'label' => __( 'Bounce', 'slp-premier' ), 'value' => 'BOUNCE' , 'description' => __( 'Continually bounce the marker. ' , 'slp-premier' ) ) ,
					array( 'label' => __( 'Drop'  , 'slp-premier' ), 'value' => 'DROP'   , 'description' => __( 'Drop the marker from the "sky" with a single bounce.' , 'slp-premier' ) ),
				),
			);
			$smart_options[ 'results_click_label_marker' ] = array(
				'page'    => 'slp_experience',
				'section' => 'results',
				'group'   => 'results_interaction',
				'use_in_javascript' => true,
				'type'    => 'dropdown' ,
				'default' => 'no_label',
				'options'       => array(
					array( 'label' => __( 'No Label'  , 'slp-premier' ), 'value' => 'no_label'  , 'description' => __( 'Do not add a label.' , 'slp-premier' ) ),
					array( 'label' => __( 'Name'      , 'slp-premier' ), 'value' => 'name'      , 'description' => __( 'Show the store name. ' , 'slp-premier' ) ) ,
				),
			);
			$smart_options[ 'results_click_map_movement' ] = array(
				'page'    => 'slp_experience',
				'section' => 'results',
				'group'   => 'results_interaction',
				'use_in_javascript' => true,
				'type'    => 'dropdown' ,
				'default' => 'stationary',
				'options'       => array(
					array( 'label' => __( 'Stationary'      , 'slp-premier' ), 'value' => 'stationary'  , 'description' => __( 'Do not move.' , 'slp-premier' ) ),
					array( 'label' => __( 'Center Location' , 'slp-premier' ), 'value' => 'center'      , 'description' => __( 'Center the location on the map. ' , 'slp-premier' ) ) ,
				),
			);
			$smart_options[ 'results_click_marker_icon_behavior' ] = array(
				'page'    => 'slp_experience',
				'section' => 'results',
				'group'   => 'results_interaction',
				'related_to' => 'results_click_marker_icon',
				'use_in_javascript' => true,
				'type'    => 'dropdown' ,
				'default' => 'as_is',
				'options'       => array(
					array( 'label' => __( 'Keep As Is'             , 'slp-premier' ), 'value' => 'as_is'       , 'description' => __( 'Do not change the marker.' , 'slp-premier' ) ),
					array( 'label' => __( 'Use Active Marker Icon' , 'slp-premier' ), 'value' => 'use_active'  , 'description' => __( 'Use the active marker icon specified here. ' , 'slp-premier' ) ) ,
				),
			);
			$smart_options[ 'results_click_marker_icon' ] = array(
				'page'    => 'slp_experience',
				'section' => 'results',
				'group'   => 'results_interaction',
				'related_to' => 'results_click_marker_icon_behavior',
				'use_in_javascript' => true,
				'type'    => 'icon' ,
			);

			$this->our_options = array_merge( $this->our_options , array_keys( $smart_options ) );
			$this->slplus->smart_options->create_smart_options( $smart_options, $this->slplus->options, $this->slplus->options_nojs );

		}

		/**
		 * General
		 */
		private function create_general_options() {
			$this->create_general_schedule_options();
		}

		/**
		 * General / Schedule
		 */
		private function create_general_schedule_options() {
			// Tasks
			$smart_options[ 'schedule_for_geocoding' ] = array(
				'page'    => 'slp_general',
				'section' => 'schedule',
				'group'   => 'tasks',
				'type'    => 'dropdown' ,
				'default' => 'never',
				'options'       => array(
					array( 'label' => __( 'Never'        , 'slp-premier' ), 'value' => 'never'       , 'description' => __( 'Not scheduled.' , 'slp-premier' ) ),
					array( 'label' => __( 'Now'          , 'slp-premier' ), 'value' => 'now'         , 'description' => __( 'Run just once.' , 'slp-premier' ) ),
					array( 'label' => __( 'Hourly'       , 'slp-premier' ), 'value' => 'hourly'      , 'description' => __( 'Run every hour.' , 'slp-premier' ) ),
					array( 'label' => __( 'Twice Daily'  , 'slp-premier' ), 'value' => 'twicedaily'  , 'description' => __( 'Run twice per day.' , 'slp-premier' ) ),
					array( 'label' => __( 'Daily'        , 'slp-premier' ), 'value' => 'daily'       , 'description' => __( 'Run once per day.' , 'slp-premier' ) ),
				),
				'call_when_changed' => array( $this , 'schedule_changes' ),
				'call_when_time'    => array( $this , 'register_cron_hooks' ),
			);
			$smart_options[ 'schedule_for_initial_distance' ] = array(
				'page'    => 'slp_general',
				'section' => 'schedule',
				'group'   => 'tasks',
				'type'    => 'dropdown' ,
				'default' => 'never',
				'options'       => array(
					array( 'label' => __( 'Never'        , 'slp-premier' ), 'value' => 'never'       , 'description' => __( 'Not scheduled.' , 'slp-premier' ) ),
					array( 'label' => __( 'Now'          , 'slp-premier' ), 'value' => 'now'         , 'description' => __( 'Run just once.' , 'slp-premier' ) ),
					array( 'label' => __( 'Hourly'       , 'slp-premier' ), 'value' => 'hourly'      , 'description' => __( 'Run every hour.' , 'slp-premier' ) ),
					array( 'label' => __( 'Twice Daily'  , 'slp-premier' ), 'value' => 'twicedaily'  , 'description' => __( 'Run twice per day.' , 'slp-premier' ) ),
					array( 'label' => __( 'Daily'        , 'slp-premier' ), 'value' => 'daily'       , 'description' => __( 'Run once per day.' , 'slp-premier' ) ),
				),
				'call_when_changed' => array( $this , 'schedule_changes' ),
				'call_when_time'    => array( $this , 'register_cron_hooks' ),
			);


			$this->our_options = array_merge( $this->our_options , array_keys( $smart_options ) );
			$this->slplus->smart_options->create_smart_options( $smart_options, $this->slplus->options, $this->slplus->options_nojs );
		}

		/**
		 * Register cron hooks.
		 *
		 * @param string    $slug  the property "name"
		 */
		public function register_cron_hooks( $slug ) {
			if ( ! in_array( $slug , $this->our_options ) ) {
				return;
			}
			$this->addon->instantiate( 'Schedule_Manager' );
			$this->addon->Schedule_Manager->create_hook( $slug );
		}

		/**
		 * Process schedule changes.
		 *
		 * @param string    $slug  the property "name"
		 * @param variable  $old_value
		 * @param variable  $new_value
		 */
		public function schedule_changes( $slug , $old_value , $new_value ) {
			switch ( $this->slplus->smart_options->$slug->page ) {
				case 'slp_general':
					$addon = $this->slplus->add_ons->instances[ 'slp-premier' ];
					$addon->instantiate( 'Schedule_Manager' );
					$addon->Schedule_Manager->put_on_schedule( $slug , $new_value );
					break;
			}
		}
	}

endif;