<?php
// Exit if access directly, dang hackers
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'SLPPremier_Tagalong' ) ) :
/**
 * Tagalong interface stuff that spans Admin, UI, and possibly AJAX.
 *
 * @property        SLPPremier $addon
 * @property        SLPPremier_Tagalong_UI $ui
 *
 * @package StoreLocatorPlus\SLPPremier\Tagalong
 * @author Lance Cleveland <lance@storelocatorplus.com>
 * @copyright 2015 - 2106 Charleston Software Associates, LLC
 */
class SLPPremier_Tagalong extends SLPlus_BaseClass_Object {
	public $addon;
	private $category_class;
	private $category_instance;
	public $ui;

	/**
	 * Start this up.
	 */
	public function initialize() {
		parent::initialize();
		$this->set_tagalong_filters();
	}

	/**
	 * Create and attach the Tagalong admin object.
	 */
	public function create_object_ui_tagalong() {
		$this->set_category_object_meta();
		if ( ! isset( $this->category_instance ) ) {
			return;
		}
		if ( ! isset( $this->ui ) ) {
			require_once( 'SLPPremier_Tagalong_UI.php' );
			$this->ui = new SLPPremier_Tagalong_UI( array( 'addon' => $this->addon , 'category_instance' => $this->category_instance ) );
		}
	}

	/**
	 * Set the filters that modify Tagalong.
	 */
	private function set_tagalong_filters() {
		add_filter( 'slp_tagalong_default_selectors', array( $this, 'add_single_parent_selector' ), 20, 1 );
	}

	/**
	 * @param SLPTagalong_Category_Selector[] $selectors
	 *
	 * @return SLPTagalong_Category_Selector[]
	 */
	function add_single_parent_selector( $selectors ) {
		$this->set_category_object_meta();
		if ( ! isset( $this->category_instance ) ) {
			return $selectors;
		}

		// Single Parent
		$selectors['single_parents'] =
			new $this->category_class( array(
				'label'    => __( 'Single Parents', 'slp-premier' ),
				'value'    => 'single_parents',
				'selected' => ( $this->category_instance->options['show_cats_on_search'] === 'single_parents' )
			) );

		return $selectors;
	}

	/**
	 * Replace the default Tagalong category separator with the setting specified by the user.
	 *
	 * @param   string $separator
	 *
	 * @return  string the user-specified separator
	 */
	function modify_category_name_separator( $separator ) {
		return $this->addon->options['category_name_separator'];
	}

	/**
	 * Set category class name and instance pointer.
	 */
	private function set_category_object_meta() {
		if ( $this->slplus->is_AddonActive( 'slp-tagalong' ) ) {
			$this->category_class = 'SLPTagalong_Category_Selector';
			$this->category_instance = $this->slplus->add_ons->instances['slp-tagalong'];
		} else if ( $this->slplus->is_AddonActive( 'slp-power' ) ) {
			$this->category_class = 'SLPPower_Category_Selector';
			$this->category_instance = $this->slplus->add_ons->instances['slp-power'];
		}
	}

}
endif;