<?php

/**
 * Add stuff to the Tagalong UI.
 *
 * @property        SLPPremier      $addon
 * @property        SLPTagalong|SLPPower_Category_Manager     $category_instance
 *
 * @package StoreLocatorPlus\SLPPremier\Tagalong\UI
 * @author Lance Cleveland <lance@storelocatorplus.com>
 * @copyright 2015-2016 Charleston Software Associates, LLC
 */
class SLPPremier_Tagalong_UI extends SLPlus_BaseClass_Object {
    public $addon;
    public $category_instance;

    /**
     * Things we do at the start.
     */
    public function initialize( ) {
        add_action( 'slp_tagalong_get_selector_html' , array( $this , 'add_ui_builders_to_selectors' ) );
    }

    /**
     * Add the UI builders to the Tagalong selectors.
     */
    function add_ui_builders_to_selectors() {
        $this->category_instance->category_selector_manager->selectors['single_parents']->ui_builder =
            array( $this , 'create_string_for_single_parent_selector' );
    }

    /**
     * Create the single parent selector.
     *
     * @return string
     */
    function create_string_for_single_parent_selector() {
        $term_ids = $this->category_instance->stores_taxonomy->get_parent_categories();

        $html = '';
        foreach ( $term_ids as $term_id ) {
            $category_html =
                wp_dropdown_categories(
                    array(
                        'child_of' => $term_id,
                        'echo' => 0,
                        'hierarchical' => 1,
                        'depth' => 99,
                        'hide_empty' => ($this->slplus->is_CheckTrue($this->category_instance->options['hide_empty']) ? 1 : 0),
                        'hide_if_empty' => true,
                        'orderby' => 'NAME',
                        'show_option_all' =>
                            $this->slplus->WPML->get_text(
                                'TAGALONG-show_option_all',
                                $this->category_instance->options['show_option_all'],
                                'slp-premier'
                            ) .
                            ' ' .
                            $this->category_instance->stores_taxonomy->get_term( $term_id , 'name' )
                            ,
                        'name'     => 'cat[]',
                        'taxonomy' => SLPLUS::locationTaxonomy
                    )
                );
            $html .= $this->category_instance->category_selector_manager->create_string_dropdown_div( $category_html , $term_id );
        }

        return $html;
    }
}