<?php
/**
 * Plugin Name: Store Locator Plus - Premier
 * Plugin URI: https://www.storelocatorplus.com/product/premier-subscription/
 * Description: Add Premier Subscription features to the Store Locator Plus plugin.
 * Author: Store Locator Plus
 * Author URI: http://storelocatorplus.com/
 * Requires at least: 3.8
 * Tested up to : 4.7
 * Version: 4.7.1-beta-4
 *
 * Text Domain: slp-premier
 * Domain Path: /languages/
 */

// We do not do any heartbeat processing
if ( defined( 'DOING_AJAX' ) && DOING_AJAX && ! empty( $_POST[ 'action' ] ) && ( $_POST['action'] === 'heartbeat' ) ) {
	return;
}

// Exit if access directly, dang hackers
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

// Since this is an add-on for Store Locator Plus
// we only want to "get started" after all plugins are loaded.
//
// This allows us to check our dependencies knowing SLP
// should be loaded into the PHP stack by now.
//
// Concept Credits: @pippinsplugins, @ipstenu
// @see https://pippinsplugins.com/checking-dependent-plugin-active/
// @see https://make.wordpress.org/plugins/2015/06/05/policy-on-php-versions/
//
function SLPPremier_loader() {
    $this_plugin_name = __( 'Store Locator Plus - Premier', 'slp-premier' );
    $min_wp_version   = '4.4';

    // Requires SLP
    //
    if ( ! defined( 'SLPLUS_PLUGINDIR' ) ) {
        add_action(
            'admin_notices',
            create_function(
                '',
                "echo '<div class=\"error\"><p>" .
                sprintf(
                    __( '%s requires Store Locator Plus to function properly. ', 'slp-premier' ),
                    $this_plugin_name
                ) . '<br/>' .
                __( 'This plugin has been deactivated.', 'slp-premier' ) .
                __( 'Please install Store Locator Plus.', 'slp-premier' ) .
                "</p></div>';"
            )
        );
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        deactivate_plugins( plugin_basename( __FILE__ ) );

        return;
    }

    // Check minimum WP version
    //
    global $wp_version;
    if ( version_compare( $wp_version, $min_wp_version, '<' ) ) {
        add_action(
            'admin_notices',
            create_function(
                '',
                "echo '<div class=\"error\"><p>" .
                sprintf(
                    __( '%s requires WordPress %s to function properly. ', 'slp-premier' ),
                    $this_plugin_name,
                    $min_wp_version
                ) .
                __( 'This plugin has been deactivated.', 'slp-premier' ) .
                __( 'Please upgrade WordPress.', 'slp-premier' ) .
                "</p></div>';"
            )
        );
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        deactivate_plugins( plugin_basename( __FILE__ ) );

        return;
    }

    // Define some path constants in an attempt to bypass depth-driven confusion.
    //
    if ( ! defined( 'SLPPREMIER_REL_DIR' ) ) {
        define( 'SLPPREMIER_REL_DIR', plugin_basename( dirname( __FILE__ ) ) );
    }  // Relative directory for this plugin in relation to wp-content/plugins
    if ( ! defined( 'SLPPREMIER_FILE' ) ) {
        define( 'SLPPREMIER_FILE', __FILE__ );
    }  // FQ File name for this file.

    // Go forth and sprout your tentacles...
    // Get some Store Locator Plus sauce.
    //
    require_once( 'include/class.slp-premier.php' );
    SLPPremier::init();
}

add_action( 'plugins_loaded', 'SLPPremier_loader' );

